/***************************************************************************
                               calias.cpp
                             -------------------
    begin                : Thu Jul 25 2002
    copyright            : (C) 2002 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qstring.h>
#include <qstringlist.h>
#include <qdom.h>
#include <qregexp.h>

// local includes
#include "calias.h"

///////////////////////////////////////////////////////////////////////////////

CAlias::CAlias() : CAliasItem()
{
	setCommand(QString::null);
}

///////////////////////////////////////////////////////////////////////////////

CAlias::CAlias(const QString& name, const QString& alias, const QString& command)
	: CAliasItem(name), m_alias(alias)
{
	setCommand(command);
}

///////////////////////////////////////////////////////////////////////////////

CAlias::~CAlias()
{
}

///////////////////////////////////////////////////////////////////////////////

void CAlias::setCommand(const QString& command)
{
	m_command = command;
	m_argMax = -1;
	for (uint i = 0; i < MAX_ARGS; ++i)
	{
		m_args[i] = command.find(QString("$%1").arg(i+1)) >= 0;
		if (m_args[i])
			m_argMax = i;
	}
}

///////////////////////////////////////////////////////////////////////////////

QString CAlias::replace(const QStringList& input)
{
	if (!m_enabled)
		return QString::null;

	if (input.count() == 0)
		return QString::null;

	if (input[0] != m_alias)
		return QString::null;

	QString result = m_command;

	for (int i = 0; i <= m_argMax; ++i)
	{
		if (m_args[i])
		{
			QString param;
			int argNum = i + 1;
			if (argNum < (int)input.count())
				param = input[argNum];
			else
				param = "";
			result.replace(QRegExp("$"+QString::number(argNum)), param);
		}
	}

	for (int i = m_argMax+2; i < (int)input.count(); ++i)
	{
		result.append(" ");
		result.append(input[i]);
	}

	return result;
}

///////////////////////////////////////////////////////////////////////////////

bool CAlias::readFromDOM(const QDomElement& dom)
{
	if (dom.isNull())
		return false;

	if (!CProfileItem::readFromDOM(dom))
		return false;

	QDomNode node = dom.namedItem("alias");
	if (node.isNull() || !node.isElement())
	{
		m_alias = QString::null;
	}
	else
	{
		QDomElement tag = node.toElement();
		m_alias = tag.text();
	}

	node = dom.namedItem("command");
	if (node.isNull() || !node.isElement())
	{
		setCommand(QString::null);
	}
	else
	{
		QDomElement tag = node.toElement();
		setCommand(tag.text());
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////

bool CAlias::writeToDOM(QDomElement& dom)
{
	if (dom.isNull())
		return false;

	if (!CProfileItem::writeToDOM(dom))
		return false;

	QDomElement tag = dom.ownerDocument().createElement("alias");
	QDomNode text = dom.ownerDocument().createTextNode(m_alias);

	tag.appendChild(text);
	dom.appendChild(tag);

	tag = dom.ownerDocument().createElement("command");
	text = dom.ownerDocument().createTextNode(m_command);

	tag.appendChild(text);
	dom.appendChild(tag);

	return true;
}

///////////////////////////////////////////////////////////////////////////////

bool CAlias::resolveCommands(QStringList& inputList)
{
	if (!m_enabled)
		return false;

	if (inputList.count() == 0)
		return false;

	bool result = false;
	for (uint i = 0; i < inputList.count(); ++i)
	{
		QString resolved = replace(splitCommand(inputList[i]));
		if (resolved != QString::null)
		{
			inputList[i] = resolved;
			result = true;
		}
	}

	return result;
}

// end of file
