/***************************************************************************
                               cmccp.h
                             -------------------
    begin                : 06.04.2002
    copyright            : (C) 2002 by Christian Welzel
    email                : gawain@camlann.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CMCCP_H
#define CMCCP_H

#include <qobject.h>
#include <qcstring.h>

#include <stdlib.h>
#include <zlib.h>

// struct for statistical purpose. 
struct cmccpstats {
	unsigned long in;
	unsigned long out;
};

class CMCCP : public QObject {
	Q_OBJECT

public:
	CMCCP();
	~CMCCP();

	// was there an error while decompressing?
	bool error() const;

	// is the object currently in decompressing mode?
	bool isCompressing() const;

	// return the statistical struct
	const cmccpstats &stats() const;

	// main function. scans and decompresses input and produces output
	int process(QByteArray &data);

signals:
	// connect to CTelnet::sendData
	void sendToServer(QString);

	// there was an error in zlib-handling, arg is the zlib error value
	void errorSignal(int);

public slots:
	// resets the object to default state
	void reset(void);

private:

	static void *zlib_alloc(void* /*opaque*/, unsigned int items, unsigned int size) { return calloc(items, size); }
	static void zlib_free(void* /*opaque*/, void *address) { free(address); }

	int decompress(unsigned char *in, ulong size, int start_pos);
	int scan(unsigned char *in, ulong size);

	cmccpstats  m_stats;
	z_stream    m_stream;
	int         m_error;
	int         m_compress;
	int         m_state;

	unsigned char *m_buffer;
	int         m_buflen;
	int         m_bufpos;
};

#endif
