/***************************************************************************
                               cmudview.cpp
                             -------------------
    begin                : Sat Jun  16 22:10:48 CET 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qwidget.h>
#include <qpainter.h>
#include <qstringlist.h>
#include <qpixmap.h>
#include <qregexp.h>
#include <qclipboard.h>
#include <qapplication.h>

// include files for KDE
#include <krun.h>
#include <kglobalsettings.h>

// application specific includes
#include "cmudview.h"
#include "kmuddebug.h"

bool CMudView::s_blinkState = true;
CMudView::CMudView(QWidget *parent, const char *name) : QWidget(parent, name, WResizeNoErase|WRepaintNoErase)
{
	m_scrollbar = new QScrollBar(Vertical, this);
	m_scrollbar->setMinValue(0);
	m_scrollbar->setLineStep(1);
	m_scrollbar->setPageStep(30);
	m_scrollbar->setMaxValue(0);
	m_scrollbar->setFocusPolicy(WheelFocus);
	m_scrollbar->hide();
	m_scrollbaroffset = 0;
	connect(m_scrollbar, SIGNAL(valueChanged(int)), this, SLOT(update()));

	m_lines.setAutoDelete(true);
	m_line = new CMudViewLine();
	m_lines.append(m_line);

	init(QFont::defaultFont(), Qt::black, Qt::white);
	setFocusPolicy(WheelFocus);

	startTimer(1000);
	m_blink = false;

	m_startLine = 0;
	m_currLine = 0;
}

CMudView::~CMudView()
{
}

void CMudView::addText(const QString& t)
{
	if(t.isEmpty() || t.isNull()) return;

	bool scroll = (m_scrollbar->value() == m_scrollbar->maxValue());
	QRegExp rx( "(((http|ftp)://[\\w\\.-_/~]+)|(\\[img\\][\\w\\.-_/~]+))" );

	QString text = t;
	text.replace("\r\n", "\n");
	QStringList textlines = QStringList::split("\n", text, true);
	QStringList::iterator it = textlines.begin();
	QStringList::iterator last = (--textlines.end());

	for (; it != textlines.end(); ++it ) {
		int pos = 0;

		if((*it).isEmpty()) ; // only create a empty line, done in section below
		else if((pos = rx.search((*it), 0)) > -1) { // try to recognize simple URIs
			do {
				m_line->append(new CMudViewItem((*it).left(pos), m_blink, _textColor, _bgColor, _font));
				if(rx.cap(1).startsWith("[img]"))
					m_line->append(new CMudViewImgItem(rx.cap(4).mid(5)));
				else
					m_line->append(new CMudViewURLItem(rx.cap(2), m_blink, Qt::blue, _bgColor, _font));
				(*it).remove(0, pos+rx.matchedLength());
			}
			while((pos = rx.search(*it, 0)) > -1);
			if(!(*it).isEmpty()) 
				m_line->append(new CMudViewItem(*it, m_blink, _textColor, _bgColor, _font));
		}
		else { // a line without an url...
			m_line->append(new CMudViewItem((*it), m_blink, _textColor, _bgColor, _font));
		}
		// for all lines we need to open a new CMudViewLine except for the last
		// the last only needs one, if the text ends in a \n
		if(it != last || (it == last && text.endsWith("\n"))) {
			m_line = new CMudViewLine(); m_lines.append(m_line);
		}
	}
	// Scrollbar stuff follows
	m_scrollbar->setMaxValue(m_lines.count()-m_scrollbaroffset-1);
	if(scroll)
		m_scrollbar->setValue(m_scrollbar->maxValue());
	if(m_scrollbar->isHidden() && m_lines.count()>m_scrollbaroffset) {
		m_scrollbar->setMinValue(0);
		m_scrollbar->show();
	}
	// do a repaint
	update();
}

void CMudView::init(const QFont &font, const QColor &textColor, const QColor &bgColor)
{
	kdDebug(DAREA_UI_VIEW) << "init: font(" << font.family() << ", " << font.pointSize()
	                       <<"pt), color(" << textColor.name() << "/" << bgColor.name() << ")" << endl;
	_defaultFont = font;
	_defaultTextColor = textColor;
	_defaultBgColor = bgColor;
	_font = _defaultFont;
	_textColor = _defaultTextColor;
	_bgColor = _defaultBgColor;
}

void CMudView::clear()
{
	_font = _defaultFont;
	_textColor = _defaultTextColor;
	_bgColor = _defaultBgColor;
	m_blink = false;
	m_lines.clear();
	m_lines.append(m_line = new CMudViewLine());
	update();
}

void CMudView::setFont(QFont font, bool useStyle)
{
	if (!useStyle)
	{
		font.setUnderline(_defaultFont.underline());
		font.setItalic(_defaultFont.italic());
		font.setBold(_defaultFont.bold());
	}
    _font = font;
}

QFont CMudView::font()
{
	return _font;
}

void CMudView::setUnderline(bool underline)
{
	_font.setUnderline(underline);
}
	
bool CMudView::underline()
{
	return _font.underline();
}

void CMudView::setItalic(bool italic)
{
	_font.setItalic(italic);
}

bool CMudView::italic()
{
	return _font.italic();
}

void CMudView::setBold(bool bold)
{
	_font.setBold(bold);
}

bool CMudView::bold()
{
	return _font.bold();
}

void CMudView::setTextColor(QColor color)
{
	_textColor = color;
}

QColor CMudView::textColor()
{
	return _textColor;
}

void CMudView::setBgColor(QColor color)
{
	_bgColor = color;
}

QColor CMudView::bgColor()
{
	return _bgColor;
}

void CMudView::setBlink(bool state)
{
	m_blink = state;
}

bool CMudView::blink()
{
	return m_blink;
}

void CMudView::paintEvent(QPaintEvent *)
{
	int i;
	CMudViewLine *line;

	QPixmap pm(size());
	pm.fill(_defaultBgColor);
	QPainter p(&pm);
	p.setPen(Qt::black);
	if(hasFocus()) p.drawWinFocusRect(0,0,width(), height());
	p.translate(3, (i=height()-3-QFontMetrics(_font).height()));

	if(m_scrollbar->value() != m_scrollbar->maxValue())
		line = m_lines.at(m_scrollbar->value()+m_scrollbaroffset);
	else
		line = m_lines.last();

	for(; line && i>=3; line = m_lines.prev()) {
		line->paint(p);
		p.translate(0, -line->size().height());
		i -= line->size().height();
	}
	bitBlt(this, 0, 0, &pm);
}

void CMudView::resizeEvent(QResizeEvent *e)
{
	m_scrollbar->setGeometry(e->size().width()-m_scrollbar->sizeHint().width(), 0, 
							 m_scrollbar->sizeHint().width(), e->size().height());
	m_scrollbaroffset = height()/QFontMetrics(_font).height();
	if(m_scrollbar->isShown() && m_lines.count()<m_scrollbaroffset)
		m_scrollbar->hide();
	if(m_scrollbar->isHidden() && m_lines.count()>m_scrollbaroffset) {
		m_scrollbar->setMaxValue(m_lines.count()-1-m_scrollbaroffset);
		m_scrollbar->setPageStep(m_scrollbaroffset);
		m_scrollbar->show();
	}
}

void CMudView::timerEvent(QTimerEvent *)
{
	s_blinkState = !s_blinkState;
	update();
}

void CMudView::wheelEvent(QWheelEvent *e)
{
	if(e->delta()<0)
		m_scrollbar->addLine();
	else
		m_scrollbar->subtractLine();
	e->accept();
}

void CMudView::keyPressEvent(QKeyEvent *e)
{
	switch(e->key()) {
		case Key_PageUp: m_scrollbar->subtractPage(); break;
		case Key_PageDown: m_scrollbar->addPage(); break;
		case Key_Home: m_scrollbar->setValue(0); break;
		case Key_End: m_scrollbar->setValue(m_scrollbar->maxValue()); break;
		case Key_Up: m_scrollbar->subtractLine(); break;
		case Key_Down: m_scrollbar->addLine(); break;
		default: e->ignore(); return;
	}
	e->accept();
}

CMudView::CMudViewLine *CMudView::findLine(const QPoint &p, int *h)
{
	int i = height();
	CMudViewLine *line;

	if(m_scrollbar->value() != m_scrollbar->maxValue())
		line = m_lines.at(m_scrollbar->value()+m_scrollbaroffset);
	else
		line = m_lines.last();

	for(; line && i >= p.y(); line = m_lines.prev()) {
		i -= line->size().height();
	}
	if(h) *h=i;
	if(line) return m_lines.next();
	return NULL;
}

QMouseEvent *CMudView::deliverMouseEvent(QMouseEvent *m)
{
	int i;
	if(CMudViewLine *line = findLine(m->pos(), &i)) {
		QPoint p(m->x(), m->y()-i-1);
		QMouseEvent me(m->type(), p, m->globalPos(), m->button(), m->state());
		line->mouseEvent(&me);
		if(me.isAccepted()) m->accept();
	}
	return m;
}

void CMudView::mousePressEvent(QMouseEvent *m)
{
	deliverMouseEvent(m);

	// remember the beginning of (possible) text selection
	m_startLine  = m_currLine  = findLine(m->pos());
	m_startPoint = m_currPoint = m->pos();

	// clear the previous selection
	CMudViewLine *line = m_lines.last();
	for(; line; line = m_lines.prev()) {
		line->clearSelection();
	}

	if(m_startLine)
		m_startLine->setSelectionStart(m->pos().x());
}

void CMudView::mouseReleaseEvent(QMouseEvent *m)
{
	deliverMouseEvent(m);

	// get the text from the selection
	QString selection;
	CMudViewLine *line = m_lines.first();
	for(; line; line = m_lines.next()) {
		selection += line->getSelection();
	}
	if(!selection.isEmpty()) {
    	QClipboard *cb = QApplication::clipboard();
		cb->setText(selection, QClipboard::Selection);
	}
	m_currLine = 0;
}

void CMudView::mouseDoubleClickEvent(QMouseEvent *m)
{
	int i;
	deliverMouseEvent(m);
	if(CMudViewLine *line = findLine(m->pos(), &i)) {
		QPoint p(m->x(), m->y()-i-1);
		QString word = line->getWord(p);
		if(!word.isEmpty()) {
    		QClipboard *cb = QApplication::clipboard();
			cb->setText(word, QClipboard::Selection);
		}
	}
}

void CMudView::mouseMoveEvent(QMouseEvent *m)
{
	CMudViewLine *old_curr;
	QPoint old_p;

	// do the scrolling if the mouse is out of widget
	old_curr = m_currLine;
	old_p = m_currPoint;

	if(m->pos().y()<0 && m_scrollbar->value()>m_scrollbar->minValue()) {
		m_scrollbar->subtractLine();
		m_currLine  = findLine(QPoint(0, 1));
	}
	else if(m->pos().y()>height() && m_scrollbar->value()<m_scrollbar->maxValue()) {
		m_scrollbar->addLine();
		m_currLine  = findLine(QPoint(0, height()-1));
	}
	else
		m_currLine  = findLine(m->pos());

	if(!m_currLine) return;

	// TODO: update the selection
	m_currPoint = m->pos();

	if(old_curr == m_currLine) {
		m_currLine->setSelectionEnd(m->pos().x());
	}
	else if(m_lines.findRef(old_curr) < m_lines.findRef(m_currLine)) {
		old_curr->setSelectionEnd(0);
		m_currLine->setSelectionStart(0);
		m_currLine->setSelectionEnd(m->pos().x());
kdDebug(DAREA_UI_VIEW) << " +++ CMV::CMVI::a line down"<<endl;
	}
	else {
kdDebug(DAREA_UI_VIEW) << " +++ CMV::CMVI::a line up"<<endl;
	}

	update();
}

/**
 * Methods of CMudViewLine class
 */

void CMudView::CMudViewLine::paint(QPainter &p)
{
	p.save();
    for(CMudViewItem *item = m_lineMembers.first(); item; item = m_lineMembers.next()) {
		item->paint(p);
		p.translate(item->size().width(), 0);
	}
	p.restore();
}

void CMudView::CMudViewLine::append(const CMudViewItem *item)
{
	m_lineMembers.append(item);
	if(m_size.height() < item->size().height()) 
		m_size.setHeight(item->size().height());
	m_size.setWidth(m_size.width() + item->size().width());
}

// if the function returns NULL, the value of *x will not be changed!
CMudView::CMudViewItem *CMudView::CMudViewLine::findItem(const QPoint &p, int *x)
{
	int i=0, j=0;

	CMudViewItem *item = m_lineMembers.first();
	if(!item) return NULL;
	do {
		i=j;
		j += item->size().width();
		if(i<p.x() && p.x()<j) break;
	}
	while((item = m_lineMembers.next()));

	if(x) *x=i;
	return item;
}

QString CMudView::CMudViewLine::getWord(const QPoint &p)
{
	int i;
	CMudViewItem *item = findItem(p, &i);

	if(item) {
		QPoint p2(p.x()-i-1, p.y());
		return item->getWord(p2);
	}
	return QString::null;
}

void CMudView::CMudViewLine::mouseEvent(QMouseEvent *e) 
{
	int i;
	CMudViewItem *item = findItem(e->pos(), &i);

	if(item) {
		QPoint p(e->x()-i-1, e->y());
		QMouseEvent me(e->type(), p, e->globalPos(), e->button(), e->state());
		item->mouseEvent(&me);
		if(me.isAccepted()) e->accept();
	}
}

QString CMudView::CMudViewLine::getSelection()
{
// TODO
	return "";
}

void CMudView::CMudViewLine::clearSelection()
{
	m_selbegin = m_selend = -1;
	CMudViewItem *item = m_lineMembers.first();
	for(; item; item = m_lineMembers.next()) {
		item->clearSelection();
	}
}

void CMudView::CMudViewLine::updateSelection() {
	CMudViewItem *item = findItem(QPoint(m_selbegin, 1));
	CMudViewItem *last = findItem(QPoint(m_selend, 1));
// TODO!
	if(item && last) {
		if(item == last) {
			item->setSelectionStart(m_selbegin);
			item->setSelectionEnd(m_selend);
		}
		else
			for(; item != last; item = m_lineMembers.next()) {
				item->clearSelection();
			}
	}
}

/** 
 * Methods of CMudViewItem class and derivates
 */

QString CMudView::CMudViewItem::getWord(const QPoint &p)
{
	int i = 0;
	QFontMetrics fm(m_font);
	QStringList words = QStringList::split(QRegExp("[\\s\\.,;\"]"), m_content, true);
	QStringList::Iterator it = words.begin();
	for (; it != words.end(); ++it ) {
		i += fm.width(*it);
		if(i>=p.x()) break;
		else i += fm.width(" ");
	}
	return *it;
}

QString CMudView::CMudViewItem::getSelection()
{
// TODO
kdDebug(DAREA_UI_VIEW) << " +++ CMV::CMVI::getSelection()"<<endl;
}

void CMudView::CMudViewItem::paint(QPainter &p)
{
	p.fillRect(0, 0, size().width(), size().height(), m_bgcolor);
	if(m_selbegin >= 0)
		p.fillRect(m_selbegin, 0, m_selend-m_selbegin, size().height(), Qt::blue);
	if(!m_blinker || CMudView::getblinkState()) {
		p.setPen(m_fgcolor);
		p.setFont(m_font);
		p.drawText(0, QFontMetrics(m_font).ascent(), m_content);
	}
}

void CMudView::CMudViewURLItem::mouseEvent(QMouseEvent *e) 
{
	QEvent::Type type;
	if(KGlobalSettings::singleClick())
		type = QEvent::MouseButtonPress;
	else
		type = QEvent::MouseButtonDblClick;
	if(e->type() == type) {
		KURL url(m_content);
		new KRun(url);
	}
}

void CMudView::CMudViewImgItem::paint(QPainter &p)
{
	p.drawPixmap(0, -m_pic.height()+QFontMetrics(m_font).height(), m_pic);
}
