/***************************************************************************
                               cprofileitem.h
                             -------------------
    begin                : Thu Jul 25 2002
    copyright            : (C) 2002 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CPROFILEITEM_H
#define CPROFILEITEM_H

// Qt includes
#include <qstring.h>

// forward declaration
class QDomElement;

/** Basic interface for profile items like aliases or triggers
  * @author Kmud Developer Team
  */

class CProfileItem {
public:
	/**
	  * Constructs a disabled item with empty name.
	  */
	CProfileItem() : m_enabled(false) {};

	/**
	  * Constructs a disabled imte with the given name.
	  * @param name the item's name. Any string allowed.
	  */
	CProfileItem(const QString& name);

	CProfileItem(const CProfileItem& item) {
		if (&item == this) return;
		m_enabled  = item.m_enabled;
		m_name     = item.m_name;
	}

	virtual ~CProfileItem() {};

	inline CProfileItem& operator=(const CProfileItem& item) {
		if (&item != this) // avoid self assignment
		{
			m_enabled  = item.m_enabled;
			m_name     = item.m_name;
		}
		return *this;
	}

	/**
	  * Read the item's content from the given QDomElement.
	  * @return true if reading succeeded, false otherwise.
	  */
	virtual bool readFromDOM(const QDomElement& dom);

	/**
	  * Write the item's content to the given QDomElement.
	  * @return true if reading succeeded, false otherwise.
	  */
	virtual bool writeToDOM(QDomElement& dom);


	/**
	  * Enable the Item.
	  * The meaning of enabled/disabled depends on the implemention of the subclass.
	  * @param enabled if true the item gets enabled, if false it gets dislabled.
	  */
	virtual void setEnabled(bool enabled) { m_enabled = enabled; }
	
	/**
	  * Returns the wether the item is enabled.
	  * The meaning of enabled/disabled depends on the implemention of the subclass.
	  * @return true if the item is enabled.
	  */
	virtual bool isEnabled() const { return m_enabled; }

	/**
	  * Sets the name of the item.
	  * @param name the name of the profile item. Any string allowed.
	  */
	void setName(const QString& name) { m_name = name; }

	/**
	  * Returns the item's name.
	  */
	QString name() const { return m_name; }

protected:
	bool m_enabled;
	QString m_name;

protected:
	void writeAttribute(QDomElement& element, const QString& name, bool value);

	bool readBoolAttribute(const QDomElement& element, const QString& name, bool def);
};

#endif
