/***************************************************************************
                               dlgconfig.cpp
                             -------------------
    begin                : Wed Jun 26 2002
    copyright            : (C) 2002 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dlgconfig.h"

#include <klocale.h>

#include <qlistview.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qsizepolicy.h>

#include <kmudpart.h>
#include <kmudsetuppane.h>

#include "../kmudcore.h"

#include <kmuddebug.h>

DlgConfig::DlgConfig(KmudCore *core,QString title,QWidget *parent) : DlgConfigBase(parent,title,true)
{	
	m_core = core;

	m_firstItem = NULL;
	m_panels = new QPtrList<KmudSetupPane>;
	m_panels->setAutoDelete(false);
	m_panels->clear();

	m_grid = new QGridLayout(m_paneConfig,1,1);
	m_widgets = NULL;
}

DlgConfig::~DlgConfig()
{
	delete m_panels;
}

void DlgConfig::populateListView()
{
	if (m_widgets)
	{
		return;
	}
	m_widgets = new QWidgetStack(m_paneConfig);

	m_treeConfig->clear();
	m_panels->clear();


	addKmudPanes();
	addPluginPanes();

	if (m_firstItem)
	{
		m_treeConfig->ensureItemVisible(m_firstItem);
		m_treeConfig->setSelected(m_firstItem,true);
	}

	m_grid->addWidget(m_widgets,0,0);
}

void DlgConfig::addPane(KmudSetupPane *pane)
{
	m_panels->append(pane);
	connect(this,SIGNAL(defaultClicked()),pane,SLOT(slotDefaultPressed()));

	pane->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
	m_widgets->addWidget(pane,m_panels->count()+1);

	QListViewItem *parent = findParent(pane->getGroup());
	QListViewItem *item = NULL;
	if (!parent)
	{
		item = new QListViewItem(m_treeConfig,pane->name());
	}
	else
	{
		item = new QListViewItem(parent,pane->name());
	}

	if (!m_firstItem)
	{
		m_widgets->raiseWidget(pane);
		m_firstItem = item;
	}
}

QListViewItem *DlgConfig::findParent(QString group)
{

	if (group.isEmpty())
	{
		return NULL;
	}

	QListViewItem *item = NULL;
	int index = 0;
	int lastIndex = 0;
	while (index!=-1)
	{
		index = group.find('/',index+1);

		QString name = group.mid(lastIndex,index-lastIndex);
		QListViewItem *child;
		if (item==NULL)
		{
			child = m_treeConfig->firstChild();
		}
		else
		{
			child = item->firstChild();
		}

		while (child!=NULL)
		{
			if (child->text(0)==name)
			{
				break;
			}

			child= child->nextSibling();
		}

		if (!child)
		{
			if (item==NULL)
			{
				child = new QListViewItem(m_treeConfig,name);
			}
			else
			{
				child = new QListViewItem(item,name);
			}
		}

		m_treeConfig->setOpen(child,true);
		item = child;

		lastIndex = index+1;
	}

	return item;

}

QString DlgConfig::getGroup(QListViewItem *item)
{

	QString group="";

	item = item->parent();

	while(item!=NULL)
	{
		group = item->text(0) + "/" + group  ;
		item = item->parent();
	}

	return group.mid(0,group.length()-1);

}

void DlgConfig::slotDefault()
{

	emit defaultClicked();
}

void DlgConfig::slotAccept()
{
	for (KmudSetupPane *pane = m_panels->first(); pane!=0 ; pane=m_panels->next())
	{
		pane->slotOkPressed();
	}

	QPtrList<KmudPart> *parts = m_core->getPartsList();
	for (KmudPart *part = parts->first(); part!=0; part = parts->next())
	{
		part->saveGlobalConfig();
	}

	accept();
}

void DlgConfig::slotCancel()
{
	for (KmudSetupPane *pane = m_panels->first(); pane!=0 ; pane=m_panels->next())
	{
		pane->slotCancelPressed();
	}

	reject();
}

void DlgConfig::slotPaneSelected(QListViewItem *item)
{
	if (item)
	{
		QString name = item->text(0);
		QString group = getGroup(item);

		for (KmudSetupPane *pane = m_panels->first(); pane!=0 ; pane=m_panels->next())
		{
			if (pane->name() == name && pane->getGroup() == group)
			{
				m_widgets->raiseWidget(pane);
				return;
			}
		}
    }

}
