/***************************************************************************
                          dlgconnectionwizard.cpp  -  description
                             -------------------
    begin                : Thu Mar 8 2001
    copyright            : (C) 2001 by KMud Development Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


// Qt includes
#include <qdom.h>
#include <qstringlist.h>
#include <qdom.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qtimer.h>

// KDE includes
#include <klistview.h>
#include <kmessagebox.h>
#include <klocale.h>

// Kmud includes
#include "dlgconnectionwizard.h"
#include "dlgcreateprofile.h"
#include "../../lib/kmudapi.h"
#include "../../lib/kmemconfig.h"
#include "../../lib/domconfig.h"
#include "../../lib/kmuddebug.h"

///////////////////////////////////////////////////////////////////////////////

DlgConnectionWizard::DlgConnectionWizard(ProfileAPI* profileAPI, QWidget *parent, const char *name )
	: DlgConnectionWizardBase(parent,name,true), m_profileAPI(profileAPI)
{
	m_edCharPasswd->setEchoMode(QLineEdit::Password);
	m_spMudPort->setValue(1);

	QObject::connect(m_btnActivate, SIGNAL(clicked()), this, SLOT(slotActivateProfile()));

	QObject::connect(m_btnProfAdd, SIGNAL(clicked()), this, SLOT(slotAddProfile()));
	QObject::connect(m_btnProfRemove, SIGNAL(clicked()), this, SLOT(slotRemoveProfile()));
	slotConnectListView();

	QObject::connect(m_btnApply, SIGNAL(clicked()), this, SLOT(slotApplyData()));
	QObject::connect(m_btnReset, SIGNAL(clicked()), this, SLOT(slotResetData()));

	QObject::connect(m_edMudName, SIGNAL(textChanged(const QString&)), this, SLOT(slotDataChanged()));
	QObject::connect(m_edMudServer, SIGNAL(textChanged(const QString&)), this, SLOT(slotDataChanged()));
	QObject::connect(m_spMudPort, SIGNAL(valueChanged(int)), this, SLOT(slotDataChanged()));
	QObject::connect(m_edMudPasswordPrompt, SIGNAL(textChanged(const QString&)), this, SLOT(slotDataChanged()));
	QObject::connect(m_edMudLoginPrompt, SIGNAL(textChanged(const QString&)), this, SLOT(slotDataChanged()));

	QObject::connect(m_edCharName, SIGNAL(textChanged(const QString&)), this, SLOT(slotDataChanged()));
	QObject::connect(m_edCharLogin, SIGNAL(textChanged(const QString&)), this, SLOT(slotDataChanged()));
	QObject::connect(m_edCharPasswd, SIGNAL(textChanged(const QString&)), this, SLOT(slotDataChanged()));

}

///////////////////////////////////////////////////////////////////////////////

DlgConnectionWizard::~DlgConnectionWizard()
{
}

///////////////////////////////////////////////////////////////////////////////

void DlgConnectionWizard::saveProfileList(KConfigBase* config, const QString& group)
{
	if (config == 0)
		return;
	
	QString oldGroup = config->group();
	config->setGroup(group);

	QStringList muds;
	for (QListViewItem* mudItem = m_listView->firstChild(); mudItem != 0; mudItem = mudItem->nextSibling())
	{
		QString mudName = mudItem->text(0);
		muds.append(mudName);
		uint state = 0;
		if (mudItem->isOpen())
			state |= ITEM_OPEN;
		if (mudItem->isSelected())
			state |= ITEM_SELECTED;
		config->writeEntry(mudName+"_state", state);
		
		QStringList chars;
		for (QListViewItem* charItem = mudItem->firstChild(); charItem != 0; charItem = charItem->nextSibling())
		{
			QString charName = charItem->text(0);
			chars.append(charName);
			uint state = 0;
			if (charItem->isSelected())
				state |= ITEM_SELECTED;
			config->writeEntry(mudName+"_"+charName+"_state", state);
		}
		config->writeEntry(mudName+"_chars", chars);
	}
	
	config->writeEntry("muds", muds);
	config->setGroup(oldGroup);
}


///////////////////////////////////////////////////////////////////////////////

void DlgConnectionWizard::loadProfileList(KConfigBase* config, const QString& group)
{
	if (config == 0)
		return;

	m_listView->clear();

	QListViewItem* selectedItem = 0;

	QString oldGroup = config->group();
	config->setGroup(group);

	QStringList muds = config->readListEntry("muds");
	for (uint i = 0; i < muds.count(); ++i)
	{
		QListViewItem* mudItem = new QListViewItem(m_listView, muds[i]);

		uint state = config->readUnsignedNumEntry(muds[i]+"_state", 0);
		QStringList chars = config->readListEntry(muds[i]+"_chars");

		if ((state & ITEM_OPEN) != 0 && chars.count() != 0)
			mudItem->setOpen(true);
		if ((state & ITEM_SELECTED) != 0 && selectedItem == 0)
			selectedItem = mudItem;

		for (uint j = 0; j < chars.count(); ++j)
		{
			QListViewItem* charItem = new QListViewItem(mudItem, chars[j]);
			uint state = config->readUnsignedNumEntry(muds[i]+"_"+chars[j]+"_state", 0);
			if ((state & ITEM_SELECTED) != 0 && selectedItem == 0)
				selectedItem = charItem;
		}
	}

	config->setGroup(oldGroup);

	if (selectedItem != 0)
		m_listView->setSelected(selectedItem, true);
}

///////////////////////////////////////////////////////////////////////////////

bool DlgConnectionWizard::loadMudProfileData(const QString& mudProfile)
{
	// clear inputs and disable them
	m_edMudName->blockSignals(true);
	m_edMudServer->blockSignals(true);
	m_spMudPort->blockSignals(true);
	m_edMudName->clear();
	m_edMudServer->clear();
	m_spMudPort->setValue(1);
	m_edMudName->setEnabled(false);
	m_edMudServer->setEnabled(false);
	m_spMudPort->setEnabled(false);
	m_edMudLoginPrompt->setEnabled(false);
	m_edMudPasswordPrompt->setEnabled(false);

	// disable ping button
	m_btnPing->setEnabled(false);

	if (mudProfile.isEmpty() || m_profileAPI == 0)
		return true;

	// try to open the profile
	QDomElement profileElement = m_profileAPI->openProfile(mudProfile);
	if (profileElement.isNull())
		return false;

	DomConfig* domConfig = DomConfig::findGroup("/", profileElement);
	if (domConfig == 0)
		return true;

	KMemConfig* config = domConfig->config();
	config->setGroup("General");
	m_edMudName->setText(config->readEntry("Name"));

	QString server = config->readEntry("Server");
	uint port = config->readUnsignedNumEntry("Port", 0);
	m_edMudServer->setText(server);
	m_spMudPort->setValue(port);

	QString loginPrompt = config->readEntry("LoginPrompt");
	QString passwordPrompt = config->readEntry("PasswordPrompt");
	m_edMudLoginPrompt->setText(loginPrompt);
	m_edMudPasswordPrompt->setText(passwordPrompt);                                      

	delete domConfig;

	if (!server.isEmpty() && port >= 1 && port <= 65535)
		m_btnPing->setEnabled(true);

	// enable inputs
    m_edMudLoginPrompt->setEnabled(true);
	m_edMudPasswordPrompt->setEnabled(true);
	m_edMudName->setEnabled(true);
	m_edMudServer->setEnabled(true);
	m_spMudPort->setEnabled(true);
	m_edMudName->blockSignals(false);
	m_edMudServer->blockSignals(false);
	m_spMudPort->blockSignals(false);

	return true;
}

///////////////////////////////////////////////////////////////////////////////

void DlgConnectionWizard::saveMudProfileData(const QString& mudProfile)
{
	if (mudProfile.isEmpty() || m_profileAPI == 0)
		return;

	// try to open the profile
	QDomElement profileElement = m_profileAPI->openProfile(mudProfile);
	if (profileElement.isNull())
		return;

	DomConfig* domConfig = DomConfig::findGroup("/", profileElement, true);
	if (domConfig == 0)
		return;

	KMemConfig* config = domConfig->config();
	config->setGroup("General");
	config->writeEntry("Name", m_edMudName->text());
	config->writeEntry("Server", m_edMudServer->text());
	config->writeEntry("Port", (uint)m_spMudPort->value());
	config->writeEntry("LoginPrompt",m_edMudLoginPrompt->text());
	config->writeEntry("PasswordPrompt",m_edMudPasswordPrompt->text());

	domConfig->save();
	delete domConfig;

	m_profileAPI->saveProfile(mudProfile);
}

///////////////////////////////////////////////////////////////////////////////

bool DlgConnectionWizard::loadCharProfileData(const QString& charProfile)
{
	// clear inputs and disable them
	m_edCharName->blockSignals(true);
	m_edCharLogin->blockSignals(true);
	m_edCharPasswd->blockSignals(true);
	m_edCharName->clear();
	m_edCharLogin->clear();
	m_edCharPasswd->clear();
	m_edCharName->setEnabled(false);
	m_edCharLogin->setEnabled(false);
	m_edCharPasswd->setEnabled(false);

	if (charProfile.isEmpty() || m_profileAPI == 0)
		return true;

	// try to open the profile
	QDomElement profileElement = m_profileAPI->openProfile(charProfile);
	if (profileElement.isNull())
		return false;

	DomConfig* domConfig = DomConfig::findGroup("/", profileElement);
	if (domConfig == 0)
		return true;

	KMemConfig* config = domConfig->config();
	config->setGroup("General");
	m_edCharName->setText(config->readEntry("Name"));

	config->setGroup("Login");
	m_edCharLogin->setText(config->readEntry("Login"));
	m_edCharPasswd->setText(config->readEntry("Password"));

	delete domConfig;

	// enable inputs
	m_edCharName->setEnabled(true);
	m_edCharLogin->setEnabled(true);
	m_edCharPasswd->setEnabled(true);
	m_edCharName->blockSignals(false);
	m_edCharLogin->blockSignals(false);
	m_edCharPasswd->blockSignals(false);

	return true;
}

///////////////////////////////////////////////////////////////////////////////

void DlgConnectionWizard::saveCharProfileData(const QString& charProfile)
{
	if (charProfile.isEmpty() || m_profileAPI == 0)
		return;

	// try to open the profile
	QDomElement profileElement = m_profileAPI->openProfile(charProfile);
	if (profileElement.isNull())
		return;

	DomConfig* domConfig = DomConfig::findGroup("/", profileElement, true);
	if (domConfig == 0)
		return;

	KMemConfig* config = domConfig->config();
	config->setGroup("General");
	config->writeEntry("Name", m_edCharName->text());

	config->setGroup("Login");
	config->writeEntry("Login", m_edCharLogin->text());
	config->writeEntry("Password", m_edCharPasswd->text());

	domConfig->save();
	delete domConfig;

	m_profileAPI->saveProfile(charProfile);
}

///////////////////////////////////////////////////////////////////////////////

void DlgConnectionWizard::accept()
{
	QListViewItem* item = m_listView->selectedItem();
	if (item == 0)
		return;

	DlgConnectionWizardBase::accept();
}

///////////////////////////////////////////////////////////////////////////////

void DlgConnectionWizard::reject()
{
	m_selectedMudProfile = QString::null;
	m_selectedCharProfile = QString::null;
	DlgConnectionWizardBase::reject();
}

///////////////////////////////////////////////////////////////////////////////

void DlgConnectionWizard::slotActivateProfile()
{
	QListViewItem* item = m_listView->selectedItem();
	if (item == 0)
		return;

	done(ACTIVATE);
}

///////////////////////////////////////////////////////////////////////////////

void DlgConnectionWizard::slotAddProfile()
{
	if (m_profileAPI == 0)
		return;

	
	KMemConfig memConfig;
	saveProfileList(&memConfig, "ProfileList");
	memConfig.setGroup("ProfileList");

	DlgCreateProfile dlg(m_profileAPI, this, "CreateProfile Dialog");
	QStringList muds = memConfig.readListEntry("muds");
	dlg.setMudList(muds);
	QListViewItem* item = m_listView->selectedItem();
	if (item != 0)
	{
		if (item->parent() != 0)
			dlg.setCurrentMud(item->parent()->text(0));
		else
			dlg.setCurrentMud(item->text(0));
	}

	if (dlg.exec())
	{
		QString mudName = dlg.mudName();
		QString charName = dlg.charName();
		if (charName.isEmpty())
		{
			// new mud profile
			if (!muds.contains(mudName))
			{
				muds.append(mudName);
				memConfig.writeEntry("muds", muds);
			}

			for (uint i = 0; i < muds.count(); ++i)
			{
				if (muds[i] == mudName)
					memConfig.writeEntry(mudName + "_state", ITEM_SELECTED);
				else
				{
					uint state = memConfig.readUnsignedNumEntry(muds[i] + "_state", 0);
					memConfig.writeEntry(muds[i] + "_state", (state & ITEM_OPEN));
				}
			}
		}
		else
		{
			// new char profile
			for (uint i = 0; i < muds.count(); ++i)
			{
				QStringList chars = memConfig.readListEntry(muds[i] + "_chars");
				if (muds[i] == mudName)
				{
					memConfig.writeEntry(mudName + "_state", ITEM_OPEN);
					if (!chars.contains(charName))
					{
						chars.append(charName);
						memConfig.writeEntry(mudName + "_chars", chars);
					}
				}
				else
				{
					uint state = memConfig.readUnsignedNumEntry(muds[i] + "_state", 0);
					memConfig.writeEntry(muds[i] + "_state", (state & ITEM_OPEN));
				}

				for (uint j = 0; j < chars.count(); ++j)
				{
					if (muds[i] == mudName && chars[j] == charName)
					{
						memConfig.writeEntry(muds[i] + "_" + chars[j] + "_state", ITEM_SELECTED);
					}
					else
					{
						memConfig.writeEntry(muds[i] + "_" + chars[j] + "_state", (uint)0);
					}
				}
			}
			
		}
		loadProfileList(&memConfig, "ProfileList");
	}

/*
	memConfig.setGroup("ProfileList");
	QStringList muds = m_profileAPI->listProfiles();
	if (muds.count() == 0)
	{
		QDomElement mudProf = m_profileAPI->openProfile("DefaultMud", true);
		m_profileAPI->saveProfile("DefaultMud");
		muds.append("DefaultMud");
	}
	memConfig.writeEntry("muds", muds);

	for (uint i = 0; i < muds.count(); ++i)
	{
		QStringList chars = m_profileAPI->listProfiles(muds[i]);
		if (chars.count() == 0 && muds[i] == "DefaultMud")
		{
			QDomElement charProf = m_profileAPI->openProfile("DefaultMud/DefaultChar", true);
			m_profileAPI->saveProfile("DefaultMud/DefaultChar");
			chars.append("DefaultChar");
		}
		memConfig.writeEntry(muds[i]+"_chars", chars);
	}
*/
}

///////////////////////////////////////////////////////////////////////////////

void DlgConnectionWizard::slotRemoveProfile()
{
	QListViewItem* item = m_listView->selectedItem();
	if (item == 0)
		return;

	QString message;
	if (item->parent() != 0)
	{
		message = QString("Do you really want to remove the\ncharacter profile %1?").arg(item->text(0));
	}
	else
	{
		message = QString("Do you really want to remove the\nmud profile %1 and all it's character?").arg(item->text(0));
	}
	message += "\n\nProfiles are only removed from this list of selected profiles,\ntheir data remains on disk.";

	if (KMessageBox::warningYesNo(this, i18n(message), i18n("Remove profile")) == KMessageBox::Yes)
		delete item;
}

///////////////////////////////////////////////////////////////////////////////

void DlgConnectionWizard::slotSelectionChanged()
{
	// deactivate all profile related buttons
	m_btnProfRemove->setEnabled(false);
	m_btnProfEdit->setEnabled(false);
	m_btnConnect->setEnabled(false);
	m_btnActivate->setEnabled(false);
	m_btnApply->setEnabled(false);
	m_btnReset->setEnabled(false);

	QListViewItem* item = m_listView->selectedItem();
	if (item == 0)
	{
		m_selectedCharProfile = QString::null;
		loadMudProfileData(QString::null);

		m_selectedMudProfile = QString::null;
		loadCharProfileData(QString::null);

		return;
	}

	QString mudProfile;
	QString charProfile;
	bool charDataOK = false;
	bool mudDataOK = true;

	QListViewItem* parentItem = item->parent();
	if (parentItem != 0)
	{
		// char profile selected
		charProfile = parentItem->text(0);
		charProfile.append("/");
		charProfile.append(item->text(0));
		
		charDataOK = loadCharProfileData(charProfile);
		if (!charDataOK)
		{
			m_selectedCharProfile = QString::null;
			disconnect(m_listView, SIGNAL(selectionChanged()), this, SLOT(slotSelectionChanged()));
			delete item;
			QTimer::singleShot(0, this, SLOT(slotConnectListView()));
			return;
		}

		item = parentItem; // set item to the mud item
	}
	else if (!m_selectedCharProfile.isEmpty()) // there was a char profile selected
	{
		m_selectedCharProfile = QString::null;
		loadCharProfileData(QString::null);
	}

	// item is a mud profile
	mudProfile = item->text(0);
	if (mudProfile != m_selectedMudProfile)  // load only if a new mud was selected
		mudDataOK = loadMudProfileData(mudProfile);

	if (!mudDataOK)
	{
		m_selectedMudProfile = QString::null;
		disconnect(m_listView, SIGNAL(selectionChanged()), this, SLOT(slotSelectionChanged()));
		delete item;
		QTimer::singleShot(0, this, SLOT(slotConnectListView()));
		return;
	}

	if (mudDataOK)
	{
		if (charDataOK)
		{
			m_btnConnect->setEnabled(true);
			m_btnActivate->setEnabled(true);
		}

		m_btnProfRemove->setEnabled(true);
		m_btnProfEdit->setEnabled(true);
	}

	m_selectedMudProfile = mudProfile;
	m_selectedCharProfile = charProfile;
}

///////////////////////////////////////////////////////////////////////////////

void DlgConnectionWizard::slotApplyData()
{
	m_btnApply->setEnabled(false);
	m_btnReset->setEnabled(false);
	saveMudProfileData(m_selectedMudProfile);
	saveCharProfileData(m_selectedCharProfile);
}

///////////////////////////////////////////////////////////////////////////////

void DlgConnectionWizard::slotResetData()
{
	m_btnApply->setEnabled(false);
	m_btnReset->setEnabled(false);
	loadMudProfileData(m_selectedMudProfile);
	loadCharProfileData(m_selectedCharProfile);
}

///////////////////////////////////////////////////////////////////////////////

void DlgConnectionWizard::slotDataChanged()
{
	m_btnApply->setEnabled(true);
	m_btnReset->setEnabled(true);
	m_btnPing->setEnabled(!m_edMudServer->text().isEmpty());
}


///////////////////////////////////////////////////////////////////////////////

void DlgConnectionWizard::slotConnectListView()
{
	QObject::connect(m_listView, SIGNAL(selectionChanged()), this, SLOT(slotSelectionChanged()));
}

// end of file

