/***************************************************************************
                               dlgcreateprofile.cpp
                             -------------------
    begin                : Sun Jul 14 2002
    copyright            : (C) 2002 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


// Qt includes
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qdom.h>

// KDE includes
#include <kconfigbase.h>

// Kmud includes
#include "dlgcreateprofile.h"
#include "../../lib/kmudapi.h"

DlgCreateProfile::DlgCreateProfile(ProfileAPI* profileAPI, QWidget *parent, const char *name ) : DlgCreateProfileBase(parent,name, true)
{
	m_profileAPI = profileAPI;

	m_edName->setFocus();
	QObject::connect(m_rbMud, SIGNAL(toggled(bool)), this, SLOT(slotProfileTypeChanged(bool)));
	QObject::connect(m_rbChar, SIGNAL(toggled(bool)), this, SLOT(slotProfileTypeChanged(bool)));

	QObject::connect(m_edName, SIGNAL(textChanged(const QString&)), this, SLOT(slotNameChanged(const QString&)));

	QObject::connect(m_btnOK, SIGNAL(clicked()), this, SLOT(accept()));
	QObject::connect(m_btnCancel, SIGNAL(clicked()), this, SLOT(reject()));

	m_rbMud->setChecked(true);
}

///////////////////////////////////////////////////////////////////////////////

DlgCreateProfile::~DlgCreateProfile()
{
}

///////////////////////////////////////////////////////////////////////////////

void DlgCreateProfile::setMudList(const QStringList& muds)
{
	m_cbMud->clear();
	m_cbMud->insertStringList(muds);
}

///////////////////////////////////////////////////////////////////////////////

void DlgCreateProfile::setCurrentMud(const QString& mudName)
{
	if (mudName.isEmpty())
		return;

	for (int i = 0; i < m_cbMud->count(); ++i)
	{
		if (m_cbMud->text(i) == mudName)
		{
			m_cbMud->setCurrentItem(i);
			m_rbChar->setChecked(true);
			break;
		}
	}
}

///////////////////////////////////////////////////////////////////////////////

void DlgCreateProfile::accept()
{
	if (m_profileAPI == 0)
		return;

	if (m_edName->text().stripWhiteSpace().isEmpty())
	{
		m_edName->clear();
		return;
	}

	m_profile = m_edName->text().stripWhiteSpace();

	QString profile;
	if (m_rbChar->isChecked())
	{
		m_mud = m_cbMud->currentText();
		m_char = m_profile;
		profile.append(m_mud);
		profile.append("/");
	}
	else
	{
		m_mud = m_profile;
		m_char = QString::null;
	}
	profile.append(m_profile);
	
	QDomElement profileElement = m_profileAPI->openProfile(profile, true);
	if (!profileElement.isNull())
		DlgCreateProfileBase::accept();
	else
	{
		m_profile = QString::null;
		m_char = QString::null;
		m_mud = QString::null;
		m_edName->clear();
	}
}

///////////////////////////////////////////////////////////////////////////////

void DlgCreateProfile::slotProfileTypeChanged(bool on)
{
	if (!on) return; // ignore corresponding untoggle

	m_cbMud->setEnabled(m_rbChar->isChecked());
}

///////////////////////////////////////////////////////////////////////////////

void DlgCreateProfile::slotNameChanged(const QString& name)
{
	if (name.stripWhiteSpace().isEmpty() || name.contains("/"))
	{
		m_btnOK->setEnabled(false);	
	}
	else
	{
		m_btnOK->setEnabled(true);	
	}
}

// end of file
