/***************************************************************************
                          dlgoptioncolors.cpp  -  description
                             -------------------
    begin                : Thu Mar 8 2001
    copyright            : (C) 2001 by KMud Development Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dlgoptioncolors.h"

#include <dlgoptioncolorsbase.h>

#include "../kmudconfigcache.h"

#include <qcolor.h>

#include <kcolorbutton.h>

DlgOptionColors::DlgOptionColors(KmudConfigCache *data,QString group,QWidget *parent, const char *name ) :  KmudSetupPane(KmudSetupPane::GLOBAL,group,parent,name)
{
	m_data = data;
	addBaseDialog(new DlgOptionColorsBase(this,name));

	loadColors();
}

DlgOptionColors::~DlgOptionColors()
{
}

void DlgOptionColors::loadColors()
{
	DlgOptionColorsBase *baseDlg = static_cast<DlgOptionColorsBase*>(getBaseDialog());

	baseDlg->cmdForeground->setColor(m_data->defaultTextColor);
	baseDlg->cmdBackground->setColor(m_data->defaultBackgroundColor);
	baseDlg->cmdLocalEchoColor->setColor(m_data->localEchoColor);
	baseDlg->cmdDebugHighlight->setColor(m_data->debugHighlight);

	// Background Colors
	baseDlg->cmdBgBlack->setColor(m_data->backgroundAnsiBlack);
	baseDlg->cmdBgBlue->setColor(m_data->backgroundAnsiBlue);
	baseDlg->cmdBgRed->setColor(m_data->backgroundAnsiRed);
	baseDlg->cmdBgMagenta->setColor(m_data->backgroundAnsiMagenta);
	baseDlg->cmdBgGreen->setColor(m_data->backgroundAnsiGreen);
	baseDlg->cmdBgCyan->setColor(m_data->backgroundAnsiCyan);
	baseDlg->cmdBgYellow->setColor(m_data->backgroundAnsiYellow);
	baseDlg->cmdBgWhite->setColor(m_data->backgroundAnsiWhite);

	// Foreground Colors
	baseDlg->cmdAnsiBlack->setColor(m_data->foregroundAnsiBlack);
	baseDlg->cmdAnsiGray->setColor(m_data->foregroundAnsiGray);
	baseDlg->cmdAnsiDarkRed->setColor(m_data->foregroundAnsiDarkRed);
	baseDlg->cmdAnsiLightRed->setColor(m_data->foregroundAnsiLightRed);
	baseDlg->cmdAnsiDarkGreen->setColor(m_data->foregroundAnsiDarkGreen);
	baseDlg->cmdAnsiLightGreen->setColor(m_data->foregroundAnsiLightGreen);
	baseDlg->cmdAnsiDarkYellow->setColor(m_data->foregroundAnsiDarkYellow);
	baseDlg->cmdAnsiLightYellow->setColor(m_data->foregroundAnsiLightYellow);
	baseDlg->cmdAnsiDarkBlue->setColor(m_data->foregroundAnsiDarkBlue);
	baseDlg->cmdAnsiLightBlue->setColor(m_data->foregroundAnsiLightBlue);
	baseDlg->cmdAnsiDarkMagenta->setColor(m_data->foregroundAnsiDarkMagenta);
	baseDlg->cmdAnsiLightMagenta->setColor(m_data->foregroundAnsiLightMagenta);
	baseDlg->cmdAnsiDarkCyan->setColor(m_data->foregroundAnsiDarkCyan);
	baseDlg->cmdAnsiLightCyan->setColor(m_data->foregroundAnsiLightCyan);
	baseDlg->cmdAnsiDarkWhite->setColor(m_data->foregroundAnsiDarkWhite);
	baseDlg->cmdAnsiLightWhite->setColor(m_data->foregroundAnsiLightWhite);


}


/** This slot is called when the delfault/reset button is pressed */
void DlgOptionColors::slotDefaultPressed()
{
	DlgOptionColorsBase *baseDlg = static_cast<DlgOptionColorsBase*>(getBaseDialog());
	baseDlg->cmdForeground->setColor(QColor(0,255,0));
	baseDlg->cmdBackground->setColor(Qt::black);
	baseDlg->cmdLocalEchoColor->setColor(Qt::red);
	baseDlg->cmdDebugHighlight->setColor(Qt::green);

	// Background Colors
	baseDlg->cmdBgBlack->setColor(Qt::black);
	baseDlg->cmdBgBlue->setColor(QColor(0, 0, 128));
	baseDlg->cmdBgRed->setColor(QColor(128, 0, 0));
	baseDlg->cmdBgMagenta->setColor(QColor(128, 0, 128));
	baseDlg->cmdBgGreen->setColor(QColor(0, 128, 0));
	baseDlg->cmdBgCyan->setColor(QColor(0, 128, 128));
	baseDlg->cmdBgYellow->setColor(QColor(128, 128, 0));
	baseDlg->cmdBgWhite->setColor(QColor(192, 192, 192));

	// Foreground Colors
	baseDlg->cmdAnsiBlack->setColor(Qt::black);
	baseDlg->cmdAnsiGray->setColor(QColor(128, 128, 128));
	baseDlg->cmdAnsiDarkRed->setColor(QColor(128, 0, 0));
	baseDlg->cmdAnsiLightRed->setColor(QColor(255, 0, 0));
	baseDlg->cmdAnsiDarkGreen->setColor(QColor(0, 128, 0));
	baseDlg->cmdAnsiLightGreen->setColor(QColor(0, 255, 0));
	baseDlg->cmdAnsiDarkYellow->setColor(QColor(128, 128, 0));
	baseDlg->cmdAnsiLightYellow->setColor(QColor(255, 255, 0));
	baseDlg->cmdAnsiDarkBlue->setColor(QColor(0, 0, 128));
	baseDlg->cmdAnsiLightBlue->setColor(QColor(0, 0, 255));
	baseDlg->cmdAnsiDarkMagenta->setColor(QColor(128, 0, 128));
	baseDlg->cmdAnsiLightMagenta->setColor(QColor(255, 0, 255));
	baseDlg->cmdAnsiDarkCyan->setColor(QColor(0, 128, 128));
	baseDlg->cmdAnsiLightCyan->setColor(QColor(0, 255, 255));
	baseDlg->cmdAnsiDarkWhite->setColor(QColor(192, 192, 192));
	baseDlg->cmdAnsiLightWhite->setColor(QColor(255, 255, 255));
}

/** This slot is called when the cancel button is pressed */
void DlgOptionColors::slotCancelPressed()
{
}

/** This slot is called when the OK button is pressed */
void DlgOptionColors::slotOkPressed()
{
	DlgOptionColorsBase *baseDlg = static_cast<DlgOptionColorsBase*>(getBaseDialog());
	m_data->defaultTextColor=baseDlg->cmdForeground->color();
	m_data->defaultBackgroundColor=baseDlg->cmdBackground->color();
	m_data->localEchoColor=baseDlg->cmdLocalEchoColor->color();
	m_data->debugHighlight=baseDlg->cmdDebugHighlight->color();

	// Background Colors
	m_data->backgroundAnsiBlack=baseDlg->cmdBgBlack->color();
	m_data->backgroundAnsiBlue=baseDlg->cmdBgBlue->color();
	m_data->backgroundAnsiRed=baseDlg->cmdBgRed->color();
	m_data->backgroundAnsiMagenta=baseDlg->cmdBgMagenta->color();
	m_data->backgroundAnsiGreen=baseDlg->cmdBgGreen->color();
	m_data->backgroundAnsiCyan=baseDlg->cmdBgCyan->color();
	m_data->backgroundAnsiYellow=baseDlg->cmdBgYellow->color();
	m_data->backgroundAnsiWhite=baseDlg->cmdBgWhite->color();

	// Foreground Colors
	m_data->foregroundAnsiBlack=baseDlg->cmdAnsiBlack->color();
	m_data->foregroundAnsiGray=baseDlg->cmdAnsiGray->color();
	m_data->foregroundAnsiDarkRed=baseDlg->cmdAnsiDarkRed->color();
	m_data->foregroundAnsiLightRed=baseDlg->cmdAnsiLightRed->color();
	m_data->foregroundAnsiDarkGreen=baseDlg->cmdAnsiDarkGreen->color();
	m_data->foregroundAnsiLightGreen=baseDlg->cmdAnsiLightGreen->color();
	m_data->foregroundAnsiDarkYellow=baseDlg->cmdAnsiDarkYellow->color();
	m_data->foregroundAnsiLightYellow=baseDlg->cmdAnsiLightYellow->color();
	m_data->foregroundAnsiDarkBlue=baseDlg->cmdAnsiDarkBlue->color();
	m_data->foregroundAnsiLightBlue=baseDlg->cmdAnsiLightBlue->color();
	m_data->foregroundAnsiDarkMagenta=baseDlg->cmdAnsiDarkMagenta->color();
	m_data->foregroundAnsiLightMagenta=baseDlg->cmdAnsiLightMagenta->color();
	m_data->foregroundAnsiDarkCyan=baseDlg->cmdAnsiDarkCyan->color();
	m_data->foregroundAnsiLightCyan=baseDlg->cmdAnsiLightCyan->color();
	m_data->foregroundAnsiDarkWhite=baseDlg->cmdAnsiDarkWhite->color();
	m_data->foregroundAnsiLightWhite=baseDlg->cmdAnsiLightWhite->color();
}

