/***************************************************************************
                          dlgoptionfonts.cpp  -  description
                             -------------------
    begin                : Thu Mar 8 2001
    copyright            : (C) 2001 by KMud Development Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dlgoptionfonts.h"

#include <dlgoptionfontsbase.h>

#include "../kmudconfigcache.h"

#include <kfontdialog.h>
#include <kmuddebug.h>
#include <qpushbutton.h>

DlgOptionFonts::DlgOptionFonts(KmudConfigCache *data,QString group,QWidget *parent, const char *name ) :  KmudSetupPane(KmudSetupPane::GLOBAL,group,parent,name)
{
	m_data = data;
	addBaseDialog(new DlgOptionFontsBase(this,name));

	DlgOptionFontsBase *baseDlg = static_cast<DlgOptionFontsBase*>(getBaseDialog());
    connect( baseDlg->cmdOutputFont, SIGNAL( clicked() ), this, SLOT( slotOutputFontButtonPressed() ) );
    connect( baseDlg->cmdInputFont, SIGNAL( clicked() ), this, SLOT( slotInputFontButtonPressed() ) );

	m_inputFont =  m_data->inputFont;
	m_outputFont = m_data->outputFont;

	baseDlg->cmdOutputFont->setText(m_outputFont.family()+" , " + QString::number(m_outputFont.pointSize()));
	baseDlg->cmdInputFont->setText(m_inputFont.family()+" , " + QString::number(m_inputFont.pointSize()));
}

DlgOptionFonts::~DlgOptionFonts()
{
	

}

/** This slot is called when the delfault/reset button is pressed */
void DlgOptionFonts::slotDefaultPressed()
{
	m_outputFont = QFont("fixed");
	m_outputFont.setPointSize(10);
	m_inputFont = QFont("helvetica");	
	m_inputFont.setPointSize(10);
	DlgOptionFontsBase *baseDlg = static_cast<DlgOptionFontsBase*>(getBaseDialog());
	baseDlg->cmdOutputFont->setText(m_outputFont.family()+" , " + QString::number(m_outputFont.pointSize()));
	baseDlg->cmdInputFont->setText(m_inputFont.family()+" , " + QString::number(m_inputFont.pointSize()));
}

/** This slot is called when the cancel button is pressed */
void DlgOptionFonts::slotCancelPressed()
{
}

/** This slot is called when the OK button is pressed */
void DlgOptionFonts::slotOkPressed()
{
	m_data->inputFont =  m_inputFont;
	m_data->outputFont =  m_outputFont;
}

void DlgOptionFonts::slotInputFontButtonPressed()
{
	KFontDialog d(this,"Choose Input Font",false,true);

	d.setFont(m_inputFont);

	if (d.exec())
	{
		DlgOptionFontsBase *baseDlg = static_cast<DlgOptionFontsBase*>(getBaseDialog());
		m_inputFont = d.font();
		baseDlg->cmdInputFont->setText(m_inputFont.family()+" , " + QString::number(m_inputFont.pointSize()));
	}

}

void DlgOptionFonts::slotOutputFontButtonPressed()
{
	KFontDialog d(this,"Choose Output Font",false,true);

	d.setFont(m_outputFont);

	if (d.exec())
	{
		DlgOptionFontsBase *baseDlg = static_cast<DlgOptionFontsBase*>(getBaseDialog());
		m_outputFont = d.font();
		baseDlg->cmdOutputFont->setText(m_outputFont.family()+" , " + QString::number(m_outputFont.pointSize()));
	}
}
