/***************************************************************************
                               kmudmainwindow.cpp
                             -------------------
    begin                : Sat Mar  3 19:17:48 CET 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for QT
#include <qdir.h>
#include <qpainter.h>
#include <qstringlist.h>
#include <qregexp.h>
#include <qlabel.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>
#include <kdockwidget.h>
#include <kstddirs.h>
#include <kstatusbar.h>
#include <kpopupmenu.h>
#include <kwin.h>

// kmud lib
#include <kmuddockwidget.h>
#include <kmuddebug.h>

// application specific includes
#include "kmudconnecthistory.h"
#include "kmudapplication.h"
#include "kmudcore.h"
#include "kmudmainwindow.h"
#include "kmudview.h"
#include "cmudstream.h"
#include "connectiontime.h"
#include "kmudconfigcache.h"

// dialog includes
#include "dialogs/dlgconnect.h"
#include "dialogs/dlgreconnect.h"
#include "dialogs/dlgconnectionwizard.h"
#include "dialogs/dlgglobalsettings.h"
#include "dialogs/dlgprofilesettings.h"

#define ID_STATUS_MSG          1
#define ID_STATUS_CONNECT_TIME 2

KmudMainWindow::KmudMainWindow(KmudCore* _core, QWidget* , const char* name)
:DockMainWindow(0,name)
{
	kdDebug(DAREA_UI_MAINWINDOW) << "constructor called" << endl;

	core = _core;

	config=kapp->config();

	setXMLFile("kmudui.rc");


	///////////////////////////////////////////////////////////////////
	// call inits to invoke all other construction parts
	initTimer();
	initStatusBar();
	initActions();

	readOptions();

	initView();


	///////////////////////////////////////////////////////////////////
	// default action states at startup
	m_mudOutputIconBlinkOn = false;
	m_mudOutputDoNotify = false;
	mudConnectionWizard->setEnabled(true);
	mudConnect->setEnabled(true);
	m_connectHistory->setEnabled(true);
	mudDisconnect->setEnabled(false);
	profileSettings->setEnabled(false);
	
	connect(core,SIGNAL(connectionStateChanged()),this,SLOT(slotUpdateConnectionActions()));
 	connect(core,SIGNAL(connectionStateChanged()),this,SLOT(slotTimerStateChanged()));
	connect(core,SIGNAL(connectionStateChanged()),this,SLOT(slotReconnect()));
	connect(core->mudStream(),SIGNAL(outputProcessed()),this,SLOT(slotMudOutputReceived()));

	editCut->setEnabled(false);
	editCopy->setEnabled(false);
	editPaste->setEnabled(false);

	kdDebug(DAREA_UI_MAINWINDOW) << "constructor finished" << endl;
}

KmudMainWindow::~KmudMainWindow()
{
	kdDebug(DAREA_UI_MAINWINDOW) << "destructor called" << endl;
	// Make sure that is disable as the applications is about to quit
	m_autoReconnect = false;
	delete m_mudOutputBlinkTimer;
	delete m_mudOutputTimer;
	delete connectTime;
	KmudApplication* kmudApp = static_cast<KmudApplication*>(kapp);
	kmudApp->deleteCore(core);
	kdDebug(DAREA_UI_MAINWINDOW) << "destructor finished" << endl;
}

void KmudMainWindow::initActions()
{
	// Create the mud menu
	new KAction ( i18n("&New Window"), KStdAccel::key( KStdAccel::New ) , this, SLOT (slotMudNew()), actionCollection(),"mudNew");
	mudConnectionWizard = new KAction ( i18n("&Connection Wizard"),BarIcon("kmud_connectionwizard.png"), 0 , this, SLOT (slotMudWizard()), actionCollection(),"mudWizard");
	mudConnect = new KToolBarPopupAction ( i18n("&Quick Connect"),"kmud_quickconnect.png", 0 , this, SLOT (slotMudQuickConnect()), actionCollection(),"mudQuick");
	connect( mudConnect->popupMenu(), SIGNAL( aboutToShow() ), this, SLOT( slotConnectMenuAboutToShow() ) );
	connect( mudConnect->popupMenu(), SIGNAL( activated( int ) ), this, SLOT( slotConnectMenuActivated( int ) ) );
	mudDisconnect =	new KAction ( i18n("&Disconnect"),BarIcon("kmud_disconnect.png"), 0 , this, SLOT (slotMudDisconnect()), actionCollection(),"mudDisconnect");
	new KAction ( i18n("C&lose Window"),BarIcon("fileclose.png"), KStdAccel::key( KStdAccel::Close ) , this, SLOT (slotMudClose()), actionCollection(),"mudClose");
	new KAction ( i18n("Q&uit"),BarIcon("exit.png"),KStdAccel::key( KStdAccel::Quit ) , this, SLOT (slotMudExit()), actionCollection(),"mudExit");

	// Create misc menus
	editCut = new KAction ( i18n("C&ut"), BarIcon("editcut.png"), KStdAccel::key( KStdAccel::Cut ) , this, SLOT (slotEditCut()), actionCollection(), "EditCut");
	editCopy = new KAction (i18n("&Copy"), BarIcon("editcopy.png"), KStdAccel::key( KStdAccel::Copy),  this, SLOT(slotEditCopy()), actionCollection(), "EditCopy");
	editPaste = new KAction (i18n("&Paste"), BarIcon("editpaste.png"), KStdAccel::key(KStdAccel::Paste),  this, SLOT(slotEditPaste()), actionCollection(), "EditPaste");
	viewMudToolBar = new KToggleAction(i18n("Show Mud Toolbar"), 0, this, SLOT(slotViewMudToolBar()), actionCollection(), "ShowMudToolbar");
	viewEditToolBar = new KToggleAction(i18n("Show Edit Toolbar"), 0, this, SLOT(slotViewEditToolBar()), actionCollection(), "ShowEditToolbar");
	viewProfileToolBar = new KToggleAction(i18n("Show Profile Toolbar"), 0, this, SLOT(slotViewProfileToolBar()), actionCollection(), "ShowProfileToolbar");
	viewStatusBar = new KToggleAction(i18n("Show St&atusbar"), 0, this, SLOT(slotViewStatusBar()), actionCollection(), "ShowStatusbar");

	editCut->setStatusText(i18n("Cuts the selected section and puts it to the clipboard"));
	editCopy->setStatusText(i18n("Copies the selected section to the clipboard"));
	editPaste->setStatusText(i18n("Pastes the clipboard contents to actual position"));
	viewMudToolBar->setStatusText(i18n("Enables/disables the mud toolbar"));
	viewEditToolBar->setStatusText(i18n("Enables/disables the edit toolbar"));
	viewProfileToolBar->setStatusText(i18n("Enables/disables the profile toolbar"));
	viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));


	// Create the profile menu    
	profileSettings = new KAction ( i18n("&Profile Settings"), 0, this,SLOT(slotProfileShowConfig()),actionCollection(),"ShowProfileConfigDialog");
	new KAction ( i18n("&Global Settings"), 0, this,SLOT(slotShowConfig()),actionCollection(),"ShowConfigDialog");

	m_connectHistory = new KmudConnectHistory(this,actionCollection());

	createGUI(0L);

	m_autoReconnect = false;
}

void KmudMainWindow::initStatusBar()
{
	///////////////////////////////////////////////////////////////////
	// STATUSBAR

	connectIcon = new QLabel((QWidget *)statusBar());
	connectIcon->setPixmap(SmallIcon("connect_no.png"));

	statusBar()->insertItem(i18n("Ready."), ID_STATUS_MSG,1,true);
	statusBar()->setItemAlignment(ID_STATUS_MSG,AlignLeft);
	statusBar()->addWidget(connectIcon,0,true);

	statusBar()->insertItem("00:00:00",ID_STATUS_CONNECT_TIME,0,true);
}

void KmudMainWindow::initTimer()
{
	connectTime = new ConnectionTime();
	connect(connectTime,SIGNAL(tick()),this,SLOT(slotUpdateTimer()) );

	m_mudOutputTimer = new QTimer(this);
	connect (m_mudOutputTimer, SIGNAL(timeout()),this, SLOT(slotMudOutputTimeout()) );
	m_mudOutputBlinkTimer = new QTimer(this);
	connect (m_mudOutputBlinkTimer, SIGNAL(timeout()),this, SLOT(slotMudOutputBlinker()) );

}

void KmudMainWindow::initView()
{

	////////////////////////////////////////////////////////////////////
	// create the main widget here that is managed by KTMainWindow's view-region and
	// connect the widget to your document to display document contents.

	mainViewDock = createDockWidget("MainViewDock", 0, this, i18n("MUD Window - Kmud"), i18n("MUD View"));
	mainView = new KmudView(mainViewDock,"KmudView");
	mainViewDock->setWidget(mainView);
	mainView->splash();
	setView(mainViewDock);
	setMainDockWidget(mainViewDock);

	// Set initual mainview properties
	KmudConfigCache* cache = core->configCache();
	mainView->init(cache->outputFont,core->configCache()->defaultTextColor,core->configCache()->defaultBackgroundColor);
	mainView->setInputFont(cache->inputFont);
	mainView->setLocalEchoColor(cache->localEchoColor);
	mainView->setLocalEchoMode(cache->localEcho);
	mainView->setInputEchoMode(cache->inputEcho);
	mainView->setSuppressNewLine(cache->suppressNewLine);
	mainView->setKeepLastInput(cache->keepLastInput);
	mainView->setSelectLastInput(cache->selectLastInput);
	mainView->setDualInputLines(cache->dualInput);
	
	mainViewDock->show();

	setCaption("Kmud",false);

}

KmudView* KmudMainWindow::getView()
{
	return mainView;
}


void KmudMainWindow::saveOptions()
{	
	config->setGroup("General Options");
	config->writeEntry("Geometry", size());
	config->writeEntry("Show Mud Toolbar", viewMudToolBar->isChecked());
	config->writeEntry("Show Edit Toolbar", viewEditToolBar->isChecked());
	config->writeEntry("Show Profile Toolbar", viewProfileToolBar->isChecked());
	config->writeEntry("Show Statusbar",viewStatusBar->isChecked());
	config->writeEntry("MudToolBarPos", (int) toolBar("mud")->barPos());
	config->writeEntry("EditToolBarPos", (int) toolBar("edit")->barPos());
	config->writeEntry("ProfileToolBarPos", (int) toolBar("profile")->barPos());

	KmudConnectHistory::saveEntries(config);
	writeDockConfig(config, "DockConfig");

	core->configCache()->save();
}


void KmudMainWindow::readOptions()
{
	KmudConnectHistory::loadEntries(config);
	config->setGroup("General Options");

	// bar status settings

	bool bViewMudToolbar = config->readBoolEntry("Show Edit Toolbar", true);
	viewMudToolBar->setChecked(bViewMudToolbar);
	slotViewMudToolBar();
	
	bool bViewEditToolbar = config->readBoolEntry("Show Edit Toolbar", true);
	viewEditToolBar->setChecked(bViewEditToolbar);
	slotViewEditToolBar();
	
	bool bViewProfileToolbar = config->readBoolEntry("Show Profile Toolbar", true);
	viewProfileToolBar->setChecked(bViewProfileToolbar);
	slotViewProfileToolBar();

	bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
	viewStatusBar->setChecked(bViewStatusbar);
	slotViewStatusBar();

	// bar position settings
	KToolBar::BarPosition EditToolbarPos, ProfileToolbarPos, MudToolbarPos;

	MudToolbarPos=(KToolBar::BarPosition) config->readNumEntry("MudToolBarPos", KToolBar::Top);
	toolBar("mud")->setBarPos(MudToolbarPos);
	
	EditToolbarPos=(KToolBar::BarPosition) config->readNumEntry("EditToolBarPos", KToolBar::Top);
	toolBar("edit")->setBarPos(EditToolbarPos);
	
	ProfileToolbarPos=(KToolBar::BarPosition) config->readNumEntry("ProfileToolBarPos", KToolBar::Top);
	toolBar("profile")->setBarPos(ProfileToolbarPos);

	QSize size=config->readSizeEntry("Geometry");
	if(!size.isEmpty())
	{
		resize(size);
	}	
}

void KmudMainWindow::saveProperties(KConfig *_cfg)
{

}


void KmudMainWindow::readProperties(KConfig* _cfg)
{

}

bool KmudMainWindow::queryClose()
{
	core->setProfile(QString::null);
	return true;
}

bool KmudMainWindow::queryExit()
{
	core->setProfile(QString::null);
	saveOptions();
	return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KmudMainWindow::slotEditCut()
{
  slotStatusMsg(i18n("Cutting selection..."));

  slotStatusMsg(i18n("Ready."));
}

void KmudMainWindow::slotEditCopy()
{
  slotStatusMsg(i18n("Copying selection to clipboard..."));

  slotStatusMsg(i18n("Ready."));
}

void KmudMainWindow::slotEditPaste()
{
  slotStatusMsg(i18n("Inserting clipboard contents..."));

  slotStatusMsg(i18n("Ready."));
}

void KmudMainWindow::slotViewEditToolBar()
{
  slotStatusMsg(i18n("Toggling edit toolbar..."));

  if(!viewEditToolBar->isChecked())
  {
    toolBar("edit")->hide();
  }
  else
  {
    toolBar("edit")->show();
  }

  slotStatusMsg(i18n("Ready."));
}

void KmudMainWindow::slotViewProfileToolBar()
{
	slotStatusMsg(i18n("Toggling profile toolbar ..."));

	if(!viewProfileToolBar->isChecked())
	{
		toolBar("profile")->hide();
	}
	else
	{
		toolBar("profile")->show();
	}

	slotStatusMsg(i18n("Ready."));
}

void KmudMainWindow::slotViewMudToolBar()
{
	slotStatusMsg(i18n("Toggling mud toolbar ..."));

	if(!viewMudToolBar->isChecked())
	{
		toolBar("mud")->hide();
	}
	else
	{
		toolBar("mud")->show();
	}

	slotStatusMsg(i18n("Ready."));
}

void KmudMainWindow::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!viewStatusBar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  slotStatusMsg(i18n("Ready."));
}


void KmudMainWindow::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}


void KmudMainWindow::slotMudNew(void)
{
  slotStatusMsg(i18n("Opening a new application window..."));

  ((KmudApplication*)kapp)->newInstance();
  //KmudMainWindow *new_window= new KmudMainWindow(core);
  //new_window->show();

  slotStatusMsg(i18n("Ready."));
}

void KmudMainWindow::slotMudWizard(void)
{
	ProfileAPI* profileAPI = core ->profileAPI();
	DlgConnectionWizard dlg(profileAPI, this, "Connection Wizard");
	dlg.loadProfileList(core->configAPI()->kmudConfig(), "Selected Profiles");
	QString charProfile;
	QString mudProfile;
	switch (dlg.exec())
	{
		case dlg.CANCEL: // Cancel
			return;
		case dlg.OK: // Connect
			mudProfile = dlg.selectedMudProfile();
			charProfile = dlg.selectedCharProfile();
			dlg.saveProfileList(core->configAPI()->kmudConfig(), "Selected Profiles");
			break;
		case dlg.ACTIVATE: // Activate Profile
			mudProfile = dlg.selectedMudProfile();
			charProfile = dlg.selectedCharProfile();
			dlg.saveProfileList(core->configAPI()->kmudConfig(), "Selected Profiles");
			core->setProfile(charProfile);
			profileSettings->setEnabled(true);
			return;
		default:
			kdDebug(DAREA_UI_MAINWINDOW) << "slotMudWizard: dialog result unknown\n";
			return;
	}
	
	kdDebug(DAREA_UI_MAINWINDOW) << "slotMudWizard: selectedProfile = " << charProfile << endl;

	if (!core->connectToMud(charProfile))
	{
		switch (core->connectionError())
		{
			case KmudAPI::ErrConnectionRefused:
				KMessageBox::error(this,i18n("Connection refused."));
				break;
			case KmudAPI::ErrHostNotFound:
				KMessageBox::error(this,i18n("Host not found."));
				break;
			case KmudAPI::ErrTimeout:
				KMessageBox::error(this,i18n("Connect timed out."));
				break;
			case KmudAPI::ErrUnknown:
				KMessageBox::error(this,i18n("Unknown connection error."));
		}
	}
}

void KmudMainWindow::slotMudQuickConnect(void)
{
	slotStatusMsg(i18n("Connecting to MUD..."));

	DlgConnect d;

	d.setHost(core->getLastHost());
	d.setPort(core->getLastPort());

	if (d.exec())
	{
		if (core->connectToMud(d.getHost()+":"+d.getPort()) == false)
		{
			switch (core->connectionError())
			{
				case KmudAPI::ErrConnectionRefused:
					KMessageBox::error(this,i18n("Connection refused."));
					break;
				case KmudAPI::ErrHostNotFound:
					KMessageBox::error(this,i18n("Host not found."));
					break;
				case KmudAPI::ErrTimeout:
					KMessageBox::error(this,i18n("Connect timed out."));
					break;
				case KmudAPI::ErrUnknown:
					KMessageBox::error(this,i18n("Unknown connection error."));
			}
		}
	}

	slotStatusMsg(i18n("Ready."));
}

void KmudMainWindow::slotMudDisconnect(void)
{
	m_autoReconnect = false;
	slotStatusMsg(i18n("Disconnecting..."));
	core->closeConnection();
	slotStatusMsg(i18n("Ready."));
}

void KmudMainWindow::slotMudClose(void)
{
}

void KmudMainWindow::slotMudExit(void)
{
	slotStatusMsg(i18n("Exiting..."));
	queryExit();
	close();
}

void KmudMainWindow::slotShowConfig(void)
{
	DlgGlobalSettings dlg(core,this);
	if (dlg.exec())
	{
		KmudConfigCache* cache = core->configCache();
		cache->save();
		
		mainView->setFont(cache->outputFont);
		mainView->setBgColor(cache->defaultBackgroundColor);
		mainView->setTextColor(cache->defaultTextColor);
		mainView->setInputFont(cache->inputFont);
		mainView->setLocalEchoColor(cache->localEchoColor);
		mainView->setLocalEchoMode(cache->localEcho);
		mainView->setInputEchoMode(cache->inputEcho);
		mainView->setSuppressNewLine(cache->suppressNewLine);
		mainView->setKeepLastInput(cache->keepLastInput);
		mainView->setSelectLastInput(cache->selectLastInput);
		mainView->setDualInputLines(cache->dualInput);

		// FIXME_jp : Add config option for keep last config
		m_mudOutputTimer->start(cache->outputNotifyWaitSeconds*1000,true);
		m_mudOutputDoNotify = false;
	}
}

void KmudMainWindow::slotProfileShowConfig()
{
	kdDebug(DAREA_UI_MAINWINDOW) << "here 1" << endl;
	DlgProfileSettings dlg(core,this);
	if (dlg.exec())
	{	
	}
}

void KmudMainWindow::slotProfileAliases(void)
{
}

void KmudMainWindow::slotProfileTriggers(void)
{
}

void KmudMainWindow::slotProfileLogging(void)
{
}

void KmudMainWindow::slotProfileToolbar(void)
{
}

void KmudMainWindow::slotUpdateConnectionActions()
{
	switch (core->connectionState())
	{
		case KmudAPI::Disconnected:
					mudConnectionWizard->setEnabled(true);
					mudConnect->setEnabled(true);
					m_connectHistory->setEnabled(true);
					mudDisconnect->setEnabled(false);
					profileSettings->setEnabled(false);
					break;
		case KmudAPI::Connecting:
					mudConnectionWizard->setEnabled(false);
					mudConnect->setEnabled(false);
					m_connectHistory->setEnabled(false);
					mudDisconnect->setEnabled(false);
					profileSettings->setEnabled(false);
					break;
		case KmudAPI::Connected:
					mudConnectionWizard->setEnabled(false);
					mudConnect->setEnabled(false);
					m_connectHistory->setEnabled(false);
					mudDisconnect->setEnabled(true);
					profileSettings->setEnabled(true);
					break;
		default:
			kdDebug(DAREA_UI_MAINWINDOW) << "unknown connection state " <<
				core->connectionState() << endl;
			break;

	}
}

/** This is called every second to update the connection timer */
void KmudMainWindow::slotUpdateTimer()
{
	///////////////////////////////////////////////////////////////////
	// Update the staus bar to show the new amount of time a user
	// has been connected to the mud
	statusBar()->changeItem(connectTime->toString(), ID_STATUS_CONNECT_TIME );
}

void KmudMainWindow::slotTimerStateChanged()
{
	switch (core->connectionState())
	{
		case KmudAPI::Disconnected:
					connectIcon->setPixmap(SmallIcon("connect_no.png"));
					connectTime->stop();
					m_mudOutputTimer->stop();
                    m_mudOutputDoNotify = false;
					m_mudOutputBlinkTimer->stop();
					break;
		case KmudAPI::Connecting:
					connectIcon->setPixmap(SmallIcon("connect_creating.png"));
					break;
		case KmudAPI::Connected:
					connectIcon->setPixmap(SmallIcon("connect_established.png"));
					connectTime->start();
					m_mudOutputTimer->start(core->configCache()->outputNotifyWaitSeconds*1000,true);
					break;
		default: // debug already handled in slotUpdateConnectionActions()
			break;
	}
}

/**
 * This slot will display the auto reconnect dialog if the mud disconnects
 */
void KmudMainWindow::slotReconnect()
{
	switch (core->connectionState())
	{
		case KmudAPI::Disconnected:
			if (m_autoReconnect && core->configCache()->autoReconnect)
			{
				bool quit = false;
				DlgReconnect* d = new DlgReconnect(this,"ReconnectDialog");
				d->setAutoTime(core->configCache()->autoReconnectSeconds);
				int result = d->exec();
				delete d;

				switch (result)
				{
					case 1 : core->reconnect();
					         break;
					case 2 : quit = true;
					         break;
					case 3 : slotMudQuickConnect();
					         break;
					case 4 : slotMudWizard();
					         break;
				}
				if (quit) close();
			}
			break;

		case KmudAPI::Connected:
			m_autoReconnect = true;
			break;

		default:
			break;
	}


}

/** This is called if one of the recent connections are selected */
void KmudMainWindow::slotRecentConnectionSelected( const QString& connection )
{
	m_autoReconnect = false;

	if (core->connectionState()!=KmudAPI::Disconnected)
	{
		core->closeConnection();
	}

	core->connectToMud(connection);

	m_autoReconnect = true;
}

/** This is called when the quick connect popup menu item is selected */
void KmudMainWindow::slotConnectMenuActivated( int i)
{
	core->connectToMud(mudConnect->popupMenu()->text(i));
}

/** This is called before the quick connect popup menu is shown */
void KmudMainWindow::slotConnectMenuAboutToShow()
{          
	QPopupMenu *popup = mudConnect->popupMenu();

	popup->clear();

	QStringList *lstConnections = KmudConnectHistory::getConnections();

	for( unsigned int i = 1 ; i <= lstConnections->count() ; i++ )
	{
		QString value = (*lstConnections)[ i - 1 ];
	    popup->insertItem( value );
	}
}

/** This slot is called to start the icon blinking */
void KmudMainWindow::slotMudOutputTimeout()
{
	m_mudOutputDoNotify = true;
}

void KmudMainWindow::slotMudOutputReceived()
{
	KmudConfigCache* cache = core->configCache();

	if (m_mudOutputDoNotify)
	{
		m_mudOutputBlinkTimer->start(1000,false);
	}

	m_mudOutputDoNotify = false;
	m_mudOutputTimer->start(cache->outputNotifyWaitSeconds*1000,true);
}


/** This slot is called to blink the icon */
void KmudMainWindow::slotMudOutputBlinker()
{
	KmudConfigCache* cache = core->configCache();

	if (cache->outputNotificationOn)
	{
		if (m_mudOutputIconBlinkOn)
			KWin::setIcons(winId(),kapp->icon(),BarIcon("kmud_null"));
		else
			KWin::setIcons(winId(),kapp->icon(),SmallIcon("kmud"));

		m_mudOutputIconBlinkOn=!m_mudOutputIconBlinkOn;

		if (kapp->activeWindow()==this)
		{
			m_mudOutputBlinkTimer->stop();
			m_mudOutputIconBlinkOn=false;
			KWin::setIcons(winId(),kapp->icon(),SmallIcon("kmud"));
		}
	}
}

