/***************************************************************************
                               kmudmainwindow.h
                             -------------------
    begin                : Sat Mar  3 19:17:48 CET 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMUDMAINWINDOW_H
#define KMUDMAINWINDOW_H
 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// include files for Qt

// include files for KDE 
#include <kapp.h>
#include <kparts/dockmainwindow.h>
#include <kparts/partmanager.h>
#include <kaccel.h>
#include <kaction.h>

// include files for Kmud


// class forward declarations
class KDockWidget;

class KmudCore;
class KmudView;
class KmudDockWidget;

class QLabel;
class ConnectionTime;
class KmudConnectHistory;

/**
  * @author Kmud Developer Team
  */

class KmudMainWindow : public KParts::DockMainWindow
{
	Q_OBJECT

public:
	/** construtor of KmudApp, calls all init functions to create the application. */
	KmudMainWindow(KmudCore* _core, QWidget* parent=0, const char* name=0);

	~KmudMainWindow();

	KmudView* getView();

protected:
	/** save general Options like all bar positions and status as well as the geometry and the recent file list to the configuration
	  * file
	  */ 	
	void saveOptions();

	/** read general Options again and initialize all variables like the recent file list */
	void readOptions();

	/** initializes the KActions of the application */
	void initActions();

	/** sets up the statusbar for the main window by initialzing a statuslabel. */
	void initStatusBar();

	/** sets up the timers like the connection timer */
	void initTimer(void);

	/** creates the centerwidget of the KTMainWindow instance and sets it as the view
	  */
	void initView();

	/** queryClose is called by KTMainWindow on each closeEvent of a window. Against the
	  * default implementation (only returns true), this calles saveModified() on the document object to ask if the document shall
	  * be saved if Modified; on cancel the closeEvent is rejected.
	  * @see KTMainWindow#queryClose
	  * @see KTMainWindow#closeEvent
	  */
	bool queryClose();

	/** queryExit is called by KTMainWindow when the last window of the application is going to be closed during the closeEvent().
	  * Against the default implementation that just returns true, this calls saveOptions() to save the settings of the last window's
	  * properties.
	  * @see KTMainWindow#queryExit
	  * @see KTMainWindow#closeEvent
	  */
	bool queryExit();

	/** saves the window properties for each open window during session end to the session config file, including saving the currently
	  * opened file by a temporary filename provided by KApplication.
	  * @see KTMainWindow#saveProperties
	  */
	void saveProperties(KConfig *_cfg);

	/** reads the session config file and restores the application's state including the last opened files and documents by reading the
	  * temporary files saved by saveProperties()
	  * @see KTMainWindow#readProperties
	  */
	void readProperties(KConfig *_cfg);


public slots:
	void slotMudNew(void);
	void slotMudWizard(void);
	void slotMudQuickConnect(void);
	void slotMudDisconnect(void);
	void slotMudClose(void);
	void slotMudExit(void);

	void slotShowConfig(void);
	void slotProfileShowConfig();
	void slotProfileAliases(void);
	void slotProfileTriggers(void);
	void slotProfileLogging(void);	
	void slotProfileToolbar(void);

	/** put the marked text/object into the clipboard and remove
	  *	it from the document
	  */
	void slotEditCut();

	/** put the marked text/object into the clipboard */
	void slotEditCopy();

	/** paste the clipboard into the document */
	void slotEditPaste();

	/** toggles the mud toolbar */
	void slotViewMudToolBar();
	
	/** toggles the edit toolbar */
	void slotViewEditToolBar();

	/** toggles the profile toolbar */
	void slotViewProfileToolBar();
	
	/** toggles the statusbar */
	void slotViewStatusBar();

	/** changes the statusbar contents for the standard label permanently, used to indicate current actions.
	  * @param text the text that is displayed in the statusbar
	  */
	void slotStatusMsg(const QString &text);

	/**
	  * updates the connection actions depending on the connection state
	  */
	void slotUpdateConnectionActions();

private slots:
	void slotMudOutputReceived();
	/** This slot is called to blink the icon */
	void slotMudOutputBlinker();
	/** This slot is called to start the icon blinking */
	void slotMudOutputTimeout();
	/** This is called when the quick connect popup menu item is selected */
	void slotConnectMenuActivated( int i);
	/** This is called before the quick connect popup menu is shown */
	void slotConnectMenuAboutToShow();
	
    /** This is called if one of the recent connections are selected */
	void slotRecentConnectionSelected( const QString& connection );
	/**
	 * This slot will display the auto reconnect dialog if the mud disconnects
	 */
	void slotReconnect();
	/** This is called every second to update the connection timer label */
	void slotUpdateTimer();
	/** This slot is used to tell the connection timer when the connection
          * state has changed */
	void slotTimerStateChanged();
private:
	/** This is used to tell if the icon is blinking */
	bool m_mudOutputDoNotify;
	/** Timer to make icon blink */
	QTimer *m_mudOutputBlinkTimer;
	/** Time limit before icon starts blinking */
	QTimer *m_mudOutputTimer;
	/** Is the null icon or the normal icon being displayed */
	bool m_mudOutputIconBlinkOn;
	/** The icon used to show the connection state */
	QLabel *connectIcon;
	/** The connection time handler */
	ConnectionTime* connectTime;
	/** pointer to the kmud core */
	KmudCore* core;


	/** the configuration object of the application */
	KConfig *config;
	/** view is the main widget which represents your working area. The View
	* class should handle all events of the view widget.  It is kept empty so
	* you can create your view according to your application's needs by
	* changing the view class.
	*/
	KDockWidget* mainViewDock;	
	KmudView *mainView;

	KAction* mudConnectionWizard;
	KToolBarPopupAction* mudConnect;
	KAction* mudDisconnect;

	KAction* editCut;
	KAction* editCopy;
	KAction* editPaste;
	KToggleAction* viewMudToolBar;
	KToggleAction* viewEditToolBar;
	KToggleAction* viewProfileToolBar;
	KToggleAction* viewStatusBar;

	KAction *profileSettings;

	bool m_autoReconnect;

	KmudConnectHistory *m_connectHistory;
};
 
#endif // KMUDMAINWINDOW_H

