/***************************************************************************
                               kmudviewfilter.cpp
                             -------------------
    begin                : Tue Apr 30 2002
    copyright            : (C) 2002 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Kmud includes
#include "cansiparser.h"
#include "kmudviewfilter.h"
#include "kmudview.h"
#include "kmudconfigmanager.h"
#include "kmudconfigcache.h"
#include <kmuddebug.h>

KmudViewFilter::KmudViewFilter(KmudView* view)
	: KmudFilter(), m_view(view), m_lightColorsOn(true)
{
	setFilterName("KmudView");
	m_parser = new CANSIParser();
	m_parser->setInterpreter(this);
	retrieveDefaults();
}

KmudViewFilter::~KmudViewFilter()
{
	delete m_parser;
}

QString KmudViewFilter::filterOutput(QString& input)
{
	kdDebug(DAREA_UI_VIEW) << "KVFilter::filterOutput(" << input << ")\n";
	m_parser->parse(input);
	input = m_buffer;
	m_buffer = QString::null;

	return input;
}

void KmudViewFilter::inputText(const QString& text)
{
	CANSIInterpreter::inputText(text);
	m_buffer.append(text);
	if (m_view != 0)
		m_view->addText(text);
}

void KmudViewFilter::inputTextColor(ANSI::Color color)
{
	CANSIInterpreter::inputTextColor(color);

	if (m_view == 0)
		return;

	KmudConfigCache* cache = KmudConfigManager::self()->configCache();
	switch (color)
	{
		case ANSI::BLACK:
			if (m_lightColorsOn)
				m_view->setTextColor(cache->foregroundAnsiGray);
			else
				m_view->setTextColor(cache->foregroundAnsiBlack);
			break;
		case ANSI::WHITE:
			if (m_lightColorsOn)
				m_view->setTextColor(cache->foregroundAnsiLightWhite);
			else
				m_view->setTextColor(cache->foregroundAnsiDarkWhite);
			break;
		case ANSI::RED:
			if (m_lightColorsOn)
				m_view->setTextColor(cache->foregroundAnsiLightRed);
			else
				m_view->setTextColor(cache->foregroundAnsiDarkRed);
			break;
		case ANSI::GREEN:
			if (m_lightColorsOn)
				m_view->setTextColor(cache->foregroundAnsiLightGreen);
			else
				m_view->setTextColor(cache->foregroundAnsiDarkGreen);
			break;
		case ANSI::BLUE:
			if (m_lightColorsOn)
				m_view->setTextColor(cache->foregroundAnsiLightBlue);
			else
				m_view->setTextColor(cache->foregroundAnsiDarkBlue);
			break;
		case ANSI::YELLOW:
			if (m_lightColorsOn)
				m_view->setTextColor(cache->foregroundAnsiLightYellow);
			else
				m_view->setTextColor(cache->foregroundAnsiDarkYellow);
			break;
		case ANSI::MAGENTA:
			if (m_lightColorsOn)
				m_view->setTextColor(cache->foregroundAnsiLightMagenta);
			else
				m_view->setTextColor(cache->foregroundAnsiDarkMagenta);
			break;
		case ANSI::CYAN:
			if (m_lightColorsOn)
				m_view->setTextColor(cache->foregroundAnsiLightCyan);
			else
				m_view->setTextColor(cache->foregroundAnsiDarkCyan);
			break;
		default:
			break;
	}
}

void KmudViewFilter::inputBackColor(ANSI::Color color)
{
	CANSIInterpreter::inputBackColor(color);
	if (m_view == 0)
		return;

	KmudConfigCache* cache = KmudConfigManager::self()->configCache();
	switch (color)
	{
		case ANSI::BLACK:
			m_view->setBgColor(cache->backgroundAnsiBlack);
			break;
		case ANSI::WHITE:
			m_view->setBgColor(cache->backgroundAnsiWhite);
			break;
		case ANSI::RED:
			m_view->setBgColor(cache->backgroundAnsiRed);
			break;
		case ANSI::GREEN:
			m_view->setBgColor(cache->backgroundAnsiGreen);
			break;
		case ANSI::BLUE:
			m_view->setBgColor(cache->backgroundAnsiBlue);
			break;
		case ANSI::YELLOW:
			m_view->setBgColor(cache->backgroundAnsiYellow);
			break;
		case ANSI::MAGENTA:
			m_view->setBgColor(cache->backgroundAnsiMagenta);
			break;
		case ANSI::CYAN:
			m_view->setBgColor(cache->backgroundAnsiCyan);
			break;
		default:
			break;
	}
}

void KmudViewFilter::inputTextAttribute(ANSI::TextAttr attribute)
{
	CANSIInterpreter::inputTextAttribute(attribute);
	if (m_view == 0)
		return;

	switch (attribute)
	{
		case ANSI::NORMAL:
			m_view->setBold(false);
			m_view->setUnderline(false);
			m_view->setTextColor(m_defaultTextColor);
			m_view->setBgColor(m_defaultBGColor);
			m_view->setBlink(false);
			m_lightColorsOn = true;
			break;
		case ANSI::BOLD:
			m_view->setBold(true);
			break;
		case ANSI::DIM:
			m_lightColorsOn = false;
			break;
		case ANSI::UNDERLINE:
			m_view->setUnderline(true);
			break;
		case ANSI::BLINK:
			m_view->setBlink(true);
			break;
		default:
			break;
	}
}

void KmudViewFilter::retrieveDefaults()
{
	if (m_view != 0)
	{
		m_defaultTextColor = m_view->textColor();
		m_defaultBGColor = m_view->bgColor();
	}
	else
	{
		m_defaultTextColor = Qt::red;
		m_defaultBGColor = Qt::black;
	}
	cache = KmudConfigManager::self()->configCache();
	m_lightColorsOn = true;
}
