/***************************************************************************
                               cmapfilter.cpp
                             -------------------
    begin                : Thu Nov 1 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cmapfilter.h"

#include "cmapmanager.h"
#include "cmapviewbase.h"
#include "cmaproom.h"
#include "cmappath.h"

#include <kmuddebug.h>
#include <kregexp.h>

CMapFilter::CMapFilter(CMapManager *manager)
{	
	mapManager = manager;
	directionCmdQueue.setAutoDelete(false);
	setFilterName("CMapFilter");
}

CMapFilter::~CMapFilter()
{
}

/** This method is called with when input is sent to the mud */
QString CMapFilter::filterInput(QString& s)
{
	QString newStr;

	int start=0;
	int end=s.find('\n',start);

	while (end!=-1)
	{
		QString command = s.mid(start,end-start);

		if (mapManager->validMoveCmd(command))
		{
			bool found = false;

			for (CMapViewBase *view = mapManager->getViewList()->first(); view !=0 ; view = mapManager->getViewList()->next())
			{
				if (view->getFollowMode())
				{
					found = true;
					break;
				}
			}

			if (found)
			{
				executeBeforeCommand(newStr,command);
				newStr += command + "\n";
				executeAfterCommand(newStr,command);

				if (mapManager->getMapData()->validRoomCheck)
				{
					directionCmdQueue.enqueue(new QString (command));
				}
				else
				{
					mapManager->movePlayerBy(command);
				}
			}
			else
			{
				newStr += command + "\n";
			}
		}
		else
		{
			newStr += command + "\n";
		}
		end=s.find('\n',end+1);
	}

	s = newStr;

	return s;
}

void CMapFilter::executeBeforeCommand(QString &newStr,QString &command)
{
	QString specialCmd = "";
	directionTyp dir = mapManager->textToDirection(command);
	if (dir == SPECIAL)
		specialCmd = command;

	CMapPath *path = mapManager->getCurrentRoom()->getPathDirection(dir,specialCmd);

	if (path)
	{
		QString roomCmd = path->getBeforeCommand();

		if (!roomCmd.stripWhiteSpace().isEmpty())
		{
			newStr+=roomCmd + "\n";
		}
	}
}

void CMapFilter::executeAfterCommand(QString &newStr,QString &command)
{

	QString specialCmd = "";
	directionTyp dir = mapManager->textToDirection(command);
	if (dir == SPECIAL)
		specialCmd = command;

	CMapPath *path = mapManager->getCurrentRoom()->getPathDirection(dir,specialCmd);

	if (path)
	{
		QString roomCmd = path->getAfterCommand();

		if (!roomCmd.stripWhiteSpace().isEmpty())
		{
			newStr+=roomCmd + "\n";
		}
	}
}

/** This method is called when output is sent to the mud */
QString CMapFilter::filterOutput(QString& s)
{
	QString *dirCmd=directionCmdQueue.dequeue();

	if ((dirCmd!=NULL))
	{
		bool found = false;

		for (CMapViewBase *view = mapManager->getViewList()->first(); view !=0 ; view = mapManager->getViewList()->next())
		{
			if (view->getFollowMode())
			{
				found = true;
				break;
			}
		}

		if (found)
		{
			bool movePlayer = true;

			for (char *s1 = mapManager->getMapData()->failedMoveMsg.first();
			     s1!=0; s1 = mapManager->getMapData()->failedMoveMsg.next())
			{
				QString str = s1;
				if (str!="")
				{
					KRegExp r(str);
					
					if ( r.match(s))
					{
						movePlayer = false;
						break;
					}
				}

			}

			if (movePlayer)
				mapManager->movePlayerBy(*dirCmd);
		}

		delete dirCmd;
	}

	return s;
}
