/********************************************f*******************************
                          mapmanager.cpp  -  description
                             -------------------
    begin                : Wed Mar 7 2001
    copyright            : (C) 2001 by KMud Development Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmudmapperfactory.h"
#include "cmapmanager.h"

#include <kstddirs.h>
#include <kiconloader.h>
#include <ktrader.h>
#include <kmessagebox.h>
#include <kstdaction.h>
#include <kfiledialog.h>

#include <qtimer.h>
#include <qvaluelist.h>

#include <kmuddockwidget.h>

#include "cmapzone.h"
#include "cmappath.h"
#include "cmaptext.h"
#include "cmaproom.h"
#include "cmapview.h"
#include "cmaplevel.h"
#include "cmapviewbase.h"
#include "cmaptoolbase.h"
#include "cmappluginbase.h"
#include "cmappluginfactory.h"
#include "cmapfilter.h"
#include "cmapclipboard.h"

#include "cmapcmdelementcreate.h"
#include "cmapcmdelementdelete.h"
#include "cmapcmdelementproperties.h"
#include "cmapcmdgroup.h"
#include "cmapcmdmovemap.h"
#include "cmapcmdleveldelete.h"
#include "cmapcmdlevelcreate.h"
#include "cmapcmdtogglepathtwoway.h"
#include "cmapfilefilterbase.h"

#include "filefilters/cmapfilefilterkmudone.h"
#include "filefilters/cmapfilefilterkconfig.h"
#include "filefilters/cmapfilefilterxml.h"

#include "dialogs/dlgmaproomproperties.h"
#include "dialogs/dlgmappathproperties.h"
#include "dialogs/dlgmaptextproperties.h"
#include "dialogs/dlgmapzoneproperties.h"
#include "dialogs/dlgspeedwalkprogress.h"
#include "dialogs/dlgmapinfo.h"
#include "dialogs/dlgmapspeedwalk.h"
#include "dialogs/dlgmapcolor.h"
#include "dialogs/dlgmapdirections.h"
#include "dialogs/dlgmapmovement.h"

#include <cmudstream.h>
#include <kmudapi.h>
#include <kmuddebug.h>
#include <kmudviewmanager.h>
#include <kmemconfig.h>
#include <kmudsetuppane.h>

#include <domconfig.h>

CMapManager::CMapManager(KmudAPI* api)
{
	kdDebug(DAREA_PARTS_MAPMANAGER) << "constructor begins" << endl;
	setName("Mapper Part");
	// setting instance name
	setInstance(KmudMapperFactory::instance());

	setXMLFile (locate("data", "kmudmapper/kmudmapperpart.rc"));

	mapData = new CMapData();
	pAPI = api;

	// Setup vars
	loginRoom = NULL;
	currentRoom = NULL;
	elementEdit = NULL;

	m_elementUtils = new CMapElementUtil(this);

	initMenus();
	initPlugins();
	initFileFilters();


	m_clipboard = new CMapClipboard(this,actionCollection(),"mapClipboard");

	mapViewList.setAutoDelete(false);


	activeView = NULL;

	readOptions();

	enableNonViewActions(false);
	enableViewControls(false);

	speedwalkActive = false;
	pathToWalk.setAutoDelete(true);

	speedwalkProgressDlg = new DlgSpeedwalkProgress(api->mainStatusBar(),"speedwalkProgress");
	speedwalkProgressDlg->hide();
	connect(speedwalkProgressDlg,SIGNAL(abortSpeedwalk()),this,SLOT(slotAbortSpeedwalk()));

	/** Create undo/redo history */
	commandHistory = new KCommandHistory(actionCollection(),true);
	//FIXME_jp: Needs to be configurable
	commandHistory->setUndoLimit(30);
	commandHistory->setRedoLimit(30);
	commandHistory->clear();
	historyGroup = NULL;
	m_commandsActive = true;

	filter = new CMapFilter(this);

	api->mudStream()->registerInputFilter(filter);
	api->mudStream()->registerOutputFilter(filter);

	connect(api, SIGNAL(profileChanged()), this, SLOT(slotProfileChanged()));
	connect(api->profileAPI(), SIGNAL(profileDataChanged(const QString&, uint)),
	        this, SLOT(slotProfileDataChanged(const QString&, uint)));

	
	m_zoneCount = 0;
	m_levelCount = 0;

	kdDebug(DAREA_PARTS_MAPMANAGER) << "constructor ends" << endl;
}

CMapManager::~CMapManager()
{
	kdDebug(DAREA_PARTS_MAPMANAGER) << "CMapManager::~CMapManager() start" << endl;
	if (mapData)
		delete mapData;

	if (commandHistory)
		delete commandHistory;

	delete m_elementUtils;
	
	kdDebug(DAREA_PARTS_MAPMANAGER) << "CMapManager::~CMapManager() views - " << mapViewList.count() << endl;
	
	kdDebug(DAREA_PARTS_MAPMANAGER) << "CMapManager::~CMapManager() end" << endl;
}

/** This method is used to get a list of profile config panes. It should return a empty list
  * If none are provided by this part */
QPtrList<KmudSetupPane> CMapManager::createProfileConfigPanes(QWidget *parent)
{
	QPtrList<KmudSetupPane> panels;

	panels.clear();
	panels.setAutoDelete(false);

	panels.append(new DlgMapDirections(this,i18n("Mud/Mapper"),parent,i18n("Directions")));
	panels.append(new DlgMapMovement(this,i18n("Mud/Mapper"),parent,i18n("Movement")));

	for (CMapPluginBase *plugin = getPluginList()->first(); plugin!=0; plugin = getPluginList()->next())
	{
		QPtrList<KmudSetupPane> pluginPanes = plugin->createProfileConfigPanes(parent);
		for (KmudSetupPane *pane=pluginPanes.first();pane!=0;pane=pluginPanes.next())
		{
			panels.append(pane);
		}
	}

	return panels;
}

/** This method is used to get a list of global config panes. It should return a empty list
  * If none are provided by this part */
QPtrList<KmudSetupPane> CMapManager::createGlobalConfigPanes(QWidget *parent)
{
	QPtrList<KmudSetupPane> panels;

	panels.clear();
	panels.setAutoDelete(false);

	panels.append(new DlgMapColor(this,i18n("Mapper"),parent,i18n("Colors")));
	panels.append(new DlgMapSpeedwalk(this,i18n("Mapper"),parent,i18n("Speedwalk")));

	for (CMapPluginBase *plugin = getPluginList()->first(); plugin!=0; plugin = getPluginList()->next())
	{
		QPtrList<KmudSetupPane> pluginPanes = plugin->createGlobalConfigPanes(parent);
		for (KmudSetupPane *pane=pluginPanes.first();pane!=0;pane=pluginPanes.next())
		{
			panels.append(pane);
		}
	}

	return panels;
}

/** This method tells the plugins that the mapper wants to add a room to the speedwalk list
  * @param room The room to add to the speedwalk list
  */
void CMapManager::addSpeedwalkRoom(CMapRoom *room)
{
	for (CMapPluginBase *plugin=getPluginList()->first();plugin!=0; plugin=getPluginList()->next())
	{
		plugin->addSpeedwalkRoom(room);
	}
}


/** This will setup the import/export file filters */
void CMapManager::initFileFilters()
{
	m_fileFilter.setAutoDelete(true);

	m_fileFilter.append(new CMapFileFilterXML(this));
	m_fileFilter.append(new CMapFileFilterKmudOne(this));
	m_fileFilter.append(new CMapFileFilterKConfig(this));
}

void CMapManager::initMenus()
{
	kdDebug(DAREA_PARTS_MAPMANAGER) << "begisn initMenus" << endl;

	// File menu
	m_fileNew = new KAction ( i18n("New Map"), 0 , this, SLOT (slotFileNew()), actionCollection(),"fileNew");
	m_fileLoad = new KAction ( i18n("Load"), 0 , this, SLOT (slotFileLoad()), actionCollection(),"fileLoad");
	m_fileSave = new KAction ( i18n("Save As"), 0 , this, SLOT (slotFileSave()), actionCollection(),"fileSave");
	m_fileInfo = new KAction ( i18n("Information"), 0 , this, SLOT (slotFileInfo()), actionCollection(),"fileInfo");

	// Tools menu
	m_toolsCreate = new KToggleAction ( i18n("&Create Mode"),BarIcon("kmud_create.png",instance()),0,this,SLOT(slotToolsCreateMode()),actionCollection(),"toolsCreate");
	m_toolsGrid = new KToggleAction ( i18n("&Grid"),BarIcon("kmud_grid.png",instance()),0,this,SLOT(slotToolsGrid()),actionCollection(),"toolsGrid");
	m_toolsUpLevel =  new KAction ( i18n("Display Upper Level"),BarIcon("kmud_lvlup.png",instance()),0,this,SLOT(slotToolsLevelUp()),actionCollection(),"toolsLevelUp");
	m_toolsDownLevel = new KAction ( i18n("Display Lower Level"),BarIcon("kmud_lvldown.png",instance()),0,this,SLOT(slotToolsLevelDown()),actionCollection(),"toolsLevelDown");
	m_toolsDeleteLevel = new KAction ( i18n("Delete Current Level"),0,this,SLOT(slotToolsLevelDelete()),actionCollection(),"toolsLevelDelete");
	m_toolsUpZone = new KAction ( i18n("Display Parent Zone"),BarIcon("kmud_zoneup.png",instance()),0,this,SLOT(slotToolsZoneUp()),actionCollection(),"toolsZoneUp");
	m_toolsZoneProp = new KAction ( i18n("Edit Zone Properties"),0,this,SLOT(slotToolsZoneProperties()),actionCollection(),"toolsZoneProperties");
	m_toolsDeleteZone = new KAction ( i18n("Delete Current zone"),0,this,SLOT(slotToolsDeleteZone()),actionCollection(),"toolsDeleteZone");

	// View menu
	m_viewNewMapView = new KAction ( i18n("Open New Map View"),0,this,SLOT(slotViewNewMap()),actionCollection(), "viewNewMap");	
	m_viewUpperLevel = new KToggleAction ( i18n("Map Upper Level"),0,this,SLOT(slotViewUpperLevel()),actionCollection(), "viewUpperLevel");
	m_viewLowerLevel = new KToggleAction ( i18n("Map Lower Level"),0,this,SLOT(slotViewLowerLevel()),actionCollection(), "viewLowerLevel");
	m_viewToolsToolbar = new KToggleAction ( i18n("Tools Toolbar"),0,this,SLOT(slotViewToolsToolbar()),actionCollection(), "viewToolsToolbar");
    m_viewToolsToolbar->setChecked(true);
	m_viewNavToolbar = new KToggleAction ( i18n("Navigation Toolbar"),0,this,SLOT(slotViewNavToolbar()),actionCollection(), "viewNavToolbar");
	m_viewNavToolbar->setChecked(true);

	// Label Submenu
	zoneMenu = new KSelectAction(i18n("Select Zone"),0,this,SLOT(slotSelectZone()),actionCollection(),"toolsZoneList");

	// Room Popup Actions
	new KAction(i18n("Set &Current Position"),0, this,SLOT(slotRoomSetCurrentPos()), actionCollection(),"roomCurrentPos");
	new KAction(i18n("Set Room to &Login Point"),0, this,SLOT(slotRoomSetLogin()),actionCollection() , "roomLoginPoint");
	new KAction(i18n("&Speed walk to room"),0, this,SLOT(slotRoomSpeedwalkTo()), actionCollection(), "roomWalkTo");
	new KAction(i18n("&Delete room"),0, this,SLOT(slotRoomDelete()), actionCollection(),"roomDelete");
	new KAction(i18n("&Properties"),0, this,SLOT(slotRoomProperties()),actionCollection(),"roomProperties");

    // Text Popup Actions
	new KAction(i18n("&Delete Text"),0, this,SLOT(slotTextDelete()), actionCollection(),"textDelete");
	new KAction(i18n("&Properties"),0, this,SLOT(slotTextProperties()), actionCollection(),"textProperties");

	// Zone Popup Actions
	new KAction(i18n("&Open Zone"),0, this,SLOT(slotZoneOpen()), actionCollection(),"zoneOpen");
	new KAction(i18n("&Open Zone in New View"),0, this,SLOT(slotZoneOpenNewView()),actionCollection(),"zoneOpenNewView");
	new KAction(i18n("&Delete Zone"),0, this,SLOT(slotZoneDelete()), actionCollection(),"zoneDelete");
	new KAction(i18n("&Properties"),0, this,SLOT(slotZoneProperties()), actionCollection(),"zoneProperties");

	// Path Popup Actions
	new KToggleAction(i18n("&One way"),0, this,SLOT(slotPathOneWay()), actionCollection(),"pathOneWay");
	new KToggleAction(i18n("&Two way"),0, this,SLOT(slotPathTwoWay()), actionCollection(),"pathTwoWay");
	new KAction(i18n("&Add Bend"),0, this,SLOT(slotPathAddBend()), actionCollection(),"pathAddBend");
	new KAction(i18n("&Remove Segment"),0, this,SLOT(slotPathDelBend()), actionCollection(),"pathDelBend");
	new KAction(i18n("&Edit Bends"),0, this,SLOT(slotPathEditBends()), actionCollection(),"pathEditBends");
	new KAction(i18n("&Delete Path"),0, this,SLOT(slotPathDelete()), actionCollection(),"pathDelete");
	new KAction(i18n("&Properties"),0, this,SLOT(slotPathProperties()), actionCollection(),"pathPorperties");


	QStringList labelPos;
	labelPos.append(i18n("Hide"));
	labelPos.append(directionToText(NORTH,""));
	labelPos.append(directionToText(NORTHEAST,""));
	labelPos.append(directionToText(EAST,""));
	labelPos.append(directionToText(SOUTHEAST,""));
	labelPos.append(directionToText(SOUTH,""));
	labelPos.append(directionToText(SOUTHWEST,""));
	labelPos.append(directionToText(WEST,""));
	labelPos.append(directionToText(NORTHWEST,""));
	labelPos.append(i18n("Custom"));
	
	labelMenu = new KSelectAction(i18n("&Label"),0,this,SLOT(slotChangeLabelPos()),actionCollection(),"labelMenu");
	labelMenu->setItems(labelPos);
}

/** Used to create the plugins */
void CMapManager::initPlugins()
{
	int pluginCount = 0;
	toolList.setAutoDelete(false);
	toolList.clear();
	pluginList.setAutoDelete(false);
	pluginList.clear();
	kdDebug(DAREA_PARTS_MAPMANAGER) << "Loading Plugins...\n";

	KTrader::OfferList offers = KTrader::self()->query("Kmud/Mapper/Plugin");
	kdDebug(DAREA_PARTS_MAPMANAGER) << "got " << offers.count() << " offers for plugins\n";

	KLibLoader* loader = KLibLoader::self();
	KLibFactory* factory;
	CMapPluginFactory* pluginFactory;
	CMapPluginBase* plugin;

	for (unsigned int i = 0; i < offers.count(); ++i)
	{
		kdDebug(DAREA_PARTS_MAPMANAGER) << "Processing offer "
			<< (i+1)
			<< ": \"" << offers[i]->name()
			<< "\" (" << offers[i]->library() << ")\n";

		factory = loader->factory(offers[i]->library());
		if (!factory)
		{
			kdWarning(DAREA_PARTS_MAPMANAGER) << "Obtaining factory failed!\nKLibLoader's error: "
				<< loader->lastErrorMessage() << endl;
		}
		else if (!factory->inherits("CMapPluginFactory"))
		{
			kdWarning(DAREA_PARTS_MAPMANAGER) << "Obtained factory is NOT a subclass of CMapPluginFactory\n";
		}
		else
		{
			kdDebug(DAREA_PARTS_MAPMANAGER) << "Obtained factory, creating tool\n";
			pluginFactory = (CMapPluginFactory*) factory;
			plugin = pluginFactory->createPlugin(this, this);

			if (!plugin)
			{
				kdWarning(DAREA_PARTS_MAPMANAGER) << "Plugin creation failed!";
			}
			else
			{
				kdDebug(DAREA_PARTS_MAPMANAGER) << "Add Plugin : " << plugin->name() << endl;
				pluginList.append(plugin);
		
				kdDebug(DAREA_PARTS_MAPMANAGER) << "Tools in plugin : " << plugin->getToolList()->count() << endl;
					for (CMapToolBase *tool =plugin->getToolList()-> first();tool!=0; tool= plugin->getToolList()->next())
				{
					kdDebug(DAREA_PARTS_MAPMANAGER) << "Add Tool : " << tool->name() << endl;
					toolList.append(tool);
				}

				insertChildClient(plugin);
				
				pluginCount++;
			}
		}
	}

	kdDebug(DAREA_PARTS_MAPMANAGER) << "Finished loading " << pluginCount << " plugins\n";
	kdDebug(DAREA_PARTS_MAPMANAGER) << "Finished loading " << toolList.count() << " tools\n";
	
	if (toolList.count() > 0)
	{

		currentTool = toolList.first();
		currentTool->setChecked(true);
	}
	else
	{
		kdWarning(DAREA_PARTS_MAPMANAGER) << "No tools loaded!\n";
		currentTool = NULL;
	}

    if (pluginCount==0)
    {
		kdWarning(DAREA_PARTS_MAPMANAGER) << "No plugins loaded!\n";
    }

   	kdDebug(DAREA_PARTS_MAPMANAGER) << "XML File : " << xmlFile() << endl; 
}

/** This is used ot get a pointer to the map view list
  * @return Pointer to map view list */
QPtrList<CMapViewBase> *CMapManager::getViewList(void)
{
	return &mapViewList;
}

void CMapManager::updateZoneListCombo(void)
{
	QStringList lst;

	for (CMapZone *zone = mapData->getFirstZone(); zone!=0; zone = mapData->getNextZone())
	{
		lst.append(zone->getLabel());
	}

	zoneMenu->clear();
	zoneMenu->setItems(lst);

	if (activeView)
	{
		CMapZone *zone = activeView->getCurrentlyViewedZone();

		if (zone)
		{
			zoneMenu->setCurrentItem(lst.findIndex(zone->getLabel()));
		}
	}
}

/** Used to get a list of the plugins */
QPtrList<CMapPluginBase> *CMapManager::getPluginList()
{
	return &pluginList;
}

/** Used to get a pointer to the map data */
CMapData *CMapManager::getMapData()
{
	return mapData;
}

/** Used to create a new view of the map */
KmudDockWidget* CMapManager::openMapView()
{
	CMapView *mapView = new CMapView(this,NULL,"mapView");
	mapViewList.append(mapView);

	if (loginRoom)
		mapView->showPosition(QPoint(loginRoom->getX(),loginRoom->getY()),loginRoom->getLevel());
	else
	{
		CMapRoom *firstRoom = findFirstRoom(NULL);
		if (firstRoom)
		{
			((CMapViewBase *)mapView)->showPosition(firstRoom->getLevel());
		}
	}
	enableViewControls(true);
	setActiveView(mapView);
	return (KmudDockWidget *)mapView;
}

/** Used to create a new view of the map */
void CMapManager::openNewMapView(CMapLevel *level)
{
	CMapView *mapView = new CMapView(this,NULL,"mapView");
	mapViewList.append(mapView);

	((CMapViewBase *)mapView)->showPosition(level);

	enableViewControls(true);
	setActiveView(mapView);
	pAPI->viewManager()->addWidget(mapView,"Kmud/Mapper",KmudViewManager::DockRight);
}

/** This method is used to covert cords so that they snap to the grid */
QPoint CMapManager::cordsSnapGrid(QPoint oldPos)
{
	QPoint newPos;

	int oldx = (int)(oldPos.x() / mapData->gridSize.width());
	int oldy =  (int)(oldPos.y() / mapData->gridSize.height());
	newPos.setX( oldx * mapData->gridSize.width());
	newPos.setY( oldy * mapData->gridSize.height());

	return newPos;
}

/**
 * Used to create a new view of the a given level and
 * center the view on the given position.
 */
void CMapManager::openNewMapView(QPoint pos,CMapLevel *level)
{
	CMapView *mapView = new CMapView(this,NULL,"mapView");
	mapViewList.append(mapView);

		((CMapViewBase *)mapView)->showPosition(pos,level);

	enableViewControls(true);
	setActiveView(mapView);
	pAPI->viewManager()->addWidget(mapView,"Kmud/Mapper",KmudViewManager::DockRight);
}

/** This is used to a view to the list of views that should be processed */
void CMapManager::addView(CMapViewBase *view)
{
	mapViewList.append(view);
}

void CMapManager::viewChanged(void)
{
	for (CMapViewBase *view =mapViewList.first(); view!=0;view=mapViewList.next())
	{
		view->viewChanged(getActiveView()->getCurrentlyViewedLevel());
	}

	if (getCurrentTool())
		getCurrentTool()->viewChangedEvent(getActiveView());
}

/** Used to set properties of the view widget */
void CMapManager::setPropertiesAllViews(QCursor *cursor,bool mouseTracking)
{	
	for (CMapViewBase *view=mapViewList.first(); view!=0;view=mapViewList.next())
	{
		view->setCursor(*cursor);
		view->setMouseTracking(mouseTracking);
	}
}

/** Used to get the first view that can be selected
  * @return The first selectedable view that is open if none are open return NULL */
CMapViewBase *CMapManager::getFirstActivableView(void)
{
	CMapViewBase *result = NULL;
	for (CMapViewBase *view=mapViewList.first();view!=0; view=mapViewList.next())
	{
		if (view->acceptFocus())
			return view;
	}

	return result;
}

/**
 * Used to enable/disable the view actions
 * @param If true then enable the actions otherwise disable the actions
 */
void CMapManager::enableViewControls(bool enabled)
{
	m_clipboard->enableActions(enabled);

	m_toolsUpLevel->setEnabled(enabled);
	m_toolsDownLevel->setEnabled(enabled);
	m_toolsDeleteLevel->setEnabled(enabled);
	m_toolsUpZone->setEnabled(enabled);
	m_toolsZoneProp->setEnabled(enabled);
	m_toolsDeleteZone->setEnabled(enabled);
	
	m_fileNew->setEnabled(enabled);
	m_fileSave->setEnabled(enabled);
	m_fileLoad->setEnabled(enabled);
}

/**
 * This method is used to disable/enable mapper actions that are not done by enableViewControls()
 * @param If true then enable the actions otherwise disable the actions
 */
void CMapManager::enableNonViewActions(bool enabled)
{
	m_fileInfo->setEnabled(enabled);
	m_viewNewMapView->setEnabled(enabled);

	m_toolsCreate->setEnabled(enabled);
	m_toolsGrid->setEnabled(enabled);
}

/** Used to delete a view of the map */
void CMapManager::closeMapView(CMapViewBase *mapView)
{
	mapViewList.remove(mapView);

	if (getActiveView()==mapView);
	{
		CMapViewBase *tempView = getFirstActivableView();
		if (mapView!=tempView && tempView!=NULL )
		{
			setActiveView(getFirstActivableView());
		}
		else
		{
			bool found = false;
			for (CMapViewBase *view = getFirstActivableView();
				 view!=0;
                 view = mapViewList.next())
			{
				if (view->acceptFocus())
				{
					setActiveView(view);
					found = true;
					break;
				}
			}

			if (!found)
			{
				enableViewControls(false);
				activeView = NULL;
			}
		}
	}
}

/** Used to unselect all the elements in a level */
void CMapManager::unselectElements(CMapLevel *level)
{
	for (CMapElement *element = level->getFirstElement() ; element!=0; element=level->getNextElement())
	{
		element->setSelected(false);
		element->setEditMode(false);
	}	
}

/** Used to convert a text direction to a direction type */
QString CMapManager::directionToText(directionTyp dir,QString specialCmd)
{
	if ((uint)dir < NUM_DIRECTIONS)
	{
		return mapData->directions[dir];
	}
	else
	{
		return specialCmd;
	}
}

/** Used to convert a direction type to a text direction */
directionTyp CMapManager::textToDirection(QString text)
{
	directionTyp dir = SPECIAL;	

	for (uint i = 0; i < NUM_DIRECTIONS; ++i)
	{
		if (text == mapData->directions[i])
		{
			if (i>9)
			{
				dir = (directionTyp)(i-10);
			}
			else
			{
				dir =  (directionTyp)i;
			}
			break;
		}
	}
	
	return dir;
}

/** Used to set the login room */
void CMapManager::setLoginRoom(CMapRoom *room)
{
	openCommandGroup(i18n("Change Login Room"));
	if (loginRoom)
	{
		CMapCmdElementProperties *cmdRemove = new CMapCmdElementProperties(this,i18n("Remove Login Room Status"),loginRoom);
		cmdRemove->getOrgProperties()->writeEntry("Login",true);
		cmdRemove->getNewProperties()->writeEntry("Login",false);

		addCommand(cmdRemove);
	}

	CMapCmdElementProperties *cmdSet = new CMapCmdElementProperties(this,i18n("Set Login Room Status"),room);

	cmdSet->getOrgProperties()->writeEntry("Login",false);
	cmdSet->getNewProperties()->writeEntry("Login",true);

	addCommand(cmdSet);

	closeCommandGroup();
}

/** Uesd to return the login room */
CMapRoom *CMapManager::getLoginRoom()
{
	return loginRoom;
}

/** Should only be called by CMapRoom.setCurrentRoom() */
void CMapManager::setCurrentRoomWithoutUndo(CMapRoom *room)
{
	currentRoom = room;
}

/** Should only be called by CMapRoom.setLoginRoom() */
void CMapManager::setLoginRoomWithoutUndo(CMapRoom *room)
{
	loginRoom = room;
}


/** Used to set the current room */
void CMapManager::setCurrentRoom(CMapRoom *room)
{
	openCommandGroup(i18n("Change Current Room"));

	CMapCmdElementProperties *cmdRemove = new CMapCmdElementProperties(this,i18n("Remove Current Room Status"),currentRoom);
	cmdRemove->getOrgProperties()->writeEntry("Current",true);
	cmdRemove->getNewProperties()->writeEntry("Current",false);
	addCommand(cmdRemove);

	CMapCmdElementProperties *cmdSet = new CMapCmdElementProperties(this,i18n("Set Current Room Status"),room);
	cmdSet->getOrgProperties()->writeEntry("Current",false);
	cmdSet->getNewProperties()->writeEntry("Current",true);
	addCommand(cmdSet);

	closeCommandGroup();
}

/** Uesd to return the current room */
CMapRoom *CMapManager::getCurrentRoom()
{
	return currentRoom;
}

/** Used to find any elements at the given location
  * @param pos The location to look for the element
  * @param level The level to look for the element on
  * @return NULL if no element was found otherwise a pointer to the element */
CMapElement *CMapManager::findElementAt(QPoint pos,CMapLevel *level)
{
	if (!level)
		return NULL;

	for ( CMapRoom *room=level->getRoomList()->first(); room != 0; room=level->getRoomList()->next() )
	{
		if (room->mouseInElement(pos,level->getZone()))
		{
			return (CMapElement *) room;
		}
	}

	for ( CMapZone *zone=level->getZoneList()->first(); zone != 0; zone=level->getZoneList()->next() )
	{
		if (zone->mouseInElement(pos,level->getZone()))
		{
			return (CMapElement *)zone;
		}
	}

	for ( CMapText *text=level->getTextList()->first(); text != 0; text = level->getTextList()->next())
	{
		if (text->mouseInElement(pos,level->getZone()))
		{
			return (CMapElement *)text;
		}
	}

	return NULL;
}

/** This method is used to find a element from a list of properties
  * @param properties The list of proerties
  * @return The element if it's found otherwise NULL */
CMapElement *CMapManager::findElement(KMemConfig *properties)
{
	CMapElement *result = NULL;

	elementTyp type = (elementTyp)properties->readUnsignedNumEntry("Type",(uint)OTHER);

	if (type!=OTHER)
	{
		if (type==PATH)
		{
			CMapLevel *srcLevel = findLevel(properties->readNumEntry("SrcLevel",-1));
			CMapRoom *srcRoom = srcLevel->findRoom(properties->readNumEntry("SrcRoom",-1));
			directionTyp srcDir = (directionTyp)properties->readNumEntry("SrcDir",0);

			QString specialCommand = properties->readEntry("SpecialCmdSrc","");
			result = srcRoom->getPathDirection(srcDir,specialCommand);

		}
		else
		{
			CMapLevel *level = findLevel(properties->readNumEntry("Level",-5));
			if (level)
			{
				if (type==ROOM)
				{
					result = level->findRoom(properties->readNumEntry("RoomID",-5));
				}
				else if (type==ZONE)
				{
					result = findZone(properties->readNumEntry("ZoneID",-5));
				}
				else
				{
					int x = properties->readNumEntry("X",-5);
					int y = properties->readNumEntry("Y",-5);

					for ( CMapText *text=level->getTextList()->first(); text != 0; text = level->getTextList()->next())
					{
						if (text->getX()==x && text->getY()==y)
						{
							result = text;
							break;
						}
					}
				}
			}
		}
	}

	return result;
}

/** Used to find any rooms at the given location
  * @param pos The location to look for the room
  * @param level The level to look for the room on
  * @return NULL if no room was found otherwise a pointer to the room */
CMapRoom *CMapManager::findRoomAt(QPoint pos,CMapLevel *level)
{
	if (!level)
		return NULL;

	for ( CMapRoom *room=level->getRoomList()->first(); room != 0; room=level->getRoomList()->next() )
	{
		if (room->mouseInElement(pos,level->getZone()))
		{
			return room;
		}
	}	

	return NULL;
}

/** Used to erase the map. This will erase all elements and can't be undone */
void CMapManager::eraseMap(void)
{
	eraseZone(mapData->rootZone);
	delete mapData->rootZone;
	mapData->rootZone = NULL;

	m_zoneCount = 0;
	m_levelCount = 0;

	for (CMapViewBase *view = getViewList()->first(); view !=0; view = getViewList()->next())
	{
		view->setLevel(NULL);
	}

	for (CMapPluginBase *plugin = getPluginList()->first(); plugin!=0; plugin = getPluginList()->next())
	{
		plugin->mapErased();
	}

	loginRoom = NULL;
	currentRoom = NULL;
	elementEdit = NULL;
}

void CMapManager::eraseZone(CMapZone *zone)
{
	if (zone == 0)
		return;
	
	for ( CMapLevel *level=zone->getLevels()->first(); level != 0; level=zone->getLevels()->next())
	{
		for ( CMapRoom *room=level->getRoomList()->first(); room != 0; room=level->getRoomList()->next() )
		{				
			room->getPathList()->clear();
			room->getConnectingPathList()->clear();
		}
		level->getRoomList()->clear();
		level->getTextList()->clear();
		for ( CMapZone* subZone=level->getZoneList()->first(); subZone != 0; subZone=level->getZoneList()->next() )
		{
			eraseZone(subZone);
		}
		level->getZoneList()->clear();
	}
	zone->getLevels()->clear();
}

/** Create new bottom or top level depending on the given direction */
CMapLevel *CMapManager::createLevel(directionTyp dir,CMapZone *intoZone)
{
	if (intoZone == NULL)
	{
		kdDebug(DAREA_PARTS_MAPMANAGER) << "CMapManager::createLevel : Unable to create level, intoZone == NULL" << endl;		
		return NULL;	
	}

	CMapLevel *result = NULL;

	CMapCmdLevelCreate *cmd = NULL;

	if (getUndoActive())
	{
		if (dir==UP)
		{		
			cmd = new CMapCmdLevelCreate(this,i18n("Create Level"),intoZone,intoZone->getLevels()->count());
		}
		else
		{
			cmd = new CMapCmdLevelCreate(this,i18n("Create Level"),intoZone,0);
		}

		addCommand(cmd);
		
		result = cmd->getLevel();
	}
	else
	{
		if (dir==UP)
		{
			result = m_elementUtils->createLevel(intoZone->getLevels()->count(),intoZone);
		}
		else
		{
			result = m_elementUtils->createLevel(0,intoZone);
		}
	}
	
	return result;
}

/** This is used to find a level with a given id
  * @param id The id of the level to find
  * @return Null if no level is found otherwise a pointer to the level */
CMapLevel *CMapManager::findLevel(unsigned int id)
{
	for (CMapZone *zone = getMapData()->getFirstZone();zone!=0;zone = getMapData()->getNextZone())
	{
		for (CMapLevel *level = zone->getLevels()->first();level!=0;level = zone->getLevels()->next())
		{
			
			if (level->getLevelID()==id)
				return level;
		}
	}

	return NULL;
}



/** This is used to find a zone with a given id
  * @param id The id of the zone to find
  * @return Null if no zone is found otherwise a pointer to the zone */
CMapZone *CMapManager::findZone(unsigned int id)
{	
	for (CMapZone *zone = getMapData()->getFirstZone();zone!=0;zone = getMapData()->getNextZone())
	{		
		if (zone->getZoneID()==id)
			return zone;
	}
	
	return NULL;
}

void CMapManager::listZones(CMapZone *zone)
{	
	for (CMapLevel *level = zone->getLevels()->first(); level!=0; level = zone->getLevels()->next())
	{
		for (CMapZone *z = level->getZoneList()->first(); z !=0; z = level->getZoneList()->next())
		{
			listZones(z);
			
		}
	}
}


/** Create new map */
void CMapManager::createNewMap()
{
	// Create the root zone
	getMapData()->rootZone = NULL;	


	CMapZone *zone = createZone(QPoint(-1,-1),NULL);

	// Create a empty room in the first level of the new zone
	CMapRoom *room = createRoom(QPoint(2 * mapData->gridSize.width(),2 * mapData->gridSize.height()),zone->getLevels()->first());
	setCurrentRoomWithoutUndo(room);
	setLoginRoomWithoutUndo(room);

	for (CMapViewBase *view=mapViewList.first();view!=0; view=mapViewList.next())
	{
		view->showPosition(currentRoom->getLowPos(),zone->getLevels()->first());
	}
	updateZoneListCombo();

	for (CMapViewBase *view = getViewList()->first(); view!=0; view=getViewList()->next())
	{
		if (view->getCurrentlyViewedLevel()==NULL)	
		{
			view->showPosition(loginRoom,true);
		}
	}

	for (CMapPluginBase *plugin = getPluginList()->first(); plugin!=0; plugin = getPluginList()->next())
	{
		plugin->newMapCreated();
	}

}

/** Used to create a new room that can be undone/redone
   * @param pos The position to create the room
   * @param level The level to create the room in
   * @return A pointer to the newly created room
   */
CMapRoom *CMapManager::createRoom(QPoint pos,CMapLevel *level)
{
	CMapRoom *result = NULL;

	if (getUndoActive())
	{
		KMemConfig properties;
		properties.setGroup("Properties");
		properties.writeEntry("Type",(int)ROOM);
		properties.writeEntry("X",pos.x());
		properties.writeEntry("Y",pos.y());
		properties.writeEntry("Level",level->getLevelID());
		CMapCmdElementCreate *command = new CMapCmdElementCreate(this,i18n("Create Room"));
		command->addElement(&properties);
		addCommand(command);
		QPtrList<CMapElement> *elements=command->getElements();

	
		for (CMapElement *el = elements->first(); el!=0; el = elements->next())
		{
			if (el->getElementType()==ROOM)
			{
				result = (CMapRoom *)el;
			}
		}
	}
	else
	{
		result = m_elementUtils->createRoom(pos,level);
	}

	return result;
}

/** Used to create a new zone */
CMapZone *CMapManager::createZone(QPoint pos,CMapLevel *level,bool levelCreate)
{
	// FIXME_jp : Allow this to call lowlevel mapper methods when undo is not active
	CMapZone *result = NULL;

	openCommandGroup(i18n("Create Zone"));

	KMemConfig properties;
	properties.setGroup("Properties");
	properties.writeEntry("Type",(int)ZONE);
	properties.writeEntry("X",pos.x());
	properties.writeEntry("Y",pos.y());

	if (level)
	{
		properties.writeEntry("Level",level->getLevelID());
	}

	CMapCmdElementCreate *command = new CMapCmdElementCreate(this,i18n("Create Zone"));
	command->addElement(&properties);

	addCommand(command);	

	QPtrList<CMapElement> *elements=command->getElements();
	
	for (CMapElement *el = elements->first(); el!=0; el = elements->next())
	{
		if (el->getElementType()==ZONE)
		{
			result = (CMapZone *)el;
		}
	}

	if (levelCreate)
	{
		createLevel(UP,result);
	}
	closeCommandGroup();

	return result;
}

/** Used to create a new text label */
CMapText *CMapManager::createText(QPoint pos,CMapLevel *level,QString str)
{
	CMapText *result = NULL;

	if (getUndoActive())
	{
		KMemConfig properties;
		properties.setGroup("Properties");
		properties.writeEntry("Type",(int)TEXT);
		properties.writeEntry("X",pos.x());
		properties.writeEntry("Y",pos.y());

		if (level)
		{
			properties.writeEntry("Level",level->getLevelID());
		}
		
		properties.writeEntry("Text",str);

		CMapCmdElementCreate *command = new CMapCmdElementCreate(this,i18n("Create Text"));
		command->addElement(&properties);
		addCommand(command);
		QPtrList<CMapElement> *elements=command->getElements();
	
		for (CMapElement *el = elements->first(); el!=0; el = elements->next())
		{
			if (el->getElementType()==TEXT)
			{
				result = (CMapText *)el;
			}
		}
	}
	else
	{
		result = m_elementUtils->createText(pos,level,str);
	}

	return result;
}

/** Used to create a new text label */
CMapText *CMapManager::createText(QPoint pos,CMapLevel *level,QString str,QFont font,QColor col)
{
	CMapText *result = NULL;

	if (getUndoActive())
	{
		KMemConfig properties;
		properties.setGroup("Properties");
		properties.writeEntry("Type",(int)TEXT);
		properties.writeEntry("X",pos.x());
		properties.writeEntry("Y",pos.y());
		if (level)
		{
			properties.writeEntry("Level",level->getLevelID());
		}

		properties.writeEntry("Text",str);
		properties.writeEntry("Font",font);
		properties.writeEntry("Color",col);

		CMapCmdElementCreate *command = new CMapCmdElementCreate(this,i18n("Create Text"));
		command->addElement(&properties);
		addCommand(command);
		QPtrList<CMapElement> *elements=command->getElements();
	
		for (CMapElement *el = elements->first(); el!=0; el = elements->next())
		{
			if (el->getElementType()==TEXT)
			{
				result = (CMapText *)el;
			}
		}
	}
	else
	{
		result = m_elementUtils->createText(pos,level,str,font,col);
	}

	return result;
}


/** Used to create a new path*/
CMapPath *CMapManager::createPath(QPoint srcPos,CMapLevel *srcLevel,directionTyp srcDir,
                                  QPoint destPos,CMapLevel *destLevel,directionTyp destDir)
{
	CMapRoom *room=NULL;
	CMapRoom *srcRoom=NULL;
	CMapRoom *destRoom=NULL;

	if (!srcLevel || !destLevel)
		return NULL;

	for ( room=srcLevel->getRoomList()->first(); room != 0; room=srcLevel->getRoomList()->next() )
	{
		if (room->getLowPos() == srcPos)
		{
			srcRoom = room;
			break;
		}
	}

	for ( room=destLevel->getRoomList()->first(); room != 0; room=destLevel->getRoomList()->next() )
	{
		if (room->getLowPos()  == destPos)
		{
			destRoom = room;
 			break;
		}
	}

	return createPath(srcRoom,srcDir,destRoom,destDir);
}

CMapPath *CMapManager::createPath(CMapRoom *srcRoom,CMapRoom *destRoom)
{
	openCommandGroup(i18n("Create Path"));
	
	CMapPath *result = NULL;

	KMemConfig properties;
	properties.setGroup("Properties");
	DlgMapPathProperties d(this,&properties,false);

	if (d.exec())
	{
		properties.printDebugOutput("Properties");
		kdDebug(DAREA_PARTS_MAPMANAGER) << "createPath 1" << endl;
		
			
		directionTyp srcDir = (directionTyp)properties.readNumEntry("SrcDir",0);
		directionTyp destDir = (directionTyp)properties.readNumEntry("DestDir",0);
		QString specialCmdSrc = properties.readEntry("SpecialCmdSrc");
		QString specialCmdDest = properties.readEntry("SpecialCmdDest");

		if (srcRoom->getPathDirection(srcDir,specialCmdSrc)==NULL &&
			destRoom->getPathDirection(destDir,specialCmdDest)==NULL)
		{
			kdDebug(DAREA_PARTS_MAPMANAGER) << "createPath 2" << endl;
			// create
			properties.writeEntry("Type",(int)PATH);
			properties.writeEntry("SrcRoom",srcRoom->getRoomID());
			properties.writeEntry("SrcDir",(int)srcDir);
			properties.writeEntry("SrcLevel",srcRoom->getLevel()->getLevelID());
			properties.writeEntry("DestRoom",destRoom->getRoomID());
			properties.writeEntry("DestDir",(int)destDir);				
			properties.writeEntry("DestLevel",destRoom->getLevel()->getLevelID());
			
			if (properties.hasKey("PathTwoWay"))
			{
				properties.writeEntry("MakePathTwoWay","");
			}

			CMapCmdElementCreate *command = new CMapCmdElementCreate(this,i18n("Create Path"));
			command->addElement(&properties);

			addCommand(command);

			command->secondStage();


			QPtrList<CMapElement> *elements=command->getElements();

			for (CMapElement *el = elements->first(); el!=0; el = elements->next())
			{
				if (el->getElementType()==PATH)
				{
					result = (CMapPath *)el;
				}
			}

			if (result && d.getTwoWay())
			{
				makePathTwoWay(result);

				CMapCmdElementProperties *cmd = new CMapCmdElementProperties(this,i18n("Set Path properties"),result->getOpsitePath());
				cmd->getOrgProperties()->writeEntry("SrcAfterCommand","");
				cmd->getOrgProperties()->writeEntry("SrcBeforeCommand","");	
				cmd->getOrgProperties()->writeEntry("SpecialCmdSrc","");
				cmd->getNewProperties()->writeEntry("SrcAfterCommand",properties.readEntry("DestAfterCommand",""));
				cmd->getNewProperties()->writeEntry("SrcBeforeCommand",properties.readEntry("DestBeforeCommand",""));
				cmd->getNewProperties()->writeEntry("SpecialCmdSrc",specialCmdDest);
				addCommand(cmd);
			}
	
		}
		else
		{
			KMessageBox::information (NULL,i18n("A path already exsits at this location"),i18n("Kmud Mapper"));
		}
		
	}

	closeCommandGroup();
	return result;
}


/** Used to create a new path*/
CMapPath *CMapManager::createPath (CMapRoom *srcRoom,directionTyp srcDir,CMapRoom *destRoom,directionTyp destDir,bool undoable)
{
	// FIXME_jp : Allow this to call lowlevel mapper methods when undo is not active
	//            but becarefull of second stage stuff

	CMapPath *result = NULL;
	
	KMemConfig properties;
	properties.setGroup("Properties");
	properties.writeEntry("Type",(int)PATH);
	properties.writeEntry("SrcRoom",srcRoom->getRoomID());
	properties.writeEntry("SrcLevel",srcRoom->getLevel()->getLevelID());
	properties.writeEntry("SrcDir",(int)srcDir);
	properties.writeEntry("DestRoom",destRoom->getRoomID());
	properties.writeEntry("DestLevel",destRoom->getLevel()->getLevelID());
	properties.writeEntry("DestDir",(int)destDir);

	CMapCmdElementCreate *command = new CMapCmdElementCreate(this,i18n("Create Path"));
	command->addElement(&properties);
	bool active = getUndoActive();
	if (!undoable)
	{
		setUndoActive(false);
	}
		
	addCommand(command);

	command->secondStage();

	if (!undoable)
	{
		setUndoActive(active);
	}

	QPtrList<CMapElement> *elements=command->getElements();

	
	for (CMapElement *el = elements->first(); el!=0; el = elements->next())
	{
		if (el->getElementType()==PATH)
		{
			result = (CMapPath *)el;
		}
	}

	return result;
}


/** Find the first room in the map, if one can't be found then create one */
CMapRoom *CMapManager::findFirstRoom(CMapRoom *exsitingRoom)
{
	// Find the first room in the map
	for (CMapZone *zone = getMapData()->getFirstZone();zone!=0;zone = getMapData()->getNextZone())
	{
		for (CMapLevel *level = zone->getLevels()->first();level!=0;level = zone->getLevels()->next())
		{
			for (CMapRoom *room =  level->getRoomList()->first(); room!=0; room = level->getRoomList()->next())
			{
				if (room!=exsitingRoom)
				{
					return room;
				}
			}
		}
	}

	// If we get to this point then no room was found so create one
	return	createRoom(QPoint(2 * mapData->gridSize.width(),2 * mapData->gridSize.height()),getMapData()->getFirstZone()->getLevels()->first());
}

void CMapManager::deleteLevel(CMapLevel *level)
{
	//FIXME_jp: Check to see what happens when all levels are deleted.
    //          It may be nessecary to create a new one.

	openCommandGroup(i18n("Delete Level"));

	for (CMapRoom *room = level->getRoomList()->last(); room!=0; room = level->getRoomList()->last())
	{
		deleteElement(room);
	}

	for (CMapText *text = level->getTextList()->last(); text!=0; text = level->getTextList()->last())
		deleteElement(text);

	for (CMapZone *zone = level->getZoneList()->last(); zone!=0; zone = level->getZoneList()->last())
		deleteElement(zone);

	// Delete the level
	CMapCmdLevelDelete *cmd = new CMapCmdLevelDelete(this,i18n("Delete Level Actual Command"),level);
	addCommand(cmd);

	closeCommandGroup();
}


/** Check to see if a string is a valid move command
  * @param dirCmd The command that was typed
  * @return True, if a valid command otherwise false */
bool CMapManager::validMoveCmd(QString dirCmd)
{
	bool result = false;

	if (dirCmd.isEmpty()) return false;

	// check for directions
	for (uint i = 0; i < NUM_DIRECTIONS; ++i)
	{
		if (mapData->directions[i] == dirCmd)
		{
			result = true;
			break;
		}
	}
	
	if (!result)
	{
		if (currentRoom)
		{
			for (CMapPath *path=currentRoom->getPathList()->first(); path!=0; path=currentRoom->getPathList()->next())
			{
				if (path->getSpecialExit())
					if (path->getSpecialCmd()==dirCmd)
					{
						result = true;
						break;
					}
			}
		}
	}

	return result;
}

/** Used to delete a element from the map, should not be used by deleteElement as
  * it does not use groups                                                */
// FIXME_jp : This method can be removed if the deleteElement method is recursive
void CMapManager::deleteElementWithoutGroup(CMapElement *element,bool delOpsite)
{
	KMemConfig properties;
	properties.setGroup("Properties");

	element->saveProperties(&properties);

	CMapCmdElementDelete *command = new CMapCmdElementDelete(this,i18n("Delete Element"),delOpsite);

	command->addElement(&properties);
	addCommand(command);
}

/** Used to delete a element from the map */
void CMapManager::deleteElement(CMapElement *element,bool delOpsite)
{
	openCommandGroup(i18n("Delete Element"));

	// If the element is a room, then we also need to delete all it's paths
	if (element->getElementType()==ROOM)
	{
		CMapRoom *room = (CMapRoom *)element;
		if (room->getLinkedElement())	
		{
			deleteElementWithoutGroup(room->getLinkedElement(),true);
		}

		// Delete the paths for the room
		for (CMapPath *path=room->getPathList()->last(); path!=0; path=room->getPathList()->last())
			deleteElementWithoutGroup(path,false);

		// Delete any paths connecting with this room
		for (CMapPath *path=room->getConnectingPathList()->last(); path!=0; path = room->getConnectingPathList()->last())
			deleteElementWithoutGroup(path,false);
	}

	if (element->getElementType()==ZONE)
	{
		// Delete the levels in the zone
		CMapZone *zone = (CMapZone *)element;
		if (zone->getLinkedElement())	
		{
			deleteElementWithoutGroup(zone->getLinkedElement(),true);
		}

		while (zone->getLevels()->first()!=0)
		{
			deleteLevel(zone->getLevels()->first());
		}
	}

	deleteElementWithoutGroup(element,delOpsite);

	closeCommandGroup();
}

/** Used to load a map */
void CMapManager::importMap(const KURL& url,CMapFileFilterBase *filter)
{
	setUndoActive(false);
	commandHistory->clear();
	historyGroup = NULL;

    eraseMap();
	
	// Load the map using the correct filter
	filter->loadData(url);

	// Set the login room for the character
	DomConfig *domConfig = getCharProfile();

	if (domConfig)
	{
		KMemConfig *config = domConfig->config();
		config->setGroup("Login");

		int roomID = config->readNumEntry("Room",-1);
		int levelID = config->readNumEntry("Login",-1);

		if (roomID != -1 && levelID!=-1)
		{
			CMapLevel *level = findLevel(levelID);
			if (level!=NULL)
			{
				CMapRoom *room = level->findRoom(roomID);

				if (room!=NULL)
				{
					setLoginRoom(room);
				}
				else
				{
					kdDebug(DAREA_PARTS_MAPMANAGER) << "Unable to fund start room" << endl;
				}
			}
			else
			{
				kdDebug(DAREA_PARTS_MAPMANAGER) << "Unable to find start level" << endl;
			}
		}
	
		delete domConfig;
	}

	if (!getLoginRoom())
	{
		CMapRoom *firstRoom = findFirstRoom(NULL);
		setLoginRoom(firstRoom);
	}

	setCurrentRoomWithoutUndo(loginRoom);

	if (getLoginRoom())
	{
		for (CMapViewBase *view = getViewList()->first(); view!=0; view=getViewList()->next())
		{
			if (view->getCurrentlyViewedLevel()==NULL)	
			{
				view->showPosition(loginRoom,true);
			}
		}
		setCurrentRoom(getLoginRoom());
	}

    updateZoneListCombo();
	
	setUndoActive(true);
}

/** Used to save a map */
void CMapManager::exportMap(const KURL& url,CMapFileFilterBase *filter)
{	
	filter->saveData(url);
	commandHistory->documentSaved ();
}

/** Used to inform to change the state of the navigation tools */
void CMapManager::activeViewChanged(void)
{
	if (activeView)
	{
		CMapZone *zone = activeView->getCurrentlyViewedZone();
		if (zone)
		{
			QStringList lst = zoneMenu->items();
			zoneMenu->setCurrentItem(lst.findIndex(zone->getLabel()));
			m_toolsUpZone->setEnabled(zone->getZone());
		}

		CMapLevel *level = activeView->getCurrentlyViewedLevel();
		if (level)
		{
			m_toolsUpLevel->setEnabled(level->getNextLevel());
			m_toolsDownLevel->setEnabled(level->getPrevLevel());
		}
	}
}

void CMapManager::levelChanged(CMapLevel *level)
{
	if (level==NULL)
		return;

	for (CMapViewBase *view=mapViewList.first();view!=0; view=mapViewList.next())
	{
		view->changedLevel(level);
	}
}

/** Used to inform the various parts of the mapper that a element has changed */
void CMapManager::changedElement(CMapElement *element)
{
	if (element==NULL)
		return;

	for (CMapPluginBase *plugin = getPluginList()->first();plugin!=0; plugin= getPluginList()->next())
	{
		plugin->elementChanged(element);
	}

	if (element->getElementType()==ZONE)
	{
		updateZoneListCombo();
	}
	
	for (CMapViewBase *view=mapViewList.first();view!=0; view=mapViewList.next())
	{
		view->changedElement(element);
	}
}

/** Used to inform the various parts of the mapper that a element has added */
void CMapManager::addedElement(CMapElement *element)
{
	if (element->getElementType()==ZONE)
		updateZoneListCombo();

	for (CMapViewBase *view=mapViewList.first();view!=0; view=mapViewList.next())
	{
		if (view->getCurrentlyViewedLevel())
			view->addedElement(element);
	}
}


/** Used to alter the path properties */
bool CMapManager::propertiesPath(CMapPath *path)
{
	DlgMapPathProperties d(this,path);

	return d.exec();
}

/** Used to alter the room properties */
bool CMapManager::propertiesRoom(CMapRoom *room)
{
	openCommandGroup("Change room properties");
	DlgMapRoomProperties d(this,room);

	bool b = d.exec();
	
	closeCommandGroup();
	
	return b;
}

/** Used to alter the zone properties */
bool CMapManager::propertiesZone(CMapZone *zone)
{
	openCommandGroup("Change zone properties");
	DlgMapZoneProperties d(this,zone);

	bool b = d.exec();
	
	closeCommandGroup();
	
	return b;
}

/** Used to alter the text properties */
bool CMapManager::propertiesText(CMapText *text)
{
	DlgMapTextProperties d(this,text);

	if (d.exec())
	{
		text->updateLinkElements();
		return true;
	}

	return false;
}

/** Used to read the map options */
void CMapManager::readOptions()
{	
	KConfig *config=instance()->config();

	// Rad map colors
	config->setGroup("MAPPER COLOUR");
	mapData->backgroundColor=QColor(201,167,55);
	mapData->backgroundColor=config->readColorEntry("Background",&mapData->backgroundColor);
	mapData->gridColor=QColor(213,178,88);
	mapData->gridColor=config->readColorEntry("Grid", &mapData->gridColor);
	mapData->lowerRoomColor=darkGray;
	mapData->lowerRoomColor=config->readColorEntry("LowerRoom", &mapData->lowerRoomColor);
	mapData->lowerZoneColor=darkGray;
	mapData->lowerZoneColor=config->readColorEntry("LowerZone", &mapData->lowerZoneColor);
	mapData->lowerTextColor=darkGray;
	mapData->lowerTextColor=config->readColorEntry("LowerText", &mapData->lowerTextColor);
	mapData->defaultRoomColor=QColor(192,192,192);
	mapData->defaultRoomColor=config->readColorEntry("DefaultRoom", &mapData->defaultRoomColor);
	mapData->defaultZoneColor=QColor(192,192,192);
	mapData->defaultZoneColor=config->readColorEntry("DefaultZone", &mapData->defaultZoneColor);
	mapData->higherRoomColor=white;
	mapData->higherRoomColor=config->readColorEntry("HigherRoom", &mapData->higherRoomColor);
	mapData->higherZoneColor=white;
	mapData->higherZoneColor=config->readColorEntry("HigherZone", &mapData->higherZoneColor);
	mapData->higherTextColor=white;
	mapData->higherTextColor=config->readColorEntry("HigherText", &mapData->higherTextColor);
	mapData->lowerPathColor=darkGray;
	mapData->lowerPathColor=config->readColorEntry("LowerPath", &mapData->lowerPathColor);
	mapData->defaultPathColor=black;
	mapData->defaultPathColor=config->readColorEntry("DefaultPath", &mapData->defaultPathColor);
	mapData->higherPathColor=white;
	mapData->higherPathColor=config->readColorEntry("HigherPath", &mapData->higherPathColor);
	mapData->defaultTextColor=black;
	mapData->defaultTextColor=config->readColorEntry("DefaultText", &mapData->defaultTextColor);
	mapData->selectedColor=blue;
	mapData->selectedColor=config->readColorEntry("Selected", &mapData->selectedColor);
	mapData->specialColor = yellow;
	mapData->specialColor = config->readColorEntry("Special", &mapData->specialColor);
	mapData->loginColor=blue;
	mapData->loginColor=config->readColorEntry("Login", &mapData->loginColor);
	mapData->editColor=red;
	mapData->editColor=config->readColorEntry("Edit",&mapData->editColor);
	mapData->currentColor=red;
	mapData->currentColor=config->readColorEntry("Current", &mapData->currentColor);

	// read speedwalk options
    config->setGroup("Speedwalk");
	mapData->speedwalkAbortActive = config->readBoolEntry("AbortActive",true);
	mapData->speedwalkAbortLimit = config->readNumEntry("AbortLimit",100);
	mapData->speedwalkDelay = config->readNumEntry("Delay",0);

	// Set menu options
	m_toolsGrid->setChecked(mapData->gridVisable);
	m_toolsCreate->setChecked(mapData->createModeActive);
	m_viewLowerLevel->setChecked(mapData->showLowerLevel);
	m_viewUpperLevel->setChecked(mapData->showUpperLevel);

	for (CMapPluginBase *plugin = getPluginList()->first(); plugin!=0; plugin = getPluginList()->next())
	{
		plugin->loadConfigOptions();
	}
}

/** Used to write the map options */
void CMapManager::saveGlobalConfig()
{
	KConfig *config = instance()->config();

	config->setGroup("MAPPER COLOUR");
	config->writeEntry("Background", mapData->backgroundColor);
	config->writeEntry("Grid", mapData->gridColor);
	config->writeEntry("LowerRoom", mapData->lowerRoomColor);
	config->writeEntry("LowerZone", mapData->lowerZoneColor);
	config->writeEntry("LowerText", mapData->lowerTextColor);
	config->writeEntry("HigherZone", mapData->higherZoneColor);
	config->writeEntry("DefaultRoom", mapData->defaultRoomColor);
	config->writeEntry("DefaultZone", mapData->defaultZoneColor);
	config->writeEntry("DefaultText", mapData->defaultTextColor);
	config->writeEntry("HigherRoom", mapData->higherRoomColor);
	config->writeEntry("HigherText", mapData->higherTextColor);
	config->writeEntry("LowerPath", mapData->lowerPathColor);
	config->writeEntry("DefaultPath",mapData->defaultPathColor);
	config->writeEntry("HigherPath", mapData->higherPathColor);
	config->writeEntry("Selected", mapData->selectedColor);
	config->writeEntry("Special", mapData->specialColor);
	config->writeEntry("Login", mapData->loginColor);
	config->writeEntry("Edit", mapData->editColor);
	config->writeEntry("Current", mapData->currentColor);

	for (CMapPluginBase *plugin = getPluginList()->first(); plugin!=0; plugin = getPluginList()->next())
	{
		plugin->saveConfigOptions();
	}

    config->setGroup("Speedwalk");
	config->writeEntry("AbortActive",mapData->speedwalkAbortActive);
	config->writeEntry("AbortLimit",mapData->speedwalkAbortLimit);
	config->writeEntry("Delay",mapData->speedwalkDelay);

	config->sync();

	redrawAllViews();
}

void CMapManager::getCounts(int *levels,int *rooms,int *paths,int *labels,int *zones)
{
	*levels = 0;
	*rooms = 0;
	*labels = 0;
	*paths = 0;
	*zones = 0;

	for (CMapZone *zone = mapData->getFirstZone(); zone !=0 ; zone = mapData->getNextZone())
	{
		for (CMapLevel *level = zone->getLevels()->first(); level !=0; level = zone->getLevels()->next())
		{
			for (CMapRoom *room = level->getRoomList()->first(); room!=0; room = level->getRoomList()->next())
			{
				*paths += room->getPathList()->count();
			}

			*rooms += level->getRoomList()->count();
			*labels += level->getTextList()->count();
			*zones += level->getZoneList()->count();
		}

		*levels += zone->getLevels()->count();
	}
}

directionTyp CMapManager::getOpsiteDirection(directionTyp dir)
{
	directionTyp result = SOUTH;
	switch (dir)
	{
		case NORTH     : result = SOUTH; break;
		case SOUTH     : result = NORTH; break;
		case EAST      : result = WEST; break;
		case WEST      : result = EAST; break;
		case NORTHWEST : result = SOUTHEAST; break;
		case NORTHEAST : result = SOUTHWEST; break;
		case SOUTHWEST : result = NORTHEAST; break;
		case SOUTHEAST : result = NORTHWEST; break;
		case UP        : result = DOWN; break;
		case DOWN      : result = UP; break;
		case SPECIAL   : result = SPECIAL; break;
	}

	return result;
}

/** Used to set the current tool */
void CMapManager::setCurrentTool(CMapToolBase *tool)
{
	if (currentTool)
		currentTool->toolUnselected();

	currentTool=tool;

	if (currentTool)
		currentTool->toolSelected();
}

/** Usd to get the current tool */
CMapToolBase *CMapManager::getCurrentTool(void)
{
	return currentTool;
}

/** This method puts a element into edit state */
void CMapManager::setEditElement(CMapElement *element)
{
	if (elementEdit)
	{
		elementEdit->setEditMode(false);
		changedElement(elementEdit);
	}

	element->setEditMode(true);
	elementEdit = element;
	changedElement(elementEdit);
}

/** This is used to remove edit state from element being edited */
void CMapManager::unsetEditElement(void)
{
	if (elementEdit)
	{
		elementEdit->setEditMode(false);
		changedElement(elementEdit);
	}
}

/** This gets the element that is being edited */
CMapElement *CMapManager::getEditElement(void)
{
	return elementEdit;
}

/** This cancels any editing */
void CMapManager::stopEditing(void)
{
	if (elementEdit)
	{
		elementEdit->setEditMode(false);
	}

	elementEdit = NULL;
}

KmudViewManager* CMapManager::getViewManager(void)
{
	return pAPI->viewManager();
}

int CMapManager::getUndoActive(void)
{
	return m_commandsActive;
}

/** This is used to tell the mapper if commands should be added to the history list */
void CMapManager::setUndoActive(bool active)
{
	m_commandsActive = active;
}

/** Used to add a command to the command history */
void CMapManager::addCommand(KCommand *command,bool execute)
{
	if (m_commandsActive)
	{
		if (historyGroup)
		{
			historyGroup->addCommand(command,execute);
		}
		else
		{
			commandHistory->addCommand(command,execute);
		}
	}
	else
	{
		command->execute();
	}
}

void CMapManager::openCommandGroup(QString name)
{
	CMapCmdGroup *group = new CMapCmdGroup(this,name);
	group->setPreviousGroup(historyGroup);
	historyGroup = group;
}

void CMapManager::closeCommandGroup(void)
{
	CMapCmdGroup *currentGroup = historyGroup;
	CMapCmdGroup *oldGroup =historyGroup->getPreviousGroup();
	historyGroup = oldGroup;
	addCommand(currentGroup,false);
}

/** move the player relative to the current position of the player
  * @param cmd The move command used to move the player */
void CMapManager::movePlayerBy(QString cmd)
{
	QString specialCmd = "";
	directionTyp dir  = textToDirection(cmd);

	if (dir == SPECIAL)
		specialCmd = cmd;

	movePlayerBy(dir,mapData->createModeActive,specialCmd);
}

/** move the player relative to the current position of the player
  * This command has support for special paths, but can only move
  * anlong exsiting special paths. In other words, it is unable to
  * create special paths.
  */
void CMapManager::movePlayerBy(directionTyp dir,bool create,QString specialCmd)
{
	openCommandGroup(i18n("Move Player"));
	CMapLevel *destLevel;

	// Make sure that the room we are currently in is visible
	CMapRoom *tmpRoom = currentRoom;

	for (CMapViewBase *view = mapViewList.first(); view!=0; view = mapViewList.next())
	{
		if (view->getCurrentlyViewedLevel() != currentRoom->getLevel() && view->getFollowMode())
		view->showPosition(currentRoom->getLowPos(),currentRoom->getLevel());
	}

	currentRoom = tmpRoom;
	// Find the destination of the path that was traveled and if no path
	// is exsits for the given direction create the room and path if necsarry
	CMapPath *path = currentRoom->getPathDirection(dir,specialCmd);

    if (path)
	{
		setCurrentRoom(path->getDestRoom());
		for (CMapViewBase *view = mapViewList.first(); view!=0; view = mapViewList.next())
		{
			view->showPosition(currentRoom->getLowPos(),currentRoom->getLevel());
		}
	}
	else
	{
		CMapRoom *srcRoom = currentRoom;
		CMapPath *opsitePath = NULL;
		int x=0,y=0;
		bool bFound = false;

		destLevel = currentRoom->getLevel();

		// Check to see if there is a path in the opsite direction that we should
		// be using
		for (CMapPath *path2=srcRoom->getConnectingPathList()->first();path2!=0;path2=srcRoom->getConnectingPathList()->next())
		{
			if (path2->getDestDir() == dir)
			{
				x = path2->getSrcRoom()->getX();
				y = path2->getSrcRoom()->getY();
				opsitePath = path2;
				bFound = true;
				break;
			}
		}

		if (!opsitePath)
		{
			// No opsite path found so we need to create a path		
			QPoint inc;
			directionToCord(dir,QSize(getMapData()->gridSize.width()*2,getMapData()->gridSize.height()*2),&inc);
			x = currentRoom->getX()+inc.x();
			y = currentRoom->getY()+inc.y();

			destLevel = currentRoom->getLevel();

			if (dir == UP)
			{
				if (currentRoom->getLevel()->getNextLevel())
				{
					destLevel = currentRoom->getLevel()->getNextLevel();
				}
				else
				{
					destLevel = createLevel(UP,currentRoom->getZone());
				}

			}

			if (dir == DOWN)
			{
				if (currentRoom->getLevel()->getPrevLevel())
				{
					destLevel = currentRoom->getLevel()->getPrevLevel();
				}
				else
				{
					destLevel = createLevel(DOWN,currentRoom->getZone());
				}
			}

			// Check to see if the map needs to be moved
			// and calulate the offset to move if it needs moving.
			if (x<getMapData()->gridSize.width()*3 || y<getMapData()->gridSize.height()*2)
			{
				int movex,movey;

				if (x<getMapData()->gridSize.width()*3)
				{
					movex = getMapData()->gridSize.width()*3 - x;
					x+=movex;
				}
				else
					movex = 0;

				if (y<getMapData()->gridSize.height()*3)
				{
					movey = getMapData()->gridSize.height()*3- y;
					y+=movey;
				}
				else
					movey = 0;

				moveMap (QPoint(movex,movey),currentRoom->getZone());
			}
		}

		// Check to see if the room already exsits
		CMapElement *elm = findElementAt(QPoint (x,y),destLevel);
		CMapRoom *newCurrentRoom = NULL;


		// FIXME_jp : Tidy this up so that the returns are not needed.
		if (!elm)
		{
			newCurrentRoom = NULL;
		}
		else if (elm->getElementType()==ZONE)
		{
			// Found a zone were we want to create a room so exit method
			closeCommandGroup();
			return;
		}
		else
		{
			newCurrentRoom = (CMapRoom *)elm;
		}

		// Create the room if it does not exsit
		if (!newCurrentRoom)
		{
			if (create)
			{
				newCurrentRoom=createRoom(QPoint (x,y),destLevel);
				directionTyp destDir = getOpsiteDirection(dir);

				// Create the new path to the room
				CMapPath *newPath = createPath(srcRoom,dir,newCurrentRoom,destDir);

				// Make the path two way if the default path type is two way
				if (getMapData()->defaultPathTwoWay && bFound == false)
				{
					makePathTwoWay(newPath);
				}
				setCurrentRoom(newCurrentRoom);
			}
			else
			{
				closeCommandGroup();
				return;
			}
		}
	}

	closeCommandGroup();
}

/** Used to walk the player in the mud to a given room */
void CMapManager::walkPlayerTo(CMapRoom *toRoom)
{
	QPtrQueue<CMapRoom> roomsToVisit;
	CMapRoom *destRoom;
	CMapRoom *srcRoom;
	CMapPath *path;
	CMapRoom *foundRoom;
	signed int time = 0;
	bool bFound = false;
	CMapPath *foundPath = NULL;

	int speedWalkAbortCount = 0;

	if (currentRoom == toRoom) return;

	if (speedwalkActive)
	{
		KMessageBox::information (NULL,i18n("Speedwalking is already in progress"),i18n("Kmud Mapper"));
		return;
	}

	speedwalkActive = true;

	pathToWalk.clear();

	// Reset the seach count for all the rooms
	for (CMapZone *zone = mapData->getFirstZone(); zone!=0; zone = mapData->getNextZone())
	{
		for (CMapLevel *level=zone->getLevels()->first(); level !=0;level = zone->getLevels()->next())
		{
			for (CMapRoom *room=level->getRoomList()->first();room !=0 ; room = level->getRoomList()->next())
			{
				room->setMoveTime(-1);
			}
		}
	}

	// Init things for the start room
	srcRoom = currentRoom;
	destRoom = (CMapRoom *)toRoom;
	srcRoom->setMoveTime(time++);

	// enqueue starting room
	roomsToVisit.enqueue(srcRoom);
	CMapRoom* room;

	// this is the Breadth First Search Algorithm
	while (!(roomsToVisit.isEmpty() || bFound))
	{
		foundRoom = roomsToVisit.dequeue();

		// for all neighbours of foundRoom
		for (path=foundRoom->getPathList()->first(); path != 0; path=foundRoom->getPathList()->next())
		{
			room = path->getDestRoom();

			if (room == destRoom)
			{
				bFound = true;
				break;
			}

			// neighbour has not been visited yet, enqueue it
			if (room->getMoveTime() == -1)
			{
				room->setMoveTime(time++);
				roomsToVisit.enqueue(room);
			}
		}
	}

	// Check to see if we were unable to find any paths
	if (!bFound)
	{
		roomsToVisit.clear();
		KMessageBox::information (NULL,i18n("The automapper was unable to find a path to requested room"),i18n("Kmud Mapper"));

		return;
	}

	speedWalkAbortCount=0;
	
	// Trace steps that need to be taken backwards from the dest to the src room
	while(destRoom != srcRoom)
	{
		time = destRoom->getConnectingPathList()->first()->getSrcRoom()->getMoveTime();
		foundRoom = destRoom->getConnectingPathList()->first()->getSrcRoom();

		// Find the room with the shortest time as this is the room we
		// should be moving to.
		for (path=destRoom->getConnectingPathList()->first();path!=0; path=destRoom->getConnectingPathList()->next())
		{
			if (time == -1 || (path->getSrcRoom()->getMoveTime()<=time && path->getSrcRoom()->getMoveTime()!=-1))
			{
				time = path->getSrcRoom()->getMoveTime();
				foundRoom = path->getSrcRoom();
				foundPath = path;
			}
		}

		pathToWalk.push(new QString(directionToText(foundPath->getSrcDir(),foundPath->getSpecialCmd())));

		destRoom=foundRoom;
		// Check to make sure that tings are not stuck in a loop and abort
		// if the move limit is reached

		speedWalkAbortCount++;
		if (mapData->speedwalkAbortActive && (speedWalkAbortCount == mapData->speedwalkAbortLimit))
		{
			KMessageBox::information (NULL,i18n("Speedwalk abort because move limit was reached"),i18n("Kmud Mapper"));

			return;
		}
	}

	speedwalkProgressDlg->setTotalSteps(pathToWalk.count());
	speedwalkProgressDlg->setProgress(0);
	pAPI->mainStatusBar()->addWidget(speedwalkProgressDlg,0,true);
	speedwalkProgressDlg->show();
	speedwalkProgress = 0;

	// Start walking path
	slotWalkPlayerAlongPath();

	// Tidy up
	roomsToVisit.clear();
}

void CMapManager::slotAbortSpeedwalk(void)
{
	pathToWalk.clear();
	speedwalkActive = false;
	speedwalkProgressDlg->setProgress(speedwalkProgressDlg->getTotalSteps());
	speedwalkProgressDlg->hide();
	pAPI->mainStatusBar()->removeWidget(speedwalkProgressDlg);
}

/** Used to recersivly move the play along a speedwalk path */
void CMapManager::slotWalkPlayerAlongPath(void)
{
	if (speedwalkActive)
	{
		QString *dir = pathToWalk.pop();

		pAPI->sendCommand(*dir+"\n",true);
		speedwalkProgressDlg->setProgress(++speedwalkProgress);

		// Walk the path
		if (!pathToWalk.isEmpty())
		{
			QTimer::singleShot( mapData->speedwalkDelay * 100, this, SLOT(slotWalkPlayerAlongPath()) );
		}
		else
		{
			slotAbortSpeedwalk();
		}
	}
}

/** This method is used to convert a direction into a offset */
void CMapManager::directionToCord(directionTyp dir, QSize distance,QPoint *pos)
{
	int x = pos->x();
	int y = pos->y();

	switch (dir)
	{
		case NORTH     : x = 0;
		                 y = -distance.height();
		                 break;
		case EAST      : x = distance.width();
		                 y = 0;
		                 break;
		case SOUTH     : x = 0;
		                 y = distance.height();
		                 break;
		case WEST      : x = -distance.width();
		                 y = 0;
                         break;
		case NORTHEAST : x = distance.width();
		                 y = -distance.height();
		                 break;
		case NORTHWEST : x = -distance.width();
		                 y = -distance.height();
		                 break;
		case SOUTHEAST : x = distance.width();
		                 y = distance.height();
		                 break;
		case SOUTHWEST : x = -distance.width();
		                 y = distance.height();
		                 break;
		case UP        : break;
		case DOWN      : break;
		case SPECIAL   : break;
	}

	pos->setX(x);
	pos->setY(y);
}

/**
 * This method is used to set the element when that a popup menu is to be showed for
 */
void CMapManager::setSelectedElement(CMapElement *element)
{
	m_selectedElement = element;	
}

/**
 * This methid is used to set the position that a element was selected when the contex menu is shown
 */
void CMapManager::setSelectedPos(QPoint pos)
{
	m_selectedPos = pos;
}

/**
 * This method is used to get the element that a popup menu is being displayed for
 */
CMapElement *CMapManager::getSelectedElement(void)
{
	return m_selectedElement;
}

/** This method is used to move the elements in a zone by the given vector */
void CMapManager::moveMap(QPoint inc,CMapZone *zone)
{
	CMapCmdMoveMap *cmd = new CMapCmdMoveMap(this,inc,zone,i18n("Move Elements in map"));
	addCommand(cmd);
}

/** This method is used to make a path two way */
void CMapManager::makePathTwoWay(CMapPath *path)
{
	if (!path->getOpsitePath())
	{
		CMapCmdTogglePathTwoWay *cmd =  new CMapCmdTogglePathTwoWay(this,i18n("Make Path Two-Way"),path);
		addCommand(cmd);
	}
}

/** This method is used to make a path one way */
void CMapManager::makePathOneWay(CMapPath *path)
{
	if (path->getOpsitePath())
	{
		CMapCmdTogglePathTwoWay *cmd =  new CMapCmdTogglePathTwoWay(this,i18n("Make Path One-Way"),path);
		addCommand(cmd);
	}
}

/** Get the level that has focues at the moment */
CMapViewBase *CMapManager::getActiveView(void)
{	
	return activeView;
}                                                                  	

void CMapManager::setActiveView(CMapViewBase *view)
{
	// If this is the same view then don't need to do anything
	if (activeView == view || view ==NULL) return;

	// If there is an active view already set mark it as not active
	if (activeView)
	{
		unselectElements(activeView->getCurrentlyViewedLevel());
		activeView->setActive(false);
		levelChanged(activeView->getCurrentlyViewedLevel());
	}

	// Make the new view active
	view->setActive(true);
	activeView = view;

	// Update the overview window with the new active view
	viewChanged();

	activeViewChanged();
}

/** Used to repaint all the views */
void CMapManager::redrawAllViews(void)
{
	for (CMapViewBase *view=mapViewList.first();view!=0; view=mapViewList.next())
	{
		view->redraw();
	}
}

/** This method is called to create a new map, when the new map menu option is selected */
void CMapManager::slotFileNew()
{
	if (KMessageBox::warningYesNo (NULL,i18n("Are you sure you want to create a new map?\nThis action can not be undone"),i18n("Kmud Mapper"))== KMessageBox::Yes)
	{
		setUndoActive(false);
		commandHistory->clear();
		historyGroup = NULL;
		eraseMap();
		createNewMap();
		setUndoActive(true);
	}
}

/** This method is called to load a map from a file when the load menu option is selected */
void CMapManager::slotFileLoad()
{
	int found = 0;
	QString filterStr = "";

	for (CMapFileFilterBase *filter = m_fileFilter.first();filter!=0;filter=m_fileFilter.next())
	{
		if (filter->supportLoad())
		{
     		filterStr = filterStr + filter->getPatternExtension() + "|"+filter->getName();
			filterStr += "\n";
			found++;
		}
	}

	if (found>0)
	{
		filterStr = filterStr.remove(filterStr.length()-1,1);

		KFileDialog dlg(":", filterStr,NULL, "filedialog", true);
	    dlg.setOperationMode( KFileDialog::Opening );

		dlg.setCaption(i18n("Load Map File"));
		dlg.setMode( KFile::File );
		if (dlg.exec())
		{

			for (CMapFileFilterBase *filter = m_fileFilter.first();filter!=0;filter=m_fileFilter.next())
			{
				if (filter->getPatternExtension()==dlg.currentFilter() && filter->supportLoad())
				{
					importMap(dlg.selectedURL(),filter);
					break;
				}
			}
		}
	}
}

/** This method is called to save the map to a file when the "save as" menu option is selected */
void CMapManager::slotFileSave()
{
	int found = 0;
	QString filterStr = "";
	
	for (CMapFileFilterBase *filter = m_fileFilter.first();filter!=0;filter=m_fileFilter.next())
	{
		if (filter->supportSave())
		{
			filterStr = filterStr + filter->getPatternExtension() + "|"+filter->getName();
			filterStr += "\n";
			found++;
		}
	}

	if (found>0)
	{
		filterStr = filterStr.remove(filterStr.length()-1,1);		
		KFileDialog dlg(":", filterStr,NULL, "filedialog", true);
	    dlg.setOperationMode( KFileDialog::Saving );

		dlg.setCaption(i18n("Save Map File"));
		dlg.setMode( KFile::File );
		if (dlg.exec())
    {

  		for (CMapFileFilterBase *filter = m_fileFilter.first();filter!=0;filter=m_fileFilter.next())
  		{	
  			if (filter->getPatternExtension()==dlg.currentFilter())
  			{
  				KURL url = dlg.selectedURL();
  				if (!url.path().endsWith(filter->getExtension()))
  				{
  					url.setPath(url.path()+filter->getExtension());
  				}
	  			exportMap(url,filter);
				
		  		break;
  			}
      }
		}
	}
}

/** This method is used to save the map to the default location */
void CMapManager::loadMap(void)
{
	QString mapDir = pAPI->profileAPI()->defaultSavePath(mudProfile);

	CMapFileFilterBase *filter = m_fileFilter.first();

	QString filename = mapDir + "/" + "map" + filter->getExtension();
	QFile file(filename);

	if (file.exists())
	{
		kdDebug(DAREA_PARTS_MAPMANAGER) << "Loading map....." << endl;
		KURL url;
		url.setPath(filename);
		importMap(url,filter);
	}
	else
	{
		kdDebug(DAREA_PARTS_MAPMANAGER) << "Creating new map...." << endl;
		createNewMap();
	}
}

/** This method is used to load the map from the default location */
void CMapManager::saveMap(void)
{
	QString mapDir = pAPI->profileAPI()->defaultSavePath(mudProfile);
	CMapFileFilterBase *filter = m_fileFilter.first();

	KURL url;
	url.setPath(mapDir + "/" + "map"+ filter->getExtension());
	exportMap(url,filter);
}

/** This methid is called to display map information when the information menu option is selected */
void CMapManager::slotFileInfo()
{
	DlgMapInfo d(this,NULL,"infoDialog");
	
	d.exec();
}

void CMapManager::slotToolsGrid()
{
	getMapData()->gridVisable = m_toolsGrid->isChecked();
	redrawAllViews();
}


void CMapManager::slotToolsLevelUp()
{
	CMapLevel *level = getActiveView()->getCurrentlyViewedLevel()->getNextLevel();
	if (level)
	{
		getActiveView()->showPosition(level,false);
		getActiveView()->redraw();
		viewChanged();
	}
}

void CMapManager::slotToolsLevelDown()
{
	CMapLevel *level = getActiveView()->getCurrentlyViewedLevel()->getPrevLevel();
	if (level)
	{
		getActiveView()->showPosition(level,false);
		getActiveView()->redraw();
		viewChanged();
	}
}

void CMapManager::slotToolsLevelDelete()
{
	CMapLevel *level = getActiveView()->getCurrentlyViewedLevel();

	if (level)
	{	
		deleteLevel(level);
		viewChanged();
	}
}

void CMapManager::slotToolsZoneUp()
{
	CMapZone *zone = getActiveView()->getCurrentlyViewedZone();
	if (zone)
	{
		CMapLevel *level = zone->getLevel();
		if (level)
		{
			getActiveView()->showPosition(zone->getLowPos(),level);
			getActiveView()->redraw();
			viewChanged();
		}

	}
}

void CMapManager::slotToolsDeleteZone()
{
	CMapZone *zone = getActiveView()->getCurrentlyViewedZone();
	if (zone)
	{
		deleteElement(zone);
	}
}

void CMapManager::slotToolsCreateMode()
{
	getMapData()->createModeActive = m_toolsCreate->isChecked();
}

void CMapManager::slotToolsZoneProperties()
{	
	CMapZone *zone = getActiveView()->getCurrentlyViewedZone();
	if (zone)
	{
		propertiesZone(zone);
	}
}

void CMapManager::slotSelectZone()
{
	for (CMapZone *zone = mapData->getFirstZone();zone!=0; zone = mapData->getNextZone())
	{
		if (zone->getLabel()==zoneMenu->currentText())
		{
			if (getActiveView())
			{
				getActiveView()->showPosition(zone->getLevels()->first());
				getActiveView()->redraw();
			}
			else
			{
				openNewMapView(zone->getLevels()->first());
			}
			break;
		}
	}	
}

void CMapManager::slotViewNewMap()
{
	KmudDockWidget *view = openMapView();
	pAPI->viewManager()->addWidget(view,"Kmud/Mapper",KmudViewManager::DockRight);
}

void CMapManager::slotViewUpperLevel()
{
	mapData->showUpperLevel = m_viewUpperLevel->isChecked();
	redrawAllViews();
}

void CMapManager::slotViewLowerLevel()
{
	mapData->showLowerLevel = m_viewLowerLevel->isChecked();
	redrawAllViews();
}

void CMapManager::slotViewToolsToolbar()
{
	QWidget* toolBar = factory()->container("tools", this);
	if (toolBar == 0 || !toolBar->inherits("QToolBar"))
		return;

	if(!m_viewToolsToolbar->isChecked())
	{
		toolBar->hide();
	}
	else
	{
		toolBar->show();
	}
}

void CMapManager::slotViewNavToolbar()
{
	QWidget* toolBar = factory()->container("navigation", this);
	if (toolBar == 0 || !toolBar->inherits("QToolBar"))
		return;

	if(!m_viewNavToolbar->isChecked())
	{
		toolBar->hide();
	}
	else
	{
		toolBar->show();
	}
}

void CMapManager::slotProfileChanged()
{
	QString profileName = pAPI->profile();
	bool profileChanged = false;
	if (profileName.isEmpty())
	{
		if (!mudProfile.isEmpty())
		{
			saveMap();
			saveMudProfile();
			profileChanged = true;
		}
		if (!charProfile.isEmpty())
		{
			saveCharProfile();
			profileChanged = true;
		}
		mudProfile = QString::null;
		charProfile = QString::null;
	}
	else
	{
		QStringList names = QStringList::split("/", profileName);
	
		if (names[0] != mudProfile)
		{
			if (!mudProfile.isEmpty())
			{
				saveMap();
			}
			saveMudProfile(); // save old profile
			loadMudProfile(pAPI->profileAPI()->openProfile(names[0])); // load new one
			mudProfile = names[0];
			profileChanged = true;
		}
		
		if (profileName != charProfile)
		{
			saveCharProfile();
			loadCharProfile(pAPI->profileAPI()->openProfile(profileName));
			charProfile = profileName;
			profileChanged = true;
		}
	}

	if (profileChanged)
	{
		if (!charProfile.isEmpty() && !mudProfile.isEmpty())
		{
			enableNonViewActions(true);
			loadMap();
		}
		else
		{
			// deactivate menus/toolbar actions if necessary
		}

		for (CMapPluginBase *plugin=getPluginList()->first();plugin!=0; plugin=getPluginList()->next())
		{
			plugin->profileChanged();
		}
	}	
}

void CMapManager::slotProfileDataChanged(const QString& profile, uint type)
{
	if (profile.isEmpty())
		return;

	// we are only interested in changes to the config structure
	if ((type & ProfileAPI::CONFIG) != 0)
	{
		if (profile == mudProfile)
		{
			QDomElement profileElement = pAPI->profileAPI()->openProfile(profile);
			if (!profileElement.isNull())
				loadMudProfile(profileElement);
		}
		else if (profile == charProfile)
		{
			QDomElement profileElement = pAPI->profileAPI()->openProfile(profile);
			if (!profileElement.isNull())
				loadCharProfile(profileElement);
		}
	}

	for (CMapPluginBase *plugin=getPluginList()->first();plugin!=0; plugin=getPluginList()->next())
	{
		plugin->profileDataChanged(profile,type);
	}
}

void CMapManager::loadMudProfile(QDomElement profileElement)
{
	kdDebug(DAREA_PARTS_MAPMANAGER) << "loadMudProfile\n";
	
	// clear old data
	mapData->initDirections();
	
	if (profileElement.isNull())
	{
		kdDebug(DAREA_PARTS_MAPMANAGER) << "mud profile element is null!\n";
		return;
	}
	
	// find mapper config
	DomConfig* domConfig = DomConfig::findGroup("/mapper", profileElement);
	if (domConfig == 0)
	{
		kdDebug(DAREA_PARTS_MAPMANAGER) << "no mapper config in mud profile!\n";
		return;
	}
	
	KMemConfig* config = domConfig->config();
	config->setGroup("Directions");
	mapData->directions[NORTH]     = config->readEntry("North",     mapData->directions[NORTH]);
	mapData->directions[SOUTH]     = config->readEntry("South",     mapData->directions[SOUTH]);
	mapData->directions[WEST]      = config->readEntry("West",      mapData->directions[WEST]);
	mapData->directions[EAST]      = config->readEntry("East",      mapData->directions[EAST]);
	mapData->directions[NORTHWEST] = config->readEntry("Northwest", mapData->directions[NORTHWEST]);
	mapData->directions[NORTHEAST] = config->readEntry("Northeast", mapData->directions[NORTHEAST]);
	mapData->directions[SOUTHWEST] = config->readEntry("Southwest", mapData->directions[SOUTHWEST]);
	mapData->directions[SOUTHEAST] = config->readEntry("Southeast", mapData->directions[SOUTHEAST]);
	mapData->directions[UP]        = config->readEntry("Up",        mapData->directions[UP]);
	mapData->directions[DOWN]      = config->readEntry("Down",      mapData->directions[DOWN]);

	mapData->directions[NORTH+(NUM_DIRECTIONS/2)]     = config->readEntry("SmNorth",     mapData->directions[NORTH+(NUM_DIRECTIONS/2)]);
	mapData->directions[SOUTH+(NUM_DIRECTIONS/2)]     = config->readEntry("SmSouth",     mapData->directions[SOUTH+(NUM_DIRECTIONS/2)]);
	mapData->directions[WEST+(NUM_DIRECTIONS/2)]      = config->readEntry("SmWest",      mapData->directions[WEST+(NUM_DIRECTIONS/2)]);
	mapData->directions[EAST+(NUM_DIRECTIONS/2)]      = config->readEntry("SmEast",      mapData->directions[EAST+(NUM_DIRECTIONS/2)]);
	mapData->directions[NORTHWEST+(NUM_DIRECTIONS/2)] = config->readEntry("SmNorthwest", mapData->directions[NORTHWEST+(NUM_DIRECTIONS/2)]);
	mapData->directions[NORTHEAST+(NUM_DIRECTIONS/2)] = config->readEntry("SmNortheast", mapData->directions[NORTHEAST+(NUM_DIRECTIONS/2)]);
	mapData->directions[SOUTHWEST+(NUM_DIRECTIONS/2)] = config->readEntry("SmSouthwest", mapData->directions[SOUTHWEST+(NUM_DIRECTIONS/2)]);
	mapData->directions[SOUTHEAST+(NUM_DIRECTIONS/2)] = config->readEntry("SmSoutheast", mapData->directions[SOUTHEAST+(NUM_DIRECTIONS/2)]);
	mapData->directions[UP+(NUM_DIRECTIONS/2)]        = config->readEntry("SmUp",        mapData->directions[UP+(NUM_DIRECTIONS/2)]);
	mapData->directions[DOWN+(NUM_DIRECTIONS/2)]      = config->readEntry("SmDown",      mapData->directions[DOWN+(NUM_DIRECTIONS/2)]);

	delete domConfig;
}

/** Used to return the character profile
  * @return A pointer to the character profile */
DomConfig *CMapManager::getCharProfile(void)
{
	QDomElement profileElement = pAPI->profileAPI()->openProfile(charProfile);
	if (profileElement.isNull())
	{
		kdDebug(DAREA_PARTS_MAPMANAGER) << "char profile element is null!\n";
		return NULL;
	}

	// find mapper config
	DomConfig* domConfig = DomConfig::findGroup("/mapper", profileElement);
	if (domConfig == 0)
	{
		kdDebug(DAREA_PARTS_MAPMANAGER) << "no mapper config in char profile!\n";
		return NULL;
	}

	return domConfig;
}

/** Used to return the mud profile
  * @return A pointer to the mud profile */
DomConfig *CMapManager::getMudProfile(void)
{
	QDomElement profileElement = pAPI->profileAPI()->openProfile(mudProfile);
	if (profileElement.isNull())
	{
		kdDebug(DAREA_PARTS_MAPMANAGER) << "mud profile element is null!\n";
		return NULL;
	}	

	// find mapper config
	DomConfig* domConfig = DomConfig::findGroup("/mapper",profileElement);
	if (domConfig == 0)
	{
		kdDebug(DAREA_PARTS_MAPMANAGER) << "no mapper config in mud profile!\n";
		return NULL;
	}

	return domConfig;
}


void CMapManager::loadCharProfile(QDomElement profileElement)
{
	kdDebug(DAREA_PARTS_MAPMANAGER) << "loadCharProfile\n";
	if (profileElement.isNull())
	{
		kdDebug(DAREA_PARTS_MAPMANAGER) << "char profile element is null!\n";
		return;
	}
	
	// find mapper config
	DomConfig* domConfig = DomConfig::findGroup("/mapper", profileElement);
	if (domConfig == 0)
	{
		kdDebug(DAREA_PARTS_MAPMANAGER) << "no mapper config in char profile!\n";
		return;
	}
	
	// put loading code here
	
	delete domConfig;
}

bool CMapManager::saveMudProfile(void)
{
	kdDebug(DAREA_PARTS_MAPMANAGER) << "saveMudProfile\n";
	
	if (mudProfile.isEmpty())
	{
		kdDebug(DAREA_PARTS_MAPMANAGER) << "no mud profile set!\n";
		return false;
	}
	
	QDomElement profileElement = pAPI->profileAPI()->openProfile(mudProfile);
	if (profileElement.isNull())
	{
		kdDebug(DAREA_PARTS_MAPMANAGER) << "mud profile element is null!\n";
		return false;
	}
	
	// find or create mapper config
	DomConfig* domConfig = DomConfig::findGroup("/mapper", profileElement, true);
	if (domConfig == 0)
	{
		kdDebug(DAREA_PARTS_MAPMANAGER) << "no mapper config in mud profile!\n";
		return false;
	}
	
	KMemConfig* config = domConfig->config();
	config->setGroup("Directions");
	config->writeEntry("North",     mapData->directions[NORTH]);
	config->writeEntry("South",     mapData->directions[SOUTH]);
	config->writeEntry("West",      mapData->directions[WEST]);
	config->writeEntry("East",      mapData->directions[EAST]);
	config->writeEntry("Northwest", mapData->directions[NORTHWEST]);
	config->writeEntry("Northeast", mapData->directions[NORTHEAST]);
	config->writeEntry("Southwest", mapData->directions[SOUTHWEST]);
	config->writeEntry("Southeast", mapData->directions[SOUTHEAST]);
	config->writeEntry("Up",        mapData->directions[UP]);
	config->writeEntry("Down",      mapData->directions[DOWN]);

	config->writeEntry("SmNorth",     mapData->directions[NORTH+(NUM_DIRECTIONS/2)]);
	config->writeEntry("SmSouth",     mapData->directions[SOUTH+(NUM_DIRECTIONS/2)]);
	config->writeEntry("SmWest",      mapData->directions[WEST+(NUM_DIRECTIONS/2)]);
	config->writeEntry("SmEast",      mapData->directions[EAST+(NUM_DIRECTIONS/2)]);
	config->writeEntry("SmNorthwest", mapData->directions[NORTHWEST+(NUM_DIRECTIONS/2)]);
	config->writeEntry("SmNortheast", mapData->directions[NORTHEAST+(NUM_DIRECTIONS/2)]);
	config->writeEntry("SmSouthwest", mapData->directions[SOUTHWEST+(NUM_DIRECTIONS/2)]);
	config->writeEntry("SmSoutheast", mapData->directions[SOUTHEAST+(NUM_DIRECTIONS/2)]);
	config->writeEntry("SmUp",        mapData->directions[UP+(NUM_DIRECTIONS/2)]);
	config->writeEntry("SmDown",      mapData->directions[DOWN+(NUM_DIRECTIONS/2)]);
	
	domConfig->save();
	delete domConfig;
	
	return pAPI->profileAPI()->saveProfile(mudProfile);
}

/** Used to room under the point the current room */
void CMapManager::slotRoomSetCurrentPos(void)
{
	setCurrentRoom((CMapRoom *)m_selectedElement);
}

/** Used to room under the point the login room */
void CMapManager::slotRoomSetLogin(void)
{
	setLoginRoom((CMapRoom *)m_selectedElement);
}

/** Used to set speedwalk to the room under the pointer */
void CMapManager::slotRoomSpeedwalkTo(void)
{
	walkPlayerTo((CMapRoom *)m_selectedElement);
}

/** Used to delete the room under the pointer */
void CMapManager::slotRoomDelete(void)
{
	deleteElement(m_selectedElement);
}

/** Used to display the properties of the room under the pointer */
void CMapManager::slotRoomProperties(void)
{
	propertiesRoom((CMapRoom *)m_selectedElement);
}

/** Used to make the path under the pointer one way */
void CMapManager::slotPathOneWay(void)
{
	makePathOneWay((CMapPath *)m_selectedElement);
}

/** Used to make the the path under the pointer two way */
void CMapManager::slotPathTwoWay(void)
{
	makePathTwoWay((CMapPath *)m_selectedElement);
}

/** Used to add a bend to the path under the pointer */
void CMapManager::slotPathAddBend(void)
{
	kdDebug(DAREA_PARTS_MAPMANAGER) << "CMapManager::CMapManager slotPathAddBend" << endl;
	openCommandGroup(i18n("Add Bend"));
	CMapPath *path = (CMapPath *)m_selectedElement;

	path->addBendWithUndo(m_selectedPos);
	if (path->getOpsitePath())
	{
		path->getOpsitePath()->addBendWithUndo(m_selectedPos);
	}
	m_clipboard->slotUnselectAll();
	path->setEditMode(true);
	changedElement(path);

	closeCommandGroup();
}

/** Used to delete the path segmeant under the pointer */
void CMapManager::slotPathDelBend(void)
{
	openCommandGroup(i18n("Delete Path Segmeant"));
	CMapPath *path = (CMapPath *)m_selectedElement;

	int seg = path->mouseInPathSeg(m_selectedPos,getActiveView()->getCurrentlyViewedZone());

	path->deletePathSegWithUndo(seg);
	if (path->getOpsitePath())
	{
		int seg = path->getOpsitePath()->mouseInPathSeg(m_selectedPos,getActiveView()->getCurrentlyViewedZone());
		path->getOpsitePath()->deletePathSegWithUndo(seg);
	}

	changedElement(path);	
	closeCommandGroup();
}

/** Used to edit the bends of the path under the pointer */
void CMapManager::slotPathEditBends(void)
{
	CMapPath *path = (CMapPath *)m_selectedElement;

	m_clipboard->slotUnselectAll();
	path->setEditMode(true);
	changedElement(path);
}

/** Used to delete the path under the pointer */
void CMapManager::slotPathDelete(void)
{
	deleteElement(m_selectedElement);
}

/** Used to display the properties of the path under the pointer */
void CMapManager::slotPathProperties(void)
{
	propertiesPath((CMapPath *)m_selectedElement);
}

/** Used to delete the text element under the pointer */
void CMapManager::slotTextDelete(void)
{
	deleteElement(m_selectedElement);
}

/** Used to display the text properties of the text element under the pointer */
void CMapManager::slotTextProperties(void)
{
	propertiesText((CMapText *)m_selectedElement);
}

/** Used to open the zone under the mouse pointer */
void CMapManager::slotZoneOpen(void)
{
	CMapZone *zone = (CMapZone *)m_selectedElement;
	getActiveView()->showPosition(QPoint(1,1),zone->getLevels()->first(),true);	
}

/** Used to open the zone under the mouse pointer */
void CMapManager::slotZoneOpenNewView(void)
{
	CMapZone *zone = (CMapZone *)m_selectedElement;
	CMapLevel *level = zone->getLevels()->first();

	openNewMapView (level);
}


/** Used to delete the zone under the mouse pointer */
void CMapManager::slotZoneDelete(void)
{
	deleteElement(m_selectedElement);
}

/** Used to display the properties of the zone under the pointer */
void CMapManager::slotZoneProperties(void)
{
	propertiesZone((CMapZone *)m_selectedElement);
}

/** Used to change the position of room/zone labels */
void CMapManager::slotChangeLabelPos()
{
	if (m_selectedElement->getElementType()==ROOM)
	{
		CMapRoom *room = (CMapRoom *)m_selectedElement;

		CMapCmdElementProperties *command = new CMapCmdElementProperties(this,i18n("Change room label position"),room);

		command->getOrgProperties()->writeEntry("LabelPos",(int)room->getLabelPosition());

		switch(labelMenu->currentItem())
		{
			case 0 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapRoom::HIDE); break;
			case 1 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapRoom::NORTH); break;
			case 2 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapRoom::NORTHEAST); break;
			case 3 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapRoom::EAST); break;
			case 4 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapRoom::SOUTHEAST); break;
			case 5 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapRoom::SOUTH); break;
			case 6 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapRoom::SOUTHWEST); break;
			case 7 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapRoom::WEST); break;
			case 8 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapRoom::NORTHWEST); break;
			case 9 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapRoom::CUSTOM); break;
			default : command->getNewProperties()->writeEntry("LabelPos",(int)CMapRoom::HIDE); break;
		}

		addCommand(command);
	}

	if (m_selectedElement->getElementType()==ZONE)
	{
		CMapZone *zone = (CMapZone *)m_selectedElement;

		CMapCmdElementProperties *command = new CMapCmdElementProperties(this,i18n("Change zone label position"),zone);

		command->getOrgProperties()->writeEntry("LabelPos",(int)zone->getLabelPosition());

		switch(labelMenu->currentItem())
		{
			case 0 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapZone::HIDE); break;
			case 1 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapZone::NORTH); break;
			case 2 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapZone::NORTHEAST); break;
			case 3 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapZone::EAST); break;
			case 4 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapZone::SOUTHEAST); break;
			case 5 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapZone::SOUTH); break;
			case 6 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapZone::SOUTHWEST); break;
			case 7 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapZone::WEST); break;
			case 8 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapZone::NORTHWEST); break;
			case 9 : command->getNewProperties()->writeEntry("LabelPos",(int)CMapZone::CUSTOM); break;
			default : command->getNewProperties()->writeEntry("LabelPos",(int)CMapZone::HIDE); break;
		}

		addCommand(command);
	}
}

bool CMapManager::saveCharProfile(void)
{
	kdDebug(DAREA_PARTS_MAPMANAGER) << "saveCharProfile\n";
	
	if (charProfile.isEmpty())
	{
		kdDebug(DAREA_PARTS_MAPMANAGER) << "no char profile set!\n";
		return false;
	}
	
	QDomElement profileElement = pAPI->profileAPI()->openProfile(charProfile);
	if (profileElement.isNull())
	{
		kdDebug(DAREA_PARTS_MAPMANAGER) << "char profile element is null!\n";
		return false;
	}
	
	// find or create mapper config
	DomConfig* domConfig = DomConfig::findGroup("/mapper", profileElement, true);
	if (domConfig == 0)
	{
		kdDebug(DAREA_PARTS_MAPMANAGER) << "no mapper config in char profile!\n";
		return false;
	}

	// put saving code here
	
	domConfig->save();
	delete domConfig;
	
	return pAPI->profileAPI()->saveProfile(charProfile);
}

/** This is a debug function and not for genreal use */
int CMapManager::getViewNumber(CMapViewBase *view)
{
	return mapViewList.find(view);
}

/** This is a debug function and not for genreal use */
void CMapManager::changeProperties(CMapElement *element,QString key,QString oldData,QString newData)
{
	CMapCmdElementProperties *cmd = new CMapCmdElementProperties(this,i18n("Change Element Property"),element);
	cmd->getOrgProperties()->writeEntry(key,oldData);
	cmd->getNewProperties()->writeEntry(key,newData);
	addCommand(cmd);
}

/** This is a debug function and not for genreal use */
void CMapManager::changeProperties(CMapElement *element,QString key,int oldData,int newData)
{
	CMapCmdElementProperties *cmd = new CMapCmdElementProperties(this,i18n("Change Element Property"),element);
	cmd->getOrgProperties()->writeEntry(key,oldData);
	cmd->getNewProperties()->writeEntry(key,newData);
	addCommand(cmd);
}

/** This is a debug function and not for genreal use */
void CMapManager::generateTestMap()
{
	kdDebug(DAREA_PARTS_MAPMANAGER) << "creating test map" << endl;
	bool smallMap = false;

	/////////////////////////////////////////////////////////////////////
	// Create a new empty map
	setUndoActive(false);
	commandHistory->clear();
	historyGroup = NULL;
	eraseMap();
	createNewMap();
	setUndoActive(true);

	/////////////////////////////////////////////////////////////////////
	// Create a test map
	// Test rooms and paths

	openCommandGroup("Create Test Map");

	if (!smallMap)
	{
		movePlayerBy(SOUTH,true,"");
		movePlayerBy(SOUTH,true,"");
		movePlayerBy(SOUTH,true,"");
		movePlayerBy(WEST,true,"");
		movePlayerBy(NORTH,true,"");
		movePlayerBy(EAST,true,"");
		movePlayerBy(EAST,true,"");
		movePlayerBy(SOUTH,true,"");
		movePlayerBy(SOUTH,true,"");
		movePlayerBy(SOUTH,true,"");
		movePlayerBy(EAST,true,"");
		movePlayerBy(EAST,true,"");
		movePlayerBy(EAST,true,"");
		movePlayerBy(EAST,true,"");

		movePlayerBy(UP,true,"");
		movePlayerBy(EAST,true,"");
		movePlayerBy(EAST,true,"");
		movePlayerBy(EAST,true,"");
		movePlayerBy(EAST,true,"");
		movePlayerBy(SOUTH,true,"");
		movePlayerBy(SOUTH,true,"");
		movePlayerBy(SOUTH,true,"");
		movePlayerBy(SOUTH,true,"");
		movePlayerBy(SOUTH,true,"");
		movePlayerBy(SOUTH,true,"");
		movePlayerBy(EAST,true,"");
		movePlayerBy(EAST,true,"");
		movePlayerBy(DOWN,true,"");

		movePlayerBy(DOWN,true,"");
		movePlayerBy(SOUTH,true,"");
		movePlayerBy(SOUTH,true,"");
		movePlayerBy(SOUTH,true,"");
		movePlayerBy(SOUTH,true,"");
		movePlayerBy(SOUTH,true,"");
		movePlayerBy(EAST,true,"");
		movePlayerBy(EAST,true,"");
		movePlayerBy(UP,true,"");

		movePlayerBy(SOUTH,true,"");
		movePlayerBy(WEST,true,"");

		// Test text
		changeProperties(getMapData()->rootZone,"Label","",i18n("Root Zone"));

	}

	CMapLevel *level = getMapData()->getFirstZone()->getLevels()->first();
	if (level->getNextLevel())
	{
		level = level->getNextLevel();
	}

	QFont font = QFont("times");
	font.setPointSize(25);
	createText(QPoint(160,20),level,"Test Map",font,Qt::black);


	if (!smallMap)
	{
		CMapRoom *test1=createRoom(QPoint(14*20,14*20),level);
		changeProperties(test1,"Label","",i18n("Test room 1"));
		changeProperties(test1,"LabelPos",(int)CMapRoom::HIDE,(int)CMapRoom::WEST);
		CMapRoom *test2=createRoom(QPoint(20*20,20*20),level);
		changeProperties(test2,"Label","",i18n("Test room 2"));
		changeProperties(test2,"LabelPos",(int)CMapRoom::HIDE,(int)CMapRoom::EAST);
		CMapRoom *test3=createRoom(QPoint(14*20,20*20),level);
		changeProperties(test3,"Label","",i18n("Test room 3"));
		changeProperties(test3,"LabelPos",(int)CMapRoom::HIDE,(int)CMapRoom::WEST);
		CMapRoom *test4=createRoom(QPoint(20*20,14*20),level);
		changeProperties(test4,"Label","",i18n("Test room 4"));
		changeProperties(test4,"LabelPos",(int)CMapRoom::HIDE,(int)CMapRoom::NORTH);

		createPath(QPoint(14*20,14*20),level,SOUTHEAST,
	         	   QPoint(20*20,20*20),level,NORTHWEST);
		createPath(QPoint(20*20,20*20),level,NORTHWEST,
			   QPoint(14*20,14*20),level,SOUTHEAST);
		createPath(QPoint(14*20,20*20),level,NORTHEAST,
		                       QPoint(20*20,14*20),level,SOUTHWEST);

		createPath(QPoint(20*20,14*20),level,SOUTHWEST,
		                       QPoint(14*20,20*20),level,NORTHEAST);

		CMapPath *specialPath = createPath(test1,SPECIAL,test4,SPECIAL);
		makePathTwoWay(specialPath);

		CMapCmdElementProperties *cmd = new CMapCmdElementProperties(this,i18n("Set Special Exit"),specialPath);
		cmd->getOrgProperties()->writeEntry("SpecialCmdSrc","");
		cmd->getOrgProperties()->writeEntry("SpecialCmdDest","");
		cmd->getOrgProperties()->writeEntry("SpecialExit",false);
		cmd->getNewProperties()->writeEntry("SpecialCmdSrc","enter");
		cmd->getNewProperties()->writeEntry("SpecialCmdDest","out");
		cmd->getNewProperties()->writeEntry("SpecialExit",true);
		addCommand(cmd);

		CMapZone *testZone = createZone(QPoint(24*20,14*20),level);
		changeProperties(testZone,"Label","",i18n("Test Zone 2"));
		changeProperties(testZone,"LabelPos",CMapRoom::HIDE,CMapRoom::EAST);

		CMapRoom *test5 = createRoom(QPoint(3*20,3*20),testZone->getLevels()->first());
		changeProperties(test5,"Label","",i18n("Test room 5"));

		CMapPath *interZonePath = createPath(test4,NORTH,test5,NORTH);
		makePathTwoWay(interZonePath);

		makePathTwoWay(createPath(test4,SOUTH,test5,SOUTH));
		makePathTwoWay(createPath(test4,EAST,test5,WEST));


		changedElement(testZone);

		createZone(QPoint(6*20,6*20),testZone->getLevels()->first());
		font.setPointSize(8);
		createText(QPoint(20,300),level,"Multi line text lable\nThis is the second line\nThis is the third line",font,Qt::black);

	}
	closeCommandGroup();
	kdDebug(DAREA_PARTS_MAPMANAGER) << "test map created" << endl;
}
