/***************************************************************************
                               dlgmapmovement.cpp
                             -------------------
    begin                : Tue Mar 18 2003
    copyright            : (C) 2003 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dlgmapmovement.h"

#include "dlgmapmovementbase.h"
#include "../cmapmanager.h"

#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qlistbox.h>
#include <qinputdialog.h>

#include <klocale.h>
#include <kregexpeditorinterface.h>
#include <kparts/componentfactory.h>

DlgMapMovement::DlgMapMovement(CMapManager *mapManager,QString group,QWidget *parent, const char *name) :  KmudSetupPane(KmudSetupPane::GLOBAL,group,parent,name)
{
	m_mapManager = mapManager;

	addBaseDialog(new DlgMapMovementBase(this,name));

	DlgMapMovementBase *baseDlg = (DlgMapMovementBase *)getBaseDialog();
	
	baseDlg->m_chkEnableValidRoomChecking->setChecked(m_mapManager->getMapData()->validRoomCheck);

	slotValidCheckStateChanged(m_mapManager->getMapData()->validRoomCheck);

	for (char *s1 = m_mapManager->getMapData()->failedMoveMsg.first();
			     s1!=0; s1 = m_mapManager->getMapData()->failedMoveMsg.next())
	{
		QString str = s1;
		if (str!="")
		{
			new QListBoxText(baseDlg->m_lstInvalidMoveStrs,str);
		}
	}

	connect(baseDlg->m_chkEnableValidRoomChecking,SIGNAL(toggled(bool)),this,SLOT(slotValidCheckStateChanged(bool)));
	connect(baseDlg->m_cmdAdd,SIGNAL(clicked()),this,SLOT(slotAddClicked()));
	connect(baseDlg->m_cmdEdit,SIGNAL(clicked()),this,SLOT(slotEditClicked()));
	connect(baseDlg->m_cmdRemove,SIGNAL(clicked()),this,SLOT(slotRemoveClicked()));
}

DlgMapMovement::~DlgMapMovement()
{
}

/** This slot is called when the delfault/reset button is pressed */
void DlgMapMovement::slotDefaultPressed()
{
	DlgMapMovementBase *baseDlg = (DlgMapMovementBase *)getBaseDialog();

	baseDlg->m_lstInvalidMoveStrs->clear();
	baseDlg->m_chkEnableValidRoomChecking->setChecked(false);		
}

/** This slot is called when the cancel button is pressed */
void DlgMapMovement::slotCancelPressed()
{
}

/** This slot is called when the OK button is pressed */
void DlgMapMovement::slotOkPressed()
{
	DlgMapMovementBase *baseDlg = (DlgMapMovementBase *)getBaseDialog();

	m_mapManager->getMapData()->validRoomCheck =  baseDlg->m_chkEnableValidRoomChecking->isChecked();

	m_mapManager->getMapData()->failedMoveMsg.clear();

	for (int i=0; i< baseDlg->m_lstInvalidMoveStrs->count();i++)
	{
		m_mapManager->getMapData()->failedMoveMsg.append(baseDlg->m_lstInvalidMoveStrs->text(i));
	}
}

void DlgMapMovement::slotValidCheckStateChanged(bool state)
{
	DlgMapMovementBase *baseDlg = (DlgMapMovementBase *)getBaseDialog();
	
	baseDlg->m_cmdAdd->setEnabled(state);
	baseDlg->m_cmdEdit->setEnabled(state);
	baseDlg->m_cmdRemove->setEnabled(state);
	baseDlg->m_lstInvalidMoveStrs->setEnabled(state);
}

void DlgMapMovement::slotAddClicked(void)
{
	
	DlgMapMovementBase *baseDlg = (DlgMapMovementBase *)getBaseDialog();
	
	QString text = "";
	bool ok = false;	

	QDialog *editorDialog = KParts::ComponentFactory::createInstanceFromQuery<QDialog>( "KRegExpEditor/KRegExpEditor" );
	if ( editorDialog )
	{
		// kdeutils was installed, so the dialog was found fetch the editor interface
		KRegExpEditorInterface *editor = static_cast<KRegExpEditorInterface *>( editorDialog->qt_cast( "KRegExpEditorInterface" ) );
		Q_ASSERT( editor ); // This should not fail!

		// now use the editor.
		editor->setRegExp("");

		// Finally exec the dialog
		if (editorDialog->exec())
		{
			ok = true;
			text = editor->regExp();
		}
	}
	else
	{
		text = QInputDialog::getText( i18n("Kmud"), i18n("Enter invalid movement string as a regular expression"), QLineEdit::Normal,
	                                      QString::null, &ok, this );
	}

    if ( ok && !text.isEmpty() )
    {
		new QListBoxText(baseDlg->m_lstInvalidMoveStrs,text);
	}
}

void DlgMapMovement::slotEditClicked(void)
{
	DlgMapMovementBase *baseDlg = (DlgMapMovementBase *)getBaseDialog();
	
	int current =  baseDlg->m_lstInvalidMoveStrs->currentItem();
	if (current!=-1)
	{
		bool ok;
		QString text = "";

		QDialog *editorDialog = KParts::ComponentFactory::createInstanceFromQuery<QDialog>( "KRegExpEditor/KRegExpEditor" );
		if ( editorDialog )
		{
			// kdeutils was installed, so the dialog was found fetch the editor interface
			KRegExpEditorInterface *editor = static_cast<KRegExpEditorInterface *>( editorDialog->qt_cast( "KRegExpEditorInterface" ) );
			Q_ASSERT( editor ); // This should not fail!

			// now use the editor.
			editor->setRegExp("");

			// Finally exec the dialog
			if (editorDialog->exec())
			{
				ok = true;
				text = editor->regExp();
			}
		}
		else
		{

			text = QInputDialog::getText( i18n("Kmud"), i18n("Enter invalid movement string as a regular expression"), QLineEdit::Normal,
		                                     baseDlg->m_lstInvalidMoveStrs->text(current), &ok, this );
		}

		if ( ok && !text.isEmpty() )
		{
			baseDlg->m_lstInvalidMoveStrs->changeItem(text,current);
		}
	}
}

void DlgMapMovement::slotRemoveClicked(void)
{
	DlgMapMovementBase *baseDlg = (DlgMapMovementBase *)getBaseDialog();
	
	int current =  baseDlg->m_lstInvalidMoveStrs->currentItem();
	if (current!=-1)
	{
		baseDlg->m_lstInvalidMoveStrs->removeItem(current);
	}
}

