/***************************************************************************
                               cmaptoolroom.cpp
                             -------------------
    begin                : Wed Aug 8 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cmaptoolroom.h"

#include <qcolor.h>

#include <klocale.h>
#include <kstddirs.h>
#include <kiconloader.h>

#include <qcursor.h>

#include "../../../cmapmanager.h"
#include "../../../cmapviewbase.h"

#include <kmuddebug.h>

CMapToolRoom::CMapToolRoom(KActionCollection *actionCollection,CMapManager *manager,QObject *parent, const char *name)
	: CMapToolBase(actionCollection,
                   i18n("Create Room"),
                   BarIcon(locate("data", "kmudmapper/toolbar/tools/kmud_room.png")),
                   manager,"toolsRoom",parent,name)

{
}

CMapToolRoom::~CMapToolRoom()
{
}

/** Called when the tool recives a mouse release event */
void CMapToolRoom::mouseReleaseEvent(QPoint mousePos,CMapLevel *currentLevel,QPainter *)
{
	//FIXME_jp : Check to see if the room was placed at the edge of the map and move map if nescerary

	//Create the room
	mapManager->createRoom(mapManager->cordsSnapGrid(mousePos),currentLevel);

	lastPos=QPoint(-100,-100);
}

/** Called when the tool recives a mouse move event */
void CMapToolRoom::mouseMoveEvent(QPoint mousePos,Qt::ButtonState,CMapLevel *,QPainter *p)
{
	int width = mapManager->getMapData()->gridSize.width();
	int height = mapManager->getMapData()->gridSize.height();

	QPoint pos = mapManager->cordsSnapGrid(mousePos);

	p->setPen(Qt::black);
	p->setBrush(QColor(Qt::black));

	p->setRasterOp(Qt::NotROP);

	if (pos.x()>=0 && pos.y()>=0)
	{
		// Erase old one
		p->drawRect(lastPos.x()+1,lastPos.y()+1,width-2,height-2);

		// Draw new one
		p->drawRect(pos.x()+1,pos.y()+1,width-2,height-2);
   	}

	lastPos=pos;
}

void CMapToolRoom::mouseEnterEvent(QPainter *)
{

	kdDebug(DAREA_PARTS_MAPPERPLUGIN) << "CMapToolRoom: mouseEnterEvent" << endl;
}

void CMapToolRoom::mouseLeaveEvent(QPainter *p)
{
	kdDebug(DAREA_PARTS_MAPPERPLUGIN) << "CMapToolRoom: mouseLeaveEvent" << endl;
}

/** This function called when a tool is selected */
void CMapToolRoom::toolSelected(void)
{
	CMapViewBase *view = mapManager->getActiveView();
	if (view)
	{
		view->setMouseTracking(true);
		view->setCursor(ArrowCursor);	
	}
	lastPos=QPoint(-100,-100);
}

/** This function is called when a tool is unselected */
void CMapToolRoom::toolUnselected(void)
{
	if (mapManager->getActiveView())
		mapManager->getActiveView()->redraw();
}

/** This method is called when the active view changes */
void CMapToolRoom::viewChangedEvent(CMapViewBase *view)
{
	if (view)
	{
		view->setMouseTracking(true);
		view->setCursor(ArrowCursor);	
	}
}
